'                       Extinction Module - J H Differential Photometry
'                                                Optec, Inc.
'
'======version history
'
'V1.20, December 2015
'   modified to accept PPparms2_JH for AllSky
'
'V1.10
'  compiled with LB4.5
'  increased dim
'
'V1.00
'
'
'=====dimension statements and initial conditions
'
    DIM info$(10,10)
    files "c:\", info$()

    DIM ExtData$(4000)                              'extinction data lines for list box table
        ExtData$(1) = "open raw data file and select comp star"
    DIM ExtData(4000,12)                            'extinction data items for list box table
    DIM CompStar$(400)                              'list of comparison C stars
        CompStar$(1) = "select"
    DIM RawData$(4000)                              'data from raw file of observations
    DIM SDitem$(4000,10)                            'individual data items from Star Data file
    DIM RDitem$(1000,20)                             'individual data items from raw file of observations
    DIM CountFinal(4000)                            'average count including integration and scale factors
    DIM JD(4000)                                    'Julean date from 2000 for each RawIndex
    DIM JT(4000)                                    'Julean century fromm 2000 for each RawIndex
    DIM X(400)                                      'air mass array for regression anaylsis
    DIM m(400)                                      'instrument magnitude array for regression anaylsis
'
'=====open and read in PPparms data for configuration
'
open "PPparms2_JH.txt" for input as #PPparms
    input #PPparms, Location$                       'latitude and longitude in degrees
    input #PPparms, KJ                              'first order extinction for J
    input #PPparms, KH                              'first order extinction for H
    input #PPparms, KKjh                            'second order extinction for j-h, default = 0
    input #PPparms, Rho                             'transformation coeff. for J using J-H
    input #PPparms, Lambda                          'transformation coeff. J-H
    input #PPparms, JDFlag                          '1 if using JD and 0 if using HJD
    input #PPparms, OBSCODE$                        'AAVSO observatory code
    input #PPparms, Spare2                          'not used
    input #PPparms, ZPj                             'zero-point constant for j
    input #PPparms, ZPjh                            'zero-point constant for j-h
    input #PPparms, Ej                              'standard error for j
    input #PPparms, Ejh                             'standard error for j-h
close #PPparms
gosub [Find_Lat_Long]

            open "IREX.txt" for output as #IREX
                print #IREX, "Extinction JH module"
                print #IREX, " "
                print #IREX, "PPparms2_JH.txt"
                print #IREX, "  Location$  ";Location$
                print #IREX, "  KJ         ";KJ
                print #IREX, "  KH         ";KH
                print #IREX, "  KKjh       ";KKjh
                print #IREX, "  Rho        ";Rho
                print #IREX, "  Lambda     ";Lambda
                print #IREX, "  JDFlag     "; JDFlag
                print #IREX, "  OBSCODE    "; OBSCODE$
                print #IREX, "  Spare2     "; Spare2
                print #IREX, "  ZPj        "; ZPj
                print #IREX, "  ZPjh       "; ZPjh
                print #IREX, "  Ej         "; Ej
                print #IREX, "  Ejh        "; Ejh
                print #IREX, " "
            close #IREX
'
'=====initialize and start up values
'
VersionNumber$ = "1.20"
PathDataFile$ = "*.raw"                             'default path for data files
PathPlotFile$ = "* .bmp"                            'default path for plot files
'
'
'======open and read in Star Data
'
open "Star Database JH.txt" for input as #StarData
    DataIndex = 0
    while eof(#StarData) = 0
        DataIndex = DataIndex + 1
        input #StarData,SDitem$(DataIndex,1),_      'comp or var star name
                        SDitem$(DataIndex,2),_      'type, either C or V
                        SDitem$(DataIndex,3),_      'RA hour
                        SDitem$(DataIndex,4),_      'RA minute
                        SDitem$(DataIndex,5),_      'RA second
                        SDitem$(DataIndex,6),_      'DEC degree
                        SDitem$(DataIndex,7),_      'DEC minute
                        SDitem$(DataIndex,8),_      'DEC second
                        SDitem$(DataIndex,9),_      'J magnitude, ##.##
                        SDitem$(DataIndex,10)       'J-H index, ##.##
    wend
    DataIndexMax = DataIndex
close #StarData
'
'=====set up main GUI control window
'
[WindowSetup]
    NOMAINWIN
    WindowWidth = 900 : WindowHeight = 457
    UpperLeftX = INT((DisplayWidth-WindowWidth)/2)
    UpperLeftY = INT((DisplayHeight-WindowHeight)/2)

[ControlSetup]
    Menu        #extinction, "File",_
                             "Open File", [Open_File],_
                             "Save Plot Graphic", [Data_Save],_
                             "Quit", [Quit_Extinction]
    Menu        #extinction, "Coefficients",_
                             "Load Saved Extinction Coefficients", [Load_Previous_Coeff],_
                             "Send Computed Coefficients to Results", [Use_Current_Coefficients],_
                             "Save Coefficients in Results to PPparms", [Save_Coefficients],_
                             "Enter Second Order Extinction Coefficients",[Enter_SOE_Coefficients],_
                             "Clear Results", [Clear_Results]
    Menu        #extinction, "Setup", "Location", [Location]
    Menu        #extinction, "Help", "About", [About], "Help", [Help]

    graphicbox  #extinction.graphicbox1,   525, 40, 350, 265

    groupbox    #extinction.groupbox1, "Raw Data File", 11, 311, 300, 90
    groupbox    #extinction.groupbox2, "Results", 325, 310, 185, 90
    groupbox    #extinction.groupbox3, "Least-Squares Analysis", 525, 310, 350, 90

    statictext  #extinction.statictext2, "HA", 35, 20, 20, 14
    statictext  #extinction.statictext2, "X", 105, 20, 10, 14
    statictext  #extinction.statictext3, "---Counts---", 167, 5, 130, 14
    statictext  #extinction.statictext4, "j", 180, 20, 10, 14
    statictext  #extinction.statictext5, "h ", 250, 20, 10, 14

    statictext  #extinction.statictext10, "Instrument Magnitude", 300, 5, 200, 14
    statictext  #extinction.statictext11, "j", 340, 20, 10, 14
    statictext  #extinction.statictext12, "h", 410, 20, 10, 14

    statictext  #extinction.statictext20, "Comp Star", 20, 365, 90, 14

    statictext  #extinction.statictext33, "K'", 372, 335, 15, 14
    statictext  #extinction.statictext34, "j", 387, 340, 10, 14
    statictext  #extinction.statictext35, "K'", 442, 335, 15, 14
    statictext  #extinction.statictext36, "h", 457, 340, 10, 14

    statictext  #extinction.statictext50, "slope", 625, 337, 50, 14
    statictext  #extinction.statictext51, "intercept", 590, 352, 90, 14
    statictext  #extinction.statictext52, "standard error", 545, 367, 140, 14

    button      #extinction.ShowJ, "show j",[Show_J.click],UL, 525, 10, 67, 25
    button      #extinction.ShowH, "show h",[Show_H.click],UL, 595, 10, 67, 25


    button      #extinction.Print, "print",[DataPrint.click], UL, 805, 360, 57, 25

    textbox     #extinction.Kj, 348, 360, 67, 25
    textbox     #extinction.Kh, 419, 360, 67, 25


    textbox     #extinction.Analysis, 685, 335, 105, 50
    textbox     #extinction.FileName, 20, 330, 280, 25
    listbox     #extinction.Table, ExtData$(),[Extinction_Table.click], 10, 40, 500, 265

    combobox    #extinction.Comp,CompStar$(),[Select_Comp.click], 110, 362, 140, 300

    Open "K' Extinction   JH Photometry" for Window as #extinction
    #extinction "trapclose [Quit_Extinction]"
    #extinction.graphicbox1 "down; fill White; flush"
    #extinction.graphicbox1 "setfocus; when mouseMove [MouseChange1]"
    #extinction.Table "selectindex 1"
    #extinction.Comp "selectindex 1"
    #extinction "font courier_new 10 14"

    print #extinction.FileName, "open raw data file"

    print #extinction.ShowJ, "!Disable"             'disable the graph buttons until there is data
    print #extinction.ShowH, "!Disable"

    print #extinction.Print, "!Disable"

[loop]

Wait                                                'finised setting up, wait here for new command
'
'======menu controls
'
[Open_File]
    print #extinction.graphicbox1, "cls"            'clear all tables and graphs for new data set
    redim ExtData$(4000)
    ExtData$(1) = "open raw data file and select comp star"
    #extinction.Table "reload"
    redim CompStar$(400)
    CompStar$(1) = "select"
    #extinction.Comp "selectindex 1"

    filedialog "Open Data File", PathDataFile$, DataFile$

    for I = len(DataFile$) to 1 step -1             'remember path for opened folder and file
        if mid$(DataFile$,I,1) = "\" then
            ShortDataFile$ = mid$(DataFile$,I+1)
            PathDataFile$ = left$(DataFile$,I)+"*raw"
            exit for
        end if
    next I

    if DataFile$ = "" then
        print #extinction.FileName, "open raw data file"
    else
        files "c:\", DataFile$, info$()
        if val(info$(0, 0)) = 0 then
            notice "cannot create new file"
        else
            open DataFile$ for input as #RawFile
            for RawIndex = 1 to 4                   'read first 4 lines with line input to capture commas
                line input #RawFile, RawData$(RawIndex)
            next RawIndex
            RawIndex = 4
            while eof(#RawFile)=0                   'read rest of data to end of file
                RawIndex = RawIndex + 1
                input #RawFile, RawData$(RawIndex)

            wend
            RawIndexMax = RawIndex
            close #RawFile

            open "IREX.txt" for append as #IREX
                print #IREX, "output from [Find_File_Name]"
                gosub [Find_File_Name]
                print #IREX, DataFileName$
                print #IREX, " "
            close #IREX

            open "IREX.txt" for append as #IREX
                print #IREX, "output from [Convert_RawFile]"
                gosub [Convert_RawFile]
                print #IREX, " "
            close #IREX


            gosub [Total_Count_RawFile]
            gosub [Julian_Day_RawFile]
            CompIndexMax = 1                     'reset array index to 1 for Comp Star list

            open "IREX.txt" for append as #IREX
                print #IREX, "output from [IREX_RawFile]"
                print #IREX, "star       Julean date   final cnt  filter SkyPast   Sky Future,"

                Filter$ = "J"
                gosub [IREX_RawFile]
                Filter$ = "H"
                gosub [IREX_RawFile]
                print #IREX, " "
            close #IREX

            print #extinction.Comp, "reload"
        end if
    end if
Wait
'
[Data_Save]                                      'save bitmap image to disk
    filedialog "Save As...", PathPlotFile$, PlotFile$
    for I = len(PlotFile$) to 1 step -1
        if mid$(PlotFile$,I,1) = "\" then
            ShortPlotFile$ = mid$(PlotFile$,I+1)
            PathPlotFile$ = left$(PlotFile$,I)+"*bmp"
            exit for
        end if
    next I

    if PlotFile$ <> "" then
        print #extinction.graphicbox1, "getbmp plot -2 -2 352 267"
        if (right$(PlotFile$,4) = ".bmp") OR (right$(PlotFile$,4) = ".BMP") then
            bmpsave "plot", PlotFile$
        else
            PlotFile$ = PlotFile$+".bmp"
            bmpsave "plot", PlotFile$
        end if
    end if
wait
'
[Load_Previous_Coeff]
    print #extinction.Kj, using("##.###",KJ)
    print #extinction.Kh, using("##.###",KH)
Wait

[Save_Coefficients]
    print #extinction.Kj, "!contents? KJtemp$"
    print #extinction.Kh, "!contents? KHtemp$"

    KJtemp = val(KJtemp$)
    KHtemp = val(KHtemp$)

    if KJtemp = 0 AND KHtemp = 0  then
        notice "nothing to save"
        Wait
    end if
    confirm "This will save all non-zero contents"+chr$(13)_
            +"Type a zero in box to keep previous saved coefficient"+chr$(13)+chr$(13)_
            +"Do you wish to save values?"; Answer$
    if Answer$ = "yes" then

        if KJtemp <> 0 then
            KJ = KJtemp
        end if
        if KHtemp <> 0 then
            KH = KHtemp
        end if
        gosub [Write_PPparms]
    end if
Wait

[Use_Current_Coefficients]
    print #extinction.Kj, using("##.###",KJtemporary)
    print #extinction.Kh, using("##.###",KHtemporary)
Wait

[Enter_SOE_Coefficients]
    confirm "The default value of K'' is 0. Use the SOE K''"+chr$(13)+_
            "Coefficient module to calculate K'' from Blue-Red "+chr$(13)+_
            "star observations and save results to PPparms. Continue"+chr$(13)+_
            "if you wish to enter a value"+chr$(13)+chr$(13)+_
            "Continue anyway?"; answerSOE$
            if answerSOE$ = "no" then [loop]
    prompt "SOE Coefficient"+chr$(13)+_
           "Enter new value, current value = "+using("###.###", KKjh); KKjh$
            if KKjh$ <> "" then
                KKjh = val(KKjh$)
                gosub [Write_PPparms]
            end if
wait
'
[Clear_Results]
    print #extinction.Kj, ""
    print #extinction.Kh, ""
wait
'

[Location]

    NOMAINWIN
    WindowWidth = 227 : WindowHeight = 240
    UpperLeftX = INT((DisplayWidth-WindowWidth)/2)
    UpperLeftY = INT((DisplayHeight-WindowHeight)/2)
    Menu        #location, "&File" , "E&xit", [Quit_location], "Save", [Save_location]

    statictext  #location.text1, "use decimal format", 35,20,150,18

    groupbox    #location.longitude, "Longitude", 10, 50, 200, 60
    radiobutton #location.E, "E", [setE], [resetE], 25,80,30,16
    radiobutton #location.W, "W", [setW], [resetW], 70,80,30,16
    textbox     #location.long, 115, 75, 70, 25

    groupbox    #location.latitude, "Latitude", 10, 120, 200, 60
    radiobutton #location.N, "N", [setN], [resetN], 25,150,30,16
    radiobutton #location.S, "S", [setS], [resetS], 70,150,30,16
    textbox     #location.lat,  115, 145, 70, 25

    Open "Edit Location" for Window as #location
        #location "trapclose [Quit_location]"
        #location "font courier_new 8 16"

    LocationFlag = 1

    LATdec$  = mid$(Location$,2,4)
    LATns$   = left$(Location$,1)
    LONGdec$ = mid$(Location$,8,5)
    LONGew$  = mid$(Location$,7,1)

    SELECT CASE LATns$
        CASE "N"
            print #location.N, "set"
        CASE "S"
            print #location.S, "set"
        CASE ELSE
            notice "error in reading N or S latitude"
            wait
    END SELECT
    SELECT CASE LONGew$
        CASE "E"
            print #location.E, "set"
        CASE "W"
            print #location.W, "set"
        CASE ELSE
            notice "error in reading W or E longitude"
            wait
    END SELECT

    print #location.lat, "  "+LATdec$
    print #location.long, " "+LONGdec$ 

    wait

    [setE]
        LONGew$ = "E"
    wait

    [setW]
        LONGew$ = "W"
    wait

    [setN]
        LATns$ = "N"
    wait

    [setS]
        LATns$ = "S"
    wait

    [resetE]
    [resetW]
    [resetN]
    [resetS]
    wait

    [Save_location]
        print #location.lat, "!contents? LATdec$";
        print #location.long, "!contents? LONGdec$";

        LATdec = val(LATdec$)
        LONGdec = val(LONGdec$)

        if LATdec < 0 or LATdec > 90 or LONGdec < 0 or LONGdec > 180 then
            notice "longitude or latitude out of range"
            wait
        end if
                                            'convert textbox values to correct format
        LATdec$  = right$("0"+str$(int(LATdec)),2)+mid$(str$((LATdec MOD 1)+0.05),2,2)
        LONGdec$ = right$("00"+str$(int(LONGdec)),3)+mid$(str$((LONGdec MOD 1)+0.05),2,2)

        Location$ = LATns$+LATdec$+"_"+LONGew$+LONGdec$

        Gosub [Write_PPparms]

        open "IREX.txt" for append as #IREX
            print #IREX, "output from [Location]"
            print #IREX, " "
            print #IREX, "Location$ = ";Location$
            print #IREX, " "
        close #IREX
    wait

    [Quit_location]
    close #location
    LocationFlag = 0
wait


[About]
    notice "K' Extinction  JH Photometry"+chr$(13)+_
           "Version "+VersionNumber$+chr$(13)+_
           "copyright  2015, Gerald Persha"+chr$(13)+_
           "www.SSPDataq.com"
Wait

[Help]
    run "hh photometry jh.chm"
Wait

[MouseChange1]
    'MouseX and MouseY contain mouse coordinates
wait

[Quit_Extinction]                   'exit program

    confirm "do you wish to exit program?"; Answer$

    if Answer$ = "yes" then
        if LocationFlag = 1 then
            close #location
        end if
        close #extinction
    else
        wait
    end if
END
'
'=====control buttons and control boxes
'
[Show_J.click]
    print #extinction.Print, "!Enable"
    KJtemporary = 0
    Yaxis$ = "\j"
    print #extinction.graphicbox1, "color black"
    gosub [Draw_Graph_Outline]
    gosub [Create_Regression_Array]
    gosub [Solve_Regression_Matrix]
    print #extinction.Analysis, using("####.###", Slope) + chr$(13) + chr$(10) +_
                                using("####.###", Intercept) + chr$(13) + chr$(10) +_
                                using("####.###", std.error)
    PointColor$ = "red"
    gosub [Draw_Best_Line]
    gosub [Draw_Description]
    print #extinction.graphicbox1, "flush"
    KJtemporary = Slope
wait
'
[Show_H.click]
    print #extinction.Print, "!Enable"
    KHtemporary = 0
    Yaxis$ = "\h"
    print #extinction.graphicbox1, "color black"
    gosub [Draw_Graph_Outline]
    gosub [Create_Regression_Array]
    gosub [Solve_Regression_Matrix]
    print #extinction.Analysis, using("####.###", Slope) + chr$(13) + chr$(10) +_
                                using("####.###", Intercept) + chr$(13) + chr$(10) +_
                                using("####.###", std.error)
    PointColor$ = "darkred"
    gosub [Draw_Best_Line]
    gosub [Draw_Description]
    print #extinction.graphicbox1, "flush"
    KHtemporary = Slope
wait
'
[Extinction_Table.click]
    #extinction.Table "selection? selected$"
wait
'
[Select_Comp.click]

    print #extinction.graphicbox1, "cls"            'clear the graphics box

    print #extinction.Comp, "selection? selectedComp$"

    print #extinction.ShowJ, "!Disable"             'disable the graph buttons until there is data
    print #extinction.ShowH, "!Disable"

    print #extinction.Print, "!Disable"

    RA = 0
    DEC = 0
                                    'get RA and DEC info of comp star from Star Data file
    for DataIndex = 1 to DataIndexMax
        if selectedComp$ = left$(SDitem$(DataIndex,1)+"           ",12) then
                                    'convert RA and DEC to decimal
            RA =  val(SDitem$(DataIndex,3)) + val(SDitem$(DataIndex,4))/60 + val(SDitem$(DataIndex,5))/3600
            RA = (RA/24) * 360       'convert RA to degreeS

            DEC = abs(val(right$(SDitem$(DataIndex,6),2))) +_
                  val(SDitem$(DataIndex,7))/60 + val(SDitem$(DataIndex,8))/3600
                                    'see if  minus sign is in the degree string and make DEC negative if so
                                    'this is need in case of DECd = - 0
            if left$(SDitem$(DataIndex,6),1) = "-" then
                DEC = DEC * -1
            end if
           exit for
        end if
    next
    if (RA = 0) AND (DEC = 0) then
        notice "could not find selected Comp Star in Star Data file"
        wait
    end if
    gosub [Create_Extinction_Table]

    for RawIndex = 5 to RawIndexMax                 'turn on graph buttons for those with data
        if RDitem$(RawIndex,9) = "J" then
            print #extinction.ShowJ, "!Enable"
        end if
        if RDitem$(RawIndex,9) = "H" then
            print #extinction.ShowH, "!Enable"
        end if
    next
wait
'
[DataPrint.click]
    printerdialog
    print #extinction.graphicbox1, "Print VGA"
wait
'
'=====subroutines for extracting data from opened raw file
'
[Convert_RawFile]       'extract individual data items from each indexed data line
    For RawIndex = 5 to RawIndexMax                                'skip header and get all raw data
        if len(RawData$(RawIndex)) < 70 then                       'delete any extra junk at end of file
            RawIndexMax = RawIndex - 1
            exit for
        end if
        RDitem$(RawIndex,1)  = mid$(RawData$(RawIndex),1,2)        'UT month
        RDitem$(RawIndex,2)  = mid$(RawData$(RawIndex),4,2)        'UT day
        RDitem$(RawIndex,3)  = mid$(RawData$(RawIndex),7,4)        'UT year
        RDitem$(RawIndex,4)  = mid$(RawData$(RawIndex),12,2)       'UT hour
        RDitem$(RawIndex,5)  = mid$(RawData$(RawIndex),15,2)       'UT minute
        RDitem$(RawIndex,6)  = mid$(RawData$(RawIndex),18,2)       'UT second
        RDitem$(RawIndex,7)  = mid$(RawData$(RawIndex),21,1)       'catalog: C, V
        RDitem$(RawIndex,8)  = mid$(RawData$(RawIndex),26,12)      'star name, SKY, SKYNEXT or SKYLAST
        RDitem$(RawIndex,9)  = mid$(RawData$(RawIndex),41,1)       'filter: J or H
        RDitem$(RawIndex,10) = mid$(RawData$(RawIndex),44,5)       'Count 1
        RDitem$(RawIndex,11) = mid$(RawData$(RawIndex),51,5)       'Count 2
        RDitem$(RawIndex,12) = mid$(RawData$(RawIndex),58,5)       'Count 3
        RDitem$(RawIndex,13) = mid$(RawData$(RawIndex),65,5)       'Count 4
        RDitem$(RawIndex,14) = mid$(RawData$(RawIndex),72,2)       'integration time in seconds: 1 or 10
        RDitem$(RawIndex,15) = mid$(RawData$(RawIndex),75,3)       'scale: 1, 10 or 100

        print #IREX,using("####",RawIndex);"  ";_
            RDitem$(RawIndex,1);"  ";_
            RDitem$(RawIndex,2);"  ";_
            RDitem$(RawIndex,3);"  ";_
            RDitem$(RawIndex,4);"  ";_
            RDitem$(RawIndex,5);"  ";_
            RDitem$(RawIndex,6);"  ";_
            RDitem$(RawIndex,7);"  ";_
            RDitem$(RawIndex,8);"  ";_
            RDitem$(RawIndex,9);"  ";_
            RDitem$(RawIndex,10);"  ";_
            RDitem$(RawIndex,11);"  ";_
            RDitem$(RawIndex,12);"  ";_
            RDitem$(RawIndex,13);"  ";_
            RDitem$(RawIndex,14);"  ";_
            RDitem$(RawIndex,15)
    next
return
'
[Total_Count_RawFile]   'compute the total averaged count and scale for gain and integration settings
    For RawIndex = 5 to RawIndexMax
        CountSum =  val(RDitem$(RawIndex,10)) +_                    'Count 1
                    val(RDitem$(RawIndex,11)) +_                    'Count 2
                    val(RDitem$(RawIndex,12)) +_                    'Count 3
                    val(RDitem$(RawIndex,13))                       'Count 4
        Divider = 4                                                 'find divider to get average of Count
        if val(RDitem$(RawIndex,13)) = 0 then
            Divider = Divider - 1
        end if
        if val(RDitem$(RawIndex,12)) = 0 then
            Divider = Divider - 1
        end if
        if val(RDitem$(RawIndex,11)) = 0 then
            Divider = Divider - 1
        end if
        if val(RDitem$(RawIndex,10)) = 0 then
            ' do something to prevent division by 0
        end if
        Integration = val(RDitem$(RawIndex,14))                     'find integration time :1 or 10 seconds
        Scale = val(RDitem$(RawIndex,15))                           'find scale factor: 1, 10 or 100
        CountFinal(RawIndex) = int((CountSum/Divider) * (100/(Integration * Scale)))
    next
return
'
[Julian_Day_RawFile]    'convert UT time and date to Julian, epcoh J2000
    For RawIndex = 5 to RawIndexMax
                                'A = int(UTyear/100)
        A = int(val(RDitem$(RawIndex,3))/100)
        B = 2 - A + int(A/4)
                                'C = int(365.25 * UTyear)
        C = int(365.25 * val(RDitem$(RawIndex,3)))
                                'D = int(30.6001 *(UTmonth + 1))
        D = int(30.6001 * (val(RDitem$(RawIndex,1)) + 1))
                                'JD = B + C + D - 730550.5 + UTday + (UThours + UTmin/60 + UTsec/3600)/24
        JD(RawIndex) = B + C + D - 730550.5 + val(RDitem$(RawIndex,2)) +_
                       (val(RDitem$(RawIndex,4)) + val(RDitem$(RawIndex,5))/60 +_
                        val(RDitem$(RawIndex,6))/3600)/24
                                'Julian century
        JT(RawIndex) = JD(RawIndex)/36525
    next
return
'
[IREX_RawFile]       'subtract skies from comp star data
    for RawIndex = 5 to RawIndexMax
                    'go through raw file and pick out the COMP stars with the selected filter J H
        if (RDitem$(RawIndex,7) = "C")  AND (RDitem$(RawIndex,9) = Filter$) then

                    'make list of unique comparison stars for COMP combobox
            CompFlag = 0
            for CompIndex = 1 to CompIndexMax          'see if the Comp Star is new to the list
                if CompStar$(CompIndex) = RDitem$(RawIndex,8) then
                    CompFlag = 1
                end if
            next
            if CompFlag = 0 then                      'add the new Comp Star to the list
                CompIndexMax = CompIndexMax + 1
                CompStar$(CompIndex) = RDitem$(RawIndex,8)
            end if

                    'find the past sky count to apply
            SkyPastCount = 0
            PastTime = 0
            for I = RawIndex to 5 step -1
                if (RDitem$(I,8) = "SKY         ") AND (RDitem$(I,9) = Filter$) OR_
                   (RDitem$(I,8) = "SKYNEXT     ") AND (RDitem$(I,9) = Filter$) then
                    SkyPastCount = CountFinal(I)
                    PastTime = JD(I)
                    exit for
                end if
            next

                    'find the future sky count to apply
            SkyFutureCount = 0
            FutureTime = 0
            for I = RawIndex to RawIndexMax step 1
                if (RDitem$(I,8) = "SKY         ") AND (RDitem$(I,9) = Filter$) OR_
                   (RDitem$(I,8) = "SKYLAST     ") AND (RDitem$(I,9) = Filter$) then
                    SkyFutureCount = CountFinal(I)
                    FutureTime = JD(I)
                    exit for
                end if
            next

                    'subtract sky from COMP count depending on SKY, SKYNEXT or SKYLAST protocol and change
                    'CountFinal value accordingly
            select case
                case (SkyPastCount = 0) AND (SkyFutureCount = 0)
                    notice "no SKY counts for COMP star at line# "; RawIndex
                    wait
                case (SkyPastCount > 0) AND (SkyFutureCount = 0)
                    CountFinal(RawIndex) = CountFinal(RawIndex) - SkyPastCount
                case (SkyPastCount = 0) AND (SkyFutureCount > 0)
                    CountFinal(RawIndex) = CountFinal(RawIndex) - SkyFutureCount
                case else
                        '                           y2 - y1
                        'interpolation:   y = y1 + --------- * (x - x1)
                        '                           x2 - x1
                    SkyCurrentCount = SkyPastCount + ((SkyFutureCount - SkyPastCount)/(FutureTime - PastTime))*_
                                      (JD(RawIndex) - PastTime)
                    CountFinal(RawIndex) = CountFinal(RawIndex) - SkyCurrentCount
            end select

            print #IREX, RDitem$(RawIndex,8);" ";_
                         using("#####.#####",JD(RawIndex));"  ";_
                         using("#######", CountFinal(RawIndex));"       ";_
                         Filter$;"   ";_
                         using("#######", SkyPastCount);"   ";_
                         using("#######",SkyFutureCount)
        end if
    next
return
'
'=====subroutines for creating estinction table
'
                                    'create ExtData() for display in listbox
[Create_Extinction_Table]
    redim ExtData$(500)
    redim ExtData(500,12)
    JDFlag = 0                      'flag used to find start and end Julian Dates for grapics print
    ExtIndex = 0
    for RawIndex = 5 to RawIndexMax
        if selectedComp$ = RDitem$(RawIndex,8) then
            ExtIndex = ExtIndex + 1
            gosub [Siderial_Time]
            gosub [Find_Air_Mass]
            ExtData(ExtIndex,1) = HA
            ExtData(ExtIndex,2) = AirMass
            select case RDitem$(RawIndex,9)
                case "J"
                    ExtData(ExtIndex,3) = CountFinal(RawIndex)               'j counts
                    ExtData(ExtIndex,4) = 0                                                                                                                       '
                    ExtData(ExtIndex,5) = -1.0857*log(CountFinal(RawIndex))  'ln(x) = 2.3026 * log(x)  j inst. mag.
                    ExtData(ExtIndex,6) = 0

                    if JDFlag = 0 then                                       'find the start and end times
                        StartJD = JD(RawIndex)
                        JDFlag = 1
                    end if
                    EndJD = JD(RawIndex)
                case "H"
                    ExtData(ExtIndex,3) =  0
                    ExtData(ExtIndex,4) = CountFinal(RawIndex)               'h counts                                                            '
                    ExtData(ExtIndex,5) = 0
                    ExtData(ExtIndex,6) = -1.0857*log(CountFinal(RawIndex))  'ln(x) = 2.3026 * log(x)  h inst. mag.

                    if JDFlag = 0 then                                       'find the start and end times
                        StartJD = JD(RawIndex)
                        JDFlag = 1
                    end if
                    EndJD = JD(RawIndex)
            end select
            ExtData$(ExtIndex) = using("###.##",ExtData(ExtIndex,1))+"  "+_  'hour angle
                                 using("#.###",ExtData(ExtIndex,2))+" "      'air mass

                                 for I = 3 to 4
                                    if ExtData(ExtIndex,I) = 0 then
                                        Temp$ = "        "
                                    else
                                        Temp$ = using("#######",ExtData(ExtIndex,I))+"    "    'j h counts
                                    end if
                                    ExtData$(ExtIndex) = ExtData$(ExtIndex) + Temp$
                                 next I

                                 for I = 5 to 6
                                    if ExtData(ExtIndex,I) = 0 then
                                        Temp$ = "        "
                                    else
                                        Temp$ = using("###.##",ExtData(ExtIndex,I))+"    "     'j h magnitude
                                    end if
                                    ExtData$(ExtIndex) = ExtData$(ExtIndex) + Temp$
                                 next I
        end if
    next
    ExtIndexMax = ExtIndex
    print #extinction.Table, "font courier_new 10 14"
    print #extinction.Table, "reload"
return
'
[Create_Regression_Array]
    RegIndex = 0
    select case Yaxis$           'select the appropriate data for desired color
        case "\j"
            color = 5
        case "\h"
            color = 6
    end select

    for ExtIndex = 1 to ExtIndexMax
        if ExtData(ExtIndex,color) <> 0 then
            RegIndex = RegIndex + 1
            X(RegIndex) = ExtData(ExtIndex,2)
            m(RegIndex) = ExtData(ExtIndex,color)
        end if
    next
    RegIndexMax = RegIndex

    open "IREX.txt" for append as #IREX
         print #IREX, "output from [Create_Regression_Array]"
         print #IREX, "color: "+right$(Yaxis$,1)
         print #IREX, "RegIndex     X         m"
         for RegIndex = 1 to RegIndexMax
            print #IREX, using("####",RegIndex)+"   "+_
                         using("###.####",X(RegIndex))+"   "+_
                         using("###.####",m(RegIndex))
         next
         print #IREX, " "
    close #IREX
return
'
'====subroutines for file operations
'
[Write_PPparms]
    open "PPparms2_JH.txt" for output as #PPparms
        print #PPparms, Location$                        'latitude and longitude
        print #PPparms, using("#.###",KJ)                'first order extinction for J
        print #PPparms, using("#.###",KH)                'first order extinction for H
        print #PPparms, using("##.###",KKjh)             'second order extinction for j-h, default = 0
        print #PPparms, using("##.###",Rho)              'transformation coeff. epislon for J using J-H
        print #PPparms, using("##.###",Lambda)           'transformation coeff. lambda for J-H
        print #PPparms, JDFlag                           '1 if using JD and 0 if using HJD
        print #PPparms, OBSCODE$                         'AAVSO observatory code
        print #PPparms, spare                            'Spare
        print #PPparms, using("##.###",ZPj)              'zero-point constant for j
        print #PPparms, using("##.###",ZPjh)             'zero-point constant for j-h
        print #PPparms, using("##.###",Ej)               'standard error for j
        print #PPparms, using("##.###",Ejh)              'standard error for j-h
    close #PPparms
return
'
[Find_File_Name]        'seperate out filename and extension from info() path/filename
    FileNameIndex = len(DataFile$)
    FileNameLength = len(DataFile$)
    while mid$(DataFile$, FileNameIndex,1)<>"\"                    'look for the last backlash
        FileNameIndex = FileNameIndex - 1
    wend
    FileNamePath$ = left$(DataFile$, FileNameIndex)
    DataFileName$ = right$(DataFile$, FileNameLength-FileNameIndex)

    print #extinction.FileName, DataFileName$                      'display filename in "File" textbox
return
'
'=====subroutines for calculations
'
[Find_Lat_Long]         'compute observers latitude and longitude in degrees
    LAT = val(mid$(Location$,2,4))
    if upper$(left$(Location$,1)) = "S" then
        LAT = LAT * -1
    end if
    LONGITUDE = val(mid$(Location$,8,5))
    if upper$(mid$(Location$,7,1)) = "W" then
        LONGITUDE = LONGITUDE * -1
    end if
return
'
[Siderial_Time]         'compute local siderial time
    MST = 280.46061837 + 360.98564736629 * JD(RawIndex) + 0.000387933 * JT(RawIndex) *_
          JT(RawIndex) - (JT(RawIndex) * JT(RawIndex) * JT(RawIndex))/38710000
    LMST = MST + LONGITUDE
    if LMST > 0 then
        while (LMST > 360)
            LMST = LMST - 360
        wend
    else
        while (LMST < 0)
            LMST = LMST + 360
        wend
    end if
return
'
[Find_Air_Mass]         'compute air mass
    HA = LMST - RA                          'find hour angle in degrees
    if HA < 0 then
        HA = HA + 360
    end if

    HAradians = HA * 3.1416/180             'convert to radians
    DECradians = DEC * 3.1416/180
    LATradians = LAT * 3.1416/180

                    'compute secant of zenith angle - distance from zenith in radians
    secZ = 1/(sin(LATradians) * sin(DECradians) + cos(LATradians) * cos(DECradians) * cos(HAradians))
                    'compute air mass using Hardie equation
                    'X = sec Z - 0.0018167(sec Z - 1) - 0.002875(sec Z - 1)^2 - 0.0008083(sec Z - 1)^3
    AirMass = secZ - 0.0018167 * (secZ - 1) - 0.002875 * (secZ - 1)^2 - 0.0008083 * (secZ - 1)^3
return
'
[Solve_Regression_Matrix]
            'linear least squares routine from Nielson
            'pulled from appendix I.4 of Astronomical Photometry, written by Hendon 1973
            'inputs     X() air mass array
            '           m() instrument magnitude array
            'outputs   Slope and Intercept,   Y = aX + b
            'number of elements in array = RegIndexMax
    a2 = 0
    a3 = 0
    c1 = 0
    c2 = 0
    a1 = RegIndexMax
    for I = 1 to RegIndexMax
        a2 = a2 + X(I)
        a3 = a3 + X(I) * X(I)
        c1 = c1 + m(I)
        c2 = c2 + m(I) * X(I)
    next
    det = 1/(a1 * a3 - a2 * a2)
    Intercept = -1 * (a2 * c2 - c1 * a3) * det
    Slope = (a1 * c2 - c1 * a2) * det

            'compute standard error using eq. 3.21
    if RegIndexMax > 2 then
        y.deviation.squared.sum = 0
        for N = 1 to RegIndexMax
            y.fit = Slope * X(N) + Intercept
            y.deviation = m(N) - y.fit
            y.deviation.squared.sum =  y.deviation.squared.sum + y.deviation^2
        next
        std.error = sqr((1/(N-2)) * y.deviation.squared.sum)
    else
        std.error = 0
    end if

    open "IREX.txt" for append as #IREX
        print #IREX, "output from [Solve_Regression_Matrix]"
        print #IREX, "color: "+right$(Yaxis$,1)
        print #IREX, "Intercept    Slope    std.error"
        print #IREX, using("###.###",Intercept)+"   "+_
                     using("###.###",Slope)+"   "+_
                     using("###.###",std.error)
        print #IREX, " "
   close #IREX
return
'
'=====graphics routines
'
[Draw_Graph_Outline]
    print #extinction.graphicbox1, "cls"
    print #extinction.graphicbox1, "font arial 8 16"
    print #extinction.graphicbox1, "down"
    print #extinction.graphicbox1, "place 10 125"
    print #extinction.graphicbox1, Yaxis$
    print #extinction.graphicbox1, "place 143 254"
    print #extinction.graphicbox1, "\X  (air mass)"
    print #extinction.graphicbox1, "font arial 6 12"
    print #extinction.graphicbox1, "place 30 235"
    print #extinction.graphicbox1, "\1.0"
    print #extinction.graphicbox1, "place 133 235"
    print #extinction.graphicbox1, "\1.5"
    print #extinction.graphicbox1, "place 233 235"
    print #extinction.graphicbox1, "\2.0"
    print #extinction.graphicbox1, "place 328 235"
    print #extinction.graphicbox1, "\2.5"
    print #extinction.graphicbox1, "line 40 20 40 220"
    print #extinction.graphicbox1, "line 40 220 340 220"
        print #extinction.graphicbox1, "font arial 6 12"
    for I = 1 to 10                     'print Y-axis divisions every 0.1 mag
        xD =  20 * I
        print #extinction.graphicbox1, "line 40 ";xD;" 45 ";xD
    next
    for I = 1 to 15                     'print X-axis division every 0.1 air mass
        yD = 40 + 20 * I
        print #extinction.graphicbox1, "line ";yD;" 220 ";yD;" 215"
    next
    for I = 0 to 3                      'print X-axis long division every 0.5 air mass
        yD = 40 + I * 100
        print #extinction.graphicbox1, "line ";yD;" 220 ";yD;" 210"
    next
return
'
[Draw_Best_Line]
    print #extinction.graphicbox1, "color ";PointColor$
    print #extinction.graphicbox1, "backcolor ";PointColor$
    StartLine = 1.0 * Slope + Intercept                         'start of best fit line at X = 1.00
    EndLine   = 2.5 * Slope + Intercept                         'end of best fit line at X = 2.50
    YaxisTop = abs(int(StartLine * 10)) * 20 + 60               'value for top of Y axis, about 0.2 mag higher
                                                                'than best fit line
    StartLine = YaxisTop - abs(int(StartLine * 200))
    EndLine = YaxisTop - abs(int(EndLine * 200))
    print #extinction.graphicbox1, "line 40 ";StartLine;" 340 ";EndLine
                                                                'plot data points
    for RegIndex = 1 to RegIndexMax
        Xpoint = int(X(RegIndex) * 200) - 160
        Ypoint = YaxisTop - abs(int(m(RegIndex) * 200))
        print #extinction.graphicbox1, "place ";Xpoint;" ";Ypoint
        print #extinction.graphicbox1, "circlefilled 3"
    next
                                                                'draw y scale values
    print #extinction.graphicbox1, "backcolor white"
    print #extinction.graphicbox1, "color black"
    print #extinction.graphicbox1, "font arial 6 12"
    print #extinction.graphicbox1, "place 10 25"
    Temporary = Slope + Intercept - .15
    ScaleTop$ = "\"+using("###.#", Temporary)
    print #extinction.graphicbox1, ScaleTop$
    print #extinction.graphicbox1, "place 10 220"
    Temporary = Temporary + 1
    ScaleBot$ = "\"+using("###.#", Temporary)
    print #extinction.graphicbox1, ScaleBot$
return
'
[Draw_Description]
                                                    'print file name
    print #extinction.graphicbox1, "place 60 300"
    print #extinction.graphicbox1, "\FILE NAME :"
    print #extinction.graphicbox1, "place 160 300"
    GraphicFileName$ = "\"+ShortDataFile$
    print #extinction.graphicbox1, GraphicFileName$ 
                                                    'print extiction star
    print #extinction.graphicbox1, "place 60 325"
    print #extinction.graphicbox1, "\STAR :"
    print #extinction.graphicbox1, "place 160 325"
    GraphicSelectedComp$ = "\"+selectedComp$
    print #extinction.graphicbox1, GraphicSelectedComp$
                                                    'print start Julian Date
    print #extinction.graphicbox1, "place 60 350"
    print #extinction.graphicbox1, "\START J2000:"
    print #extinction.graphicbox1, "place 160 350"
    GraphicStartJD$ = "\"+Using("####.####", StartJD)
    print #extinction.graphicbox1, GraphicStartJD$ 
                                                    'print end Julian Date
    print #extinction.graphicbox1, "place 60 375"
    print #extinction.graphicbox1, "\END J2000:"
    print #extinction.graphicbox1, "place 160 375"
    GraphicEndJD$ = "\"+Using("####.####", EndJD)
    print #extinction.graphicbox1, GraphicEndJD$ 
                                                    'print slope
    print #extinction.graphicbox1, "place 60 400"
    print #extinction.graphicbox1, "\SLOPE:"
    print #extinction.graphicbox1, "place 160 400"
    GraphicSlope$ = "\"+Using("####.###", Slope)
    print #extinction.graphicbox1, GraphicSlope$ 
                                                    'printer intercept
    print #extinction.graphicbox1, "place 60 425"
    print #extinction.graphicbox1, "\INTERCEPT:"
    print #extinction.graphicbox1, "place 160 425"
    GraphicIntercept$ = "\"+Using("####.###", Intercept)
    print #extinction.graphicbox1, GraphicIntercept$ 
                                                    'print standard error
    print #extinction.graphicbox1, "place 60 450"
    print #extinction.graphicbox1, "\STD. ERROR:"
    print #extinction.graphicbox1, "place 160 450"
    GraphicStd.error$ = "\"+Using("####.###", std.error)
    print #extinction.graphicbox1, GraphicStd.error$ 
return
'
end
