'                        Show Data Module - Differential Photometry
'                                      Optec, Inc.
'
'======version history
'V1.20, December 2015
'   modified PPparms2_JH for AllSky
'
'V1.10, October 2015
'   compiled with LB4.50
'   added 0.02 and 0.20 mag scales
'   expanded Y-axis by 2x
'   changed AAVSO submission to include append file
'
'V1.03, November 13, 2014
'   added file header
'
'V1.02, September 4, 2014
'   fixed error in start time for index
'
'V1.01, June 1014
'   fixed errorn in saving AAVSO observer code
'
'V1.00
'
'
'=====dimension statements and initial conditions
'
    DIM info$(10,10)
    files "c:\", info$()

    DIM TimeScale$(15)                  'time scale selections
    DIM TimeScaleArray(15)              'x-axis legend for marks every 10 divisions
    DIM TimeScaleArray$(9)              'x-axis legend for marks every 10 divisions

    DIM MagScale$(7)                    'magnitude scale selections
    DIM MagScaleArray(20)               'y-axis legend marks every division
    DIM MagScaleArray$(20)              'y-axis legend marks every division

    DIM TimeString$(3000,2)             'JD and magnitude array file for processing with PERIOD04

    DIM VarData$(3000)                  'data lines from variable file of observations, index var = VarIndex
    DIM VarItem$(3000,11)               'data items from variable file of observations, index var = VarIndex
                                        'VarItem$(x,1)  = UT day
                                        'VarItem$(x,2)  = UT month
                                        'VarItem$(x,3)  = UT year
                                        'VarItem$(x,4)  = UT hour
                                        'VatItem$(x,5)  = UT minute
                                        'VarItem$(x,6)  = UT second
                                        'VarItem$(x,7)  = Julian Datae J2000
                                        'VarItem$(x,8)  = J magnitude
                                        'VarItem$(x,9)  = J standard error
                                        'VarItem$(x,10) = J-H
                                        'VarItem$(x,11) = J-H standard error

    DIM AAVSOitem$(3000,15)             'data lines for making AAVSO format output, index var = VarIndex
                                        'AAVSOitem$(x,1)  = VAR$, variable star name
                                        'AAVSOitem$(x,2)  = JD in JDN, add 2451545 to J2000 date
                                        'AAVSOitem$(x,3)  = magnitude
                                        'AAVSOitem$(x,4)  = magnitude standard error
                                        'AAVSOitem$(x,5)  = filter, J H
                                        'AAVSOitem$(x,6)  = YES, used transformation coefficients
                                        'AAVSOitem$(x,7)  = STD, magnitude type is standard
                                        'AAVSOitem$(x,8)  = COMP$, comparison star name
                                        'AAVSOitem$(x,9)  = na, instrument magnitude of comparison
                                        'AAVSOitem$(x,10) = na, name of check star
                                        'AAVSOitem$(x,11) = na, instrument magnitude of check star
                                        'AAVSOitem$(x,12) = na, airmass of onservation
                                        'AAVSOitem$(x,13) = Group$, grouping identifier
                                        'AAVSOitem$(x,14) = CHART$, chart name
                                        'AAVSOitem$(x,15) = na, notes to data line

    DIM DrawLabel(50)

    DIM DateDecending(100)
    DIM DateAscending(100)
    DIM MagDecending(100)
    DIM MagAscending(100)
    DIM MagDiff(100)
'
'=====open and read in PPparms data for configuration
'
open "PPparms2_JH.txt" for input as #PPparms
    input #PPparms, Location$                       'latitude and longitude in degrees
    input #PPparms, KJ                              'first order extinction for J
    input #PPparms, KH                              'first order extinction for H
    input #PPparms, KKjh                            'second order extinction for j-h, default = 0
    input #PPparms, Rho                             'transformation coeff. for J using J-H
    input #PPparms, Lambda                          'transformation coeff. J-H
    input #PPparms, JDFlag                          '1 if using JD and 0 if using HJD
    input #PPparms, OBSCODE$                        'AAVSO observatory code
    input #PPparms, Spare2                          '0 not used at this time
    input #PPparms, ZPj                             'zero-point constant for j
    input #PPparms, ZPjh                            'zero-point constant for j-h
    input #PPparms, Ej                              'standard error for j
    input #PPparms, Ejh                             'standard error for j-h
close #PPparms

            open "IREX.txt" for output as #IREX
                print #IREX, "Reduction JH module"
                print #IREX, " "
                print #IREX, "PPparms2 JH.txt"
                print #IREX, "  Location$     ";Location$
                print #IREX, "  KJ            ";KJ
                print #IREX, "  KH            ";KH
                print #IREX, "  KKjh          ";KKjh
                print #IREX, "  Rho           ";Rho
                print #IREX, "  Lambda        ";Lambda
                print #IREX, "  JDFlag        "; JDFlag
                print #IREX, "  OBSCODE       "; OBSCODE$
                print #IREX, "  Spare2        "; Spare2
                print #IREX, "  ZPj           "; ZPj
                print #IREX, "  ZPjh          "; ZPjh
                print #IREX, "  Ej            "; Ej
                print #IREX, "  Ejh           "; Ejh
                print #IREX, " "
            close #IREX
'
'=====initialize and start up values
'
    VersionNumber$    = "1.20"
    PathDataFile$     = "* .var"        'default path for data files
    PathPlotFile$     = "* .bmp"        'default path for plot files
    PathDataFastFile$ = "* .raw"        'default path for fast data files
    PathAAVSO$        = "* .txt"        'default path for AAVSO format data files

    ConnectDotFlag = 0                  'use dots for plotting
    CursorID$ = "A"                     'default to A cursor at start up
    CursorFlag.B = 0                    'turn off cursor B values and B-A values
    CursorFlag = 0                      'no cursor
    PlotErrorFlag = 0                   'do not plot error limits
    AAVSOFlag = 0                       'AAVSO export window is closed
    LabelCounter = 0
'
'=====set up main GUI control window
'
[WindowSetup]
    NOMAINWIN
    WindowWidth = 1024 : WindowHeight = 732
    UpperLeftX = INT((DisplayWidth-WindowWidth)/2)
    UpperLeftY = INT((DisplayHeight-WindowHeight)/2)

[ControlSetup]
    Menu        #show, "File",      "Open Var File", [OpenFile],_
                                    "Save Plot", [DataSave],_
                                    "Export AAVSO format", [AAVSO],_
                                    "Quit", [QuitShowData]
    Menu        #show, "Config",     "Plot Dots", [PlotDots],_
                                     "Plot Lines",[PlotLines],_
                                     "Plot Std. Error",[PlotErrors]
    Menu        #show, "Cursor",     "Cursor A", [CursorA],_
                                     "Cursor B", [CursorB],_
                                     "Cursor C (time of minimum)", [CursorC]
    Menu        #show, "Edit",       "Edit Title",[EditTitle],_
                                     "Edit File Info",[EditFileInfo],_
                                     "Add Label",[AddLabel],_
                                     "Undo Label",[DeleteLabel]
    Menu        #show, " Analysis ", "Eclipsing Binary Analysis", [FindMin],_
                                     "Phase Plot", [FindPeriod],_
                                     "Fourier Analysis", [FindFourier]
    Menu        #show, "Help",       "About", [About], "Help", [Help]

    graphicbox  #show.graphicbox1,   11, 7, 995, 575

    groupbox    #show.groupbox1, "Variable Star File", 11, 585, 250, 85
    groupbox    #show.groupbox2, "Time Scale", 272, 585, 116, 85
    groupbox    #show.groupbox3, "Mag Scale", 399, 585, 116, 85
    groupbox    #show.groupbox7, "Plot", 526, 585, 125, 85
    groupbox    #show.groupbox4, "Move Date", 662, 585, 134, 85
    groupbox    #show.groupbox5, "Move Mag", 806, 585, 110, 85

    textbox     #show.FileName, 19, 620, 235, 23
    textbox     #show.StartDate, 670, 642, 90, 23
    textbox     #show.StartMag, 814, 642, 65, 23

    statictext  #show.StartDateText, "Enter Start Date", 677,627,90,15
    statictext  #show.StartMagText, "Enter Max Mag", 810,627,80,15

    button      #show.PlotJ, "J",[Plot],UL, 532, 605, 51, 27
    button      #show.PlotIndex, "J-H", [PlotIndex],UL, 532, 635, 51, 27
    button      #show.PlotH, "H", [PlotH], UL, 588, 605, 51, 27

    button      #show.MoveTimeLeftFast, "<<",[MoveTimeLeftFast],UL, 670, 600, 25, 25
    button      #show.MoveTimeLeft, "<",[MoveTimeLeft], UL, 702, 600, 25, 25
    button      #show.MoveTimeRight, ">",[MoveTimeRight], UL, 734, 600, 25, 25
    button      #show.MoveTimeRightFast, ">>",[MoveTimeRightFast],UL, 765, 600, 25, 25
    button      #show.EnterDate, "",[EnterStartDate],UL, 765, 640, 25, 25

    button      #show.MoveMagUp, "",[MoveMagUp], UL, 831, 600, 25, 25
    button      #show.MoveMagDown, "",[MoveMagDown], UL, 862, 600, 25, 25
    button      #show.EnterMag, "", [EnterMag], UL, 884, 640, 25, 25

    combobox    #show.TimeScale, TimeScale$(),[TimeScale], 280, 620, 100, 25
    combobox    #show.Magnitude, MagScale$(), [MagScale], 407, 620, 100, 25

    Open "Plot Data - JH Photometry" for Window as #show

    #show "trapclose [QuitShowData]"
    #show.graphicbox1 "down; fill White; flush"
    #show "font courier_new 10 14"

    print #show.StartDateText, "!font ariel 8"
    print #show.StartMagText, "!font ariel 8"

    print #show.EnterDate, ""
    print #show.EnterMag, ""

    print #show.MoveMagUp, "!font symbol 10 12"
    print #show.MoveMagUp, ""
    print #show.MoveMagDown, "!font symbol 10 12"
    print #show.MoveMagDown, ""

    print #show.MoveTimeLeftFast, "!disable"
    print #show.MoveTimeLeft, "!disable"
    print #show.MoveTimeRight, "!disable"
    print #show.MoveTimeRightFast, "!disable"
    print #show.EnterDate, "!disable"

    print #show.MoveMagUp, "!disable"
    print #show.MoveMagDown, "!disable"
    print #show.EnterMag, "!disable"

    print #show.PlotH, "!disable"
    print #show.PlotJ, "!disable"
    print #show.PlotIndex, "!disable"

    print #show.TimeScale, "disable"
    print #show.Magnitude, "disable"

    print #show.graphicbox1, "setfocus; when leftButtonDown [FindPosition]"
    print #show.graphicbox1, "setfocus; when rightButtonDown [EraseCursor]"

    gosub [Draw_Graph_Outline]

    print #show.FileName, "open data file"

Wait                                'finished setting up, wait here for new command
'
'======menu controls
'
'
[OpenFile]                                          'opens variable star file
    filedialog "Open Data File", PathDataFile$, DataFile$


    for I = len(DataFile$) to 1 step -1             'remember path for opened folder and file
        if mid$(DataFile$,I,1) = "\" then
            ShortDataFile$ = mid$(DataFile$,I+1)
            PathDataFile$ = left$(DataFile$,I)+"*var"
            exit for
        end if
    next I

    if DataFile$ = "" then
        wait
    else
        open DataFile$ for input as #VarFile
        files "c:\", DataFile$, info$()
        if val(info$(0, 0)) = 0 then
            notice "cannot create new file"
        else
            gosub [Find_File_Name]                  'extract filename from path and print in file box

            open "IREX.txt" for output as #IREX
                print #IREX, "filename, output from [Find_File_Name] ";DataFileName$
                print #IREX, " "
            close #IREX

            JDFlag$ = ""
            CompStar$ = ""
            VarStar$ = ""
            VarIndex = 0
            FileHeaderFlag = 0
                while eof(#VarFile)=0               'read variable data to end of file
                    VarIndex = VarIndex + 1
                    line input #VarFile, VarData$(VarIndex)
                    if left$(VarData$(VarIndex),1) = "#" then
                        FileHeaderFlag = 1
                        VarIndex = 0
                    end if
                wend
                VarIndexMax = VarIndex
            close #VarFile
            if FileHeaderFlag = 1 then              'read the header info only if it exists
                open DataFile$ for input as #VarFile
                input #VarFile, Temp$, JDFlag$, CompStar$, VarStar$
                close #VarFile
            end if
        end if
    end if
                                                    'get file name without type
    PlotFileName$ = upper$(left$(DataFileName$, (len(DataFileName$) - 4)))

    Redim TimeScale$(15)
    TimeScale$(1)  = " 1.000m"
    TimeScale$(2)  = " 2.000m"
    TimeScale$(3)  = " 5.000m"
    TimeScale$(4)  = "10.000m"
    TimeScale$(5)  = " 1.000h"
    TimeScale$(6)  = " 2.000h"
    TimeScale$(7)  = " 5.000h"
    TimeScale$(8)  = "10.000h"
    TimeScale$(9)  = " 1.000d"
    TimeScale$(10) = " 2.000d"
    TimeScale$(11) = " 5.000d"
    TimeScale$(12) = "10.000d"
    TimeScale$(13) = "20.000d"
    print #show.TimeScale, "reload"
    print #show.TimeScale, "select ";"select"

    Redim MagScale$(7)
    MagScale$(1)   = " 0.01"
    MagScale$(2)   = " 0.02"
    MagScale$(3)   = " 0.05"
    MagScale$(4)   = " 0.10"
    MagScale$(5)   = " 0.20"
    MagScale$(6)   = " 0.50"
    MagScale$(7)   = " 1.00"
    print #show.Magnitude, "reload"
    print #show.Magnitude, "select ";""

    gosub [ConvertVarFile]                          'extract all data

    PlotFlag = 1                                    'plot V magnitude
    CursorFlag = 0                                  'no cursor
    FileHeader$ = ""                                'blank out file info for variable star data
    CursorFlag.B = 0

    print #show.PlotIndex, "!enable"
    print #show.PlotJ, "!enable"
    print #show.PlotH, "!disable"

    print #show.MoveTimeLeft, "!disable"
    print #show.MoveTimeLeftFast, "!disable"
    print #show.MoveTimeRight, "!disable"
    print #show.MoveTimeRightFast, "!disable"
    print #show.EnterDate, "!disable"

    print #show.MoveMagUp, "!disable"
    print #show.MoveMagDown, "!disable"
    print #show.EnterMag, "!disable"

    print #show.TimeScale, "enable"
    print #show.Magnitude, "enable"

    print #show.StartDate, ""
    print #show.StartMag,  ""

    print #show.graphicbox1, "delsegment ";CursorValuesSegment.A
    print #show.graphicbox1, "delsegment ";CursorValuesSegment.B
    print #show.graphicbox1, "delsegment ";CursorValuesSegment.C
    print #show.graphicbox1, "delsegment ";MagScaleSegment
    print #show.graphicbox1, "delsegment ";TimeScaleSegment
    print #show.graphicbox1, "delsegment ";DrawDataSegment
    print #show.graphicbox1, "delsegment ";CursorSegmentA
    print #show.graphicbox1, "delsegment ";CursorSegmentB
    print #show.graphicbox1, "delsegment ";CursorSegmentC
    print #show.graphicbox1, "delsegment ";CursorSegmentD
    print #show.graphicbox1, "delsegment ";CursorDiffSegment
    print #show.graphicbox1, "delsegment ";DrawTitle

    print #show.graphicbox1, "delsegment ";DrawJSegment
    print #show.graphicbox1, "delsegment ";DrawHSegment
    print #show.graphicbox1, "delsegment ";DrawJHSegment

    For I = 0 to LabelCounter
        print #show.graphicbox1, "delsegment ";DrawLabel(I)
    next
    LabelCounter = 0

    CursorFlag.C = 0
    CursorFlag.B = 0
    CursorFlag.A = 0
    CursorTime.A$ = ""
    CursorTime.B$ = ""
    CursorTime.C$ = ""
    CursorTime.A = 0
    CursorTime.B = 0
    CursorTime.C = 0
    CursorID$ = "A"

    print #show.graphicbox1, "redraw"
Wait
'
[DataSave]                                      'save bitmap image to disk
    filedialog "Save As...", PathPlotFile$, PlotFile$
    for I = len(PlotFile$) to 1 step -1
        if mid$(PlotFile$,I,1) = "\" then
            ShortPlotFile$ = mid$(PlotFile$,I+1)
            PathPlotFile$ = left$(PlotFile$,I)+"*bmp"
            exit for
        end if
    next I

    if PlotFile$ <> "" then
        print #show.graphicbox1, "getbmp plot 0 0 993 323"
        if (right$(PlotFile$,4) = ".bmp") OR (right$(PlotFile$,4) = ".BMP") then
            bmpsave "plot", PlotFile$
        else
            PlotFile$ = PlotFile$+".bmp"
            bmpsave "plot", PlotFile$
        end if
    end if
wait
'
'
[FindPosition]                                  'find position from cursor mouse values
    if CursorFlag = 1 then
        gosub [DrawCursor]

        if PlotFlag = 1 OR PlotFlag = 2 then
            CursorTime = val(StartTime$) + ((MouseX - 40)/PixelTime)
            CursorTime$ = using("####.####",CursorTime)

            if PlotFlag = 1 then
                CursorMag = MagScaleArray(0) + (MouseY - 20)/PixelMag
                CursorMag$ = using("###.###",CursorMag)
            else
                CursorMag = VarColorMax - (MouseY - 20)/PixelMag
                CursorMag$ = using("###.###",CursorMag)
            end if
        else
            CursorTime = (MouseX - 40)/PixelTime + StartTime

                HR = int(CursorTime / 3600)
                MIN = int((CursorTime MOD 3600) / 60)
                SEC = CursorTime - 3600 * HR - 60 * MIN

                if HR >= 24 then
                    HR = HR - 24
                end if

                HR$ = using("##",HR)
                MIN$ = using("##",MIN)
                SEC$ = using("##.###",SEC)

            CursorTime$ = HR$+":"+MIN$+":"+SEC$

            CursorMag = CountMax - (MouseY - 20)/PixelMag
            CursorMag$ = using("####",CursorMag)
        end if

        Select Case CursorID$
            Case "A"
                CursorMag.A = CursorMag
                CursorMag.A$ = CursorMag$
                CursorTime.A = CursorTime
                CursorTime.A$ = CursorTime$
                gosub [DrawCursorValues.A]

                Select Case
                    case MagScale = 0.01 or MagScale = 0.02 or MagScale = 0.05
                        print #show.StartDate, CursorTime.A$
                        CursorMagMax.A$ = left$(CursorMag.A$,6)
                        print #show.StartMag, CursorMagMax.A$
                    case MagScale = 0.10 or MagScale = 0.20 or MagScale = 0.50
                        print #show.StartDate, CursorTime.A$
                        CursorMagMax.A$ = left$(CursorMag.A$,5)
                        print #show.StartMag, CursorMagMax.A$
                    case MagScale = 1.0
                        print #show.StartDate, CursorTime.A$
                        CursorMagMax.A$ = left$(CursorMag.A$,3)
                        print #show.StartMag, CursorMagMax.A$
                    case MagScale = 5 or MagScale = 10
                        print #show.StartDate, left$(CursorTime.A$,8)
                        CursorMagMax.A$ = str$(int((CursorMag.A+5)/10)*10)
                        print #show.StartMag, CursorMagMax.A$
                    case MagScale = 50 or MagScale = 100
                        print #show.StartDate, left$(CursorTime.A$,8)
                        CursorMagMax.A$ = str$(int((CursorMag.A+50)/100)*100)
                        print #show.StartMag, CursorMagMax.A$ 
                    case MagScale = 1000
                        print #show.StartDate, left$(CursorTime.A$,8)
                        CursorMagMax.A$ = str$(int((CursorMag.A+500)/1000)*1000)
                        print #show.StartMag, CursorMagMax.A$ 
                End Select
                CursorFlag.A = 1
            Case "B"
                CursorMag.B = CursorMag
                CursorMag.B$ = CursorMag$
                CursorTime.B = CursorTime
                CursorTime.B$ = CursorTime$ 
                gosub [DrawCursorValues.B]

                CursorFlag.B = 1
            Case "C"
                CursorTime.C = CursorTime
                CursorTime.C$ = CursorTime$
                gosub [DrawCursorValues.C]
        End Select

        CursorTimeDiff = CursorTime.B - CursorTime.A
        if CursorTimeDiff < 0 then
            Sign = 1
            CursorTimeDiff = abs(CursorTimeDiff)
        else
            Sign = 0
        end if

        CursorMagDiff = CursorMag.B - CursorMag.A

        if PlotFlag = 1 OR PlotFlag = 2 then
            CursorTimeDiff$ = using("###.####", CursorTimeDiff)
            CursorMagDiff$ = using("###.###",CursorMagDiff)
        end if

        if PlotFlag = 0 then
                HR = int(CursorTimeDiff / 3600)
                MIN = int((CursorTimeDiff MOD 3600) / 60)
                SEC = CursorTimeDiff - 3600 * HR - 60 * MIN

                if HR >= 24 then
                    HR = HR - 24
                end if

                HR$ = using("##",HR)
                MIN$ = using("##",MIN)
                SEC$ = using("##.###",SEC)

            CursorTimeDiff$ = HR$+":"+MIN$+":"+SEC$ 
            CursorMagDiff$ = using("#####",CursorMagDiff)
        end if

        if CursorFlag.B = 1 and CursorFlag.A = 1 then
            gosub [DrawCursorDiffValues]
        else
            CursorTimeDiff$ = ""
            CursorMagDiff$ = ""
            gosub [DrawCursorDiffValues]
        end if
    else
        notice "plot the data first"
    end if
wait
'
[EraseCursor]
    print #show.graphicbox1, "delsegment ";CursorSegmentA
    print #show.graphicbox1, "delsegment ";CursorSegmentB
    print #show.graphicbox1, "delsegment ";CursorSegmentC
    print #show.graphicbox1, "delsegment ";CursorSegmentD
    print #show.graphicbox1, "delsegment ";CursorValuesSegment.A
    print #show.graphicbox1, "delsegment ";CursorValuesSegment.B
    print #show.graphicbox1, "delsegment ";CursorValuesSegment.C
    print #show.graphicbox1, "delsegment ";CursorDiffSegment
    print #show.graphicbox1, "redraw"
    CursorFlag.C = 0
    CursorFlag.B = 0
    CursorFlag.A = 0
    CursorTime.A$ = ""
    CursorTime.B$ = ""
    CursorTime.C$ = ""
 '   CursorID$ = "A"
wait
'
[PlotDots]
        ConnectDotFlag = 0
wait
'
[PlotLines]
        ConnectDotFlag = 1
wait
'
[PlotErrors]
    confirm "include error limits in data?"; ErrorAnswer$
    if ErrorAnswer$ = "yes" then
        PlotErrorFlag = 1
    else
        PlotErrorFlag = 0
    end if
wait
'
[EditTitle]
    TempFileName$ = PlotFileName$
    prompt "Enter new title"; TempFileName$
    if TempFileName$ <> "" then
        PlotFileName$ = TempFileName$ 
        gosub [DrawTitles]
    end if
wait
'
[EditFileInfo]
    TempFileName$ = FileHeader$
    prompt "Enter new file info";TempFileName$
    if TempFileName$ <> "" then
        FileHeader$ = TempFileName$
        gosub [DrawTitles]
    end if
wait
'
[AddLabel]
    if CursorFlag = 0 then
        notice "plot data first"
    else
        prompt "add label at cursor position"; Label$
        if Label$ <> "" then
            gosub [DrawLabel]
        end if
    end if
wait
'
[DeleteLabel]
    print #show.graphicbox1, "delsegment ";DrawLabel(LabelCounter)
    print #show.graphicbox1, "redraw"
    LabelCounter = LabelCounter - 1
wait
'
[CursorA]
    CursorID$ = "A"
wait
'
[CursorB]
    CursorID$ = "B"
wait
'
[CursorC]
    CursorID$ = "C"
wait
'
[About]
    notice "Data Plotter  JH Photometry"+chr$(13)+_
           "Version "+VersionNumber$+chr$(13)+_
           "copyright 2013-2015, Gerald Persha."+chr$(13)+_
           "www.SSPDataq.com"
Wait
'
[Help]
    run "hh photometry jh.chm"
Wait
'
[QuitShowData]                   'exit program
    confirm "do you wish to exit program?"; Answer$

    if Answer$ = "yes" then
        if AAVSOFlag = 1 then
            close #export
        end if
        if FindMinimumFlag = 1 then
            close #FindMinimum
        end if
        if FindPeriodFlag = 1 then
            close #FindPeriod
        end if
        if FindFourierFlag = 1 then
            close #FindFourier
        end if
        close #show
        END
    else
        wait
    end if
'
'=====comboboxes input
'
[TimeScale]
    print #show.TimeScale, "selection? TimeScaleIndex$"
    Select Case TimeScaleIndex$
        Case " 1.000m"
            TimeScale = 60
        Case " 2.000m"
            TimeScale = 120
        Case " 5.000m"
            TimeScale = 300
        Case "10.000m"
            TimeScale = 600
        Case " 1.000h"
            TimeScale = 3600
        Case " 2.000h"
            TimeScale = 7200
        Case " 5.000h"
            TimeScale = 18000
        Case "10.000h"
            TimeScale = 36000
        Case " 1.000d"
            TimeScale = 86400
        Case " 2.000d"
            TimeScale = 172800
        Case " 5.000d"
            TimeScale = 432000
        Case "10.000d"
            TimeScale = 864000
        Case "20.000d"
            TimeScale = 1728000
    End Select

    open "IREX.txt" for append as #IREX
        print #IREX, "Time Scale Pick ";TimeScale
        print #IREX, " "
    close #IREX
wait
'
[MagScale]
    print #show.Magnitude, "selection? MagScaleIndex$"
    Select Case MagScaleIndex$
        Case " 0.01"
            MagScale = 0.01
       Case " 0.02"
            MagScale = 0.02
        Case " 0.05"
            MagScale = 0.05
        Case " 0.10"
            MagScale = 0.10
        Case " 0.20"
            MagScale = 0.20
        Case " 0.50"
            MagScale = 0.50
        Case " 1.00"
            MagScale = 1.00
        Case "   5"
            MagScale = 5
        Case "  10"
            MagScale = 10
        Case "  50"
            MagScale = 50
        Case " 100"
            MagScale = 100
         Case "1000"
            MagScale = 1000
    End Select

    CountsMax = MagScale * 10

    open "IREX.txt" for append as #IREX
        print #IREX, "Mag Scale Pick ";MagScale
        print #IREX, " "
    close #IREX
wait
'
'=====control buttons
'
[Plot]
    if PlotFlag = 2 then
        PlotFlag = 1
    end if


    print #show.graphicbox1, "delsegment ";DrawJHSegment

    print #show.graphicbox1, "delsegment ";CursorSegmentA
    print #show.graphicbox1, "delsegment ";CursorSegmentB
    print #show.graphicbox1, "delsegment ";CursorValuesSegment.A
    print #show.graphicbox1, "delsegment ";CursorValuesSegment.B
    print #show.graphicbox1, "delsegment ";CursorDiffSegment

    For I = 0 to LabelCounter
        print #show.graphicbox1, "delsegment ";DrawLabel(I)
    next
    LabelCounter = 0

    print #show.graphicbox1, "redraw"
    CursorFlag.B = 0
    CursorID$ = "A"

    if PlotFlag = 1 then
        print #show.StartDateText, "Enter Start Date"
        print #show.StartMagText, "Enter Max Mag"
        gosub [PlotJ]
    end if
wait
'
[PlotIndex]

    print #show.graphicbox1, "delsegment ";DrawJHSegment

    print #show.graphicbox1, "delsegment ";CursorSegmentA
    print #show.graphicbox1, "delsegment ";CursorSegmentB
    print #show.graphicbox1, "delsegment ";CursorValuesSegment.A
    print #show.graphicbox1, "delsegment ";CursorValuesSegment.B
    print #show.graphicbox1, "delsegment ";CursorDiffSegment

    For I = 0 to LabelCounter
        print #show.graphicbox1, "delsegment ";DrawLabel(I)
    next
    LabelCounter = 0

    print #show.graphicbox1, "redraw"
    CursorFlag.B = 0
    CursorID$ = "A"

    PlotFlag = 2                                            'plotting index values

    print #show.StartDateText, "Enter Start Date"
    print #show.StartMagText, "Enter Max Mag"

    gosub [PlotColorIndex]
wait
'
'
[PlotH]
    HmagFlag = 1
    gosub [DrawH]
wait
'
'
[MoveTimeLeft]
    print #show.graphicbox1, "delsegment ";CursorSegmentA
    print #show.graphicbox1, "delsegment ";CursorSegmentB

    For I = 0 to LabelCounter
        print #show.graphicbox1, "delsegment ";DrawLabel(I)
    next
    LabelCounter = 0
    print #show.graphicbox1, "redraw"

    select case PlotFlag
        case 0
            StartTime = StartTime - 10 * TimeScale
            gosub [FindStartTime]
            gosub [DrawData]
        case 1
            StartJ2000 = StartJ2000 - 10 * TimeScaleJ2000
            StartJ2000$ = using("####.####",StartJ2000)
            StartTime$ = StartJ2000$ 
            gosub [DrawData]
            if HmagFlag = 1 then
                gosub [DrawHmags]
            end if
        case 2
            StartJ2000 = StartJ2000 - 10 * TimeScaleJ2000
            StartJ2000$ = using("####.####",StartJ2000)
            StartTime$ = StartJ2000$
            gosub [DrawColorIndicies]
        end select
        gosub [DrawTimeScale]
wait
'
[MoveTimeRight]
    print #show.graphicbox1, "delsegment ";CursorSegmentA
    print #show.graphicbox1, "delsegment ";CursorSegmentB
    For I = 0 to LabelCounter
        print #show.graphicbox1, "delsegment ";DrawLabel(I)
    next
    LabelCounter = 0
    print #show.graphicbox1, "redraw"

    select case PlotFlag
        case 0
            StartTime = StartTime + 10 * TimeScale
            gosub [FindStartTime]
            gosub [DrawData]
        case 1
            StartJ2000 = StartJ2000 + 10 * TimeScaleJ2000
            StartJ2000$ = using("####.####",StartJ2000)
            StartTime$ = StartJ2000$ 
            gosub [DrawData]
            if HmagFlag = 1 then
                gosub [DrawHmags]
            end if
        case 2
            StartJ2000 = StartJ2000 + 10 * TimeScaleJ2000
            StartJ2000$ = using("####.####",StartJ2000)
            StartTime$ = StartJ2000$
            gosub [DrawColorIndicies]
        end select
        gosub [DrawTimeScale]
wait
'
[MoveTimeLeftFast]
    print #show.graphicbox1, "delsegment ";CursorSegmentA
    print #show.graphicbox1, "delsegment ";CursorSegmentB
    For I = 0 to LabelCounter
        print #show.graphicbox1, "delsegment ";DrawLabel(I)
    next
    LabelCounter = 0
    print #show.graphicbox1, "redraw"

    select case PlotFlag
        case 0
            StartTime = StartTime - 50 * TimeScale
            gosub [FindStartTime]
            gosub [DrawData]
        case 1
            StartJ2000 = StartJ2000 - 50 * TimeScaleJ2000
            StartJ2000$ = using("####.####",StartJ2000)
            StartTime$ = StartJ2000$ 
            gosub [DrawHmags]
            gosub [DrawData]
            if HmagFlag = 1 then
                gosub [DrawHmags]
            end if
        case 2
            StartJ2000 = StartJ2000 - 50 * TimeScaleJ2000
            StartJ2000$ = using("####.####",StartJ2000)
            StartTime$ = StartJ2000$
            gosub [DrawColorIndicies]
        end select
        gosub [DrawTimeScale]
wait
'
[MoveTimeRightFast]
    print #show.graphicbox1, "delsegment ";CursorSegmentA
    print #show.graphicbox1, "delsegment ";CursorSegmentB
    For I = 0 to LabelCounter
        print #show.graphicbox1, "delsegment ";DrawLabel(I)
    next
    LabelCounter = 0
    print #show.graphicbox1, "redraw"

    select case PlotFlag
        case 0
            StartTime = StartTime + 50 * TimeScale
            gosub [FindStartTime]
            gosub [DrawData]
        case 1
            StartJ2000 = StartJ2000 + 50 * TimeScaleJ2000
            StartJ2000$ = using("####.####",StartJ2000)
            StartTime$ = StartJ2000$ 
            gosub [DrawHmags]
            gosub [DrawData]
            if HmagFlag = 1 then
                gosub [DrawHmags]
            end if
        case 2
            StartJ2000 = StartJ2000 + 50 * TimeScaleJ2000
            StartJ2000$ = using("####.####",StartJ2000)
            StartTime$ = StartJ2000$
            gosub [DrawColorIndicies]
        end select
        gosub [DrawTimeScale]
wait
'
[EnterStartDate]
    print #show.graphicbox1, "delsegment ";CursorSegmentA
    print #show.graphicbox1, "delsegment ";CursorSegmentB
    For I = 0 to LabelCounter
        print #show.graphicbox1, "delsegment ";DrawLabel(I)
    next
    LabelCounter = 0
    print #show.graphicbox1, "redraw"

    print #show.StartDate, "!contents?"
    input #show.StartDate, NewDate$

        if NewDate$ = "" then
            notice "enter valid value"
            wait
        end if

    Select Case PlotFlag
        case 0
            For I = 1 to 5
                if mid$(NewDate$,I,1) = ":" then
                    C1 = I
                    exit for
                end if
            next
            HR$ = left$(NewDate$,(C1 - 1))

            if val(HR$) > 24 OR val(HR$) < 0 then
                notice "entire valid hour value"
                wait
            end if

            For I = 1 to 5
                if mid$(NewDate$,(C1 + 1 + I),1) = ":" then
                    C2 = C1 + 1 + I
                    exit for
                end if
            next
            MIN$ = mid$(NewDate$, (C1+1), I)

            if val(MIN$) > 60 OR val(MIN$) < 0 then
                notice "entire valid minute value"
                wait
            end if

            SEC$ = mid$(NewDate$, (C2+1))

            if val(SEC$) > 60 OR val(SEC$) < 0 then
                notice "entire valid second value"
                wait
            end if

            StartTime = val(HR$)*3600 + val(MIN$)*60 + val(SEC$)

            gosub [FindStartTime]
            gosub [DrawData]

            open "IREX.txt" for append as #IREX
                print #IREX, "Output from start time textbox"
                print #IREX, "NewDate$ = ";NewDate$;"  StartTime = ";StartTime;"  HR$ = ";HR$;"  MIN$ = ";MIN$;"  SEC$ = ";SEC$
                print #IREX, " "
            close #IREX
        case 1

            if val(NewDate$) < 1000 OR val(NewDate$) > 9999 then
                notice "enter valid date (JD2000 1000 to 9999)"
                wait
            end if

            StartJ2000$ = using("####.####",val(NewDate$))
            StartJ2000 = val(StartJ2000$)
            StartTime$ = StartJ2000$ 

            gosub [DrawData]
            if HmagFlag = 1 then
                gosub [DrawHmags]
            end if
        case 2

            if val(NewDate$) < 1000 OR val(NewDate$) > 9999 then
                notice "enter valid date (J2000 1000 to 9999)"
                wait
            end if

            StartJ2000$ = using("####.####",val(NewDate$))
            StartTime$ = StartJ2000$ 
            gosub [DrawColorIndicies]
        end select

        gosub [DrawTimeScale]
wait
'
[MoveMagUp]
    print #show.graphicbox1, "delsegment ";CursorSegmentA
    print #show.graphicbox1, "delsegment ";CursorSegmentB
    For I = 0 to LabelCounter
        print #show.graphicbox1, "delsegment ";DrawLabel(I)
    next
    LabelCounter = 0
    print #show.graphicbox1, "redraw"

    Select Case PlotFlag
        case 0
            CountMax = CountMax + MagScale
            gosub [DrawData]
        case 1
            MagScaleArray(0) = MagScaleArray(0) - MagScale
            gosub [DrawData]
            if HmagFlag = 1 then
                gosub [DrawHmags]
            end if
        case 2
            VarColorMax = VarColorMax + MagScale
            MagScaleArray(0) = VarColorMax
            gosub [DrawColorIndicies]
        end select

    gosub [MakeMagArray]
    gosub [DrawMagScale]
wait
'
[MoveMagDown]
    print #show.graphicbox1, "delsegment ";CursorSegmentA
    print #show.graphicbox1, "delsegment ";CursorSegmentB
    For I = 0 to LabelCounter
        print #show.graphicbox1, "delsegment ";DrawLabel(I)
    next
    LabelCounter = 0
    print #show.graphicbox1, "redraw"

    Select Case PlotFlag
        case 0
            CountMax = CountMax - MagScale
            gosub [DrawData]
        case 1
            MagScaleArray(0) = MagScaleArray(0) + MagScale
            gosub [DrawData]
            if HmagFlag = 1 then
                gosub [DrawHmags]
            end if
        case 2
            VarColorMax = VarColorMax - MagScale
            MagScaleArray(0) = VarColorMax
            gosub [DrawColorIndicies]
        end select

    gosub [MakeMagArray]
    gosub [DrawMagScale]
wait
'
[EnterMag]
    print #show.graphicbox1, "delsegment ";CursorSegmentA
    print #show.graphicbox1, "delsegment ";CursorSegmentB
    For I = 0 to LabelCounter
        print #show.graphicbox1, "delsegment ";DrawLabel(I)
    next
    LabelCounter = 0
    print #show.graphicbox1, "redraw"

    print #show.StartMag, "!contents?"
    input #show.StartMag, TestContents$ 
        if TestContents$ = "" then
            notice "enter valid value"
            wait
        end if


    Select Case PlotFlag
        case 0
            print #show.StartMag, "!contents?"
            input #show.StartMag, CountMax$ 
            CountMax = val(CountMax$)
            gosub [DrawData]
        case 1
            print #show.StartMag, "!contents?"
            input #show.StartMag, VarMagMax$
            VarMagMax = val(VarMagMax$)
            MagScaleArray(0) = VarMagMax
            gosub [DrawData]
            if HmagFlag = 1 then
                gosub [DrawHmags]
            end if
        case 2
            print #show.StartMag, "!contents?"
            input #show.StartMag, VarColorMax$
            VarColorMax = val(VarColorMax$)
            MagScaleArray(0) = VarColorMax
            gosub [DrawColorIndicies]
        end select

    gosub [MakeMagArray]
    gosub [DrawMagScale]
wait
'
'=====subroutines
'
[ConvertVarFile]       'extract individual data items from each indexed data line

    open "IREX.txt" for append as #IREX
        print #IREX, "Variable star file data, output from [Convert_VarFile]"

    For VarIndex = 1 to VarIndexMax
        if len(VarData$(VarIndex)) < 30 then                       'delete any extra junk at end of file
            VarIndexMax = VarIndex-1
            exit for
        end if
        VarItem$(VarIndex,1)  = mid$(VarData$(VarIndex),1,2)        'UT day
        VarItem$(VarIndex,2)  = mid$(VarData$(VarIndex),4,2)        'UT month
        VarItem$(VarIndex,3)  = mid$(VarData$(VarIndex),7,4)        'UT year
        VarItem$(VarIndex,4)  = mid$(VarData$(VarIndex),13,2)       'UT hour
        VarItem$(VarIndex,5)  = mid$(VarData$(VarIndex),16,2)       'UT minute
        VarItem$(VarIndex,6)  = mid$(VarData$(VarIndex),19,2)       'UT second
        VarItem$(VarIndex,7)  = mid$(VarData$(VarIndex),21,9)       'Julian date
        VarItem$(VarIndex,8)  = mid$(VarData$(VarIndex),33,6)       'J magnitude
        VarItem$(VarIndex,9)  = mid$(VarData$(VarIndex),40,5)       'J standard error
        VarItem$(VarIndex,10) = mid$(VarData$(VarIndex),47,6)       'J-H
        VarItem$(VarIndex,11) = mid$(VarData$(VarIndex),54,5)       'J-H standard error

        Print #IREX,VarIndex;"  ";_
            VarItem$(VarIndex,1);" ";_
            VarItem$(VarIndex,2);" ";_
            VarItem$(VarIndex,3);" ";_
            VarItem$(VarIndex,4);" ";_
            VarItem$(VarIndex,5);" ";_
            VarItem$(VarIndex,6);" ";_
            VarItem$(VarIndex,7);" ";_
            VarItem$(VarIndex,8);" ";_
            VarItem$(VarIndex,9);" ";_
            VarItem$(VarIndex,10);" ";_
            VarItem$(VarIndex,11)
    next

    StartJ2000$ = VarItem$(1,7)
    StartJ2000  = val(VarItem$(1,7))

    print #IREX, " "
    close #IREX

    For I = 0 to LabelCounter
        print #show.graphicbox1, "delsegment ";DrawLabel(I)
    next
    LabelCounter = 0

    print #show.graphicbox1, "redraw"

    CursorID$ = "A"
return
'
[PlotJ]
    if DataFile$ = "" then
        notice "open a variable star data file first"
        wait
    end if

    #show.TimeScale "selectionindex? TimeScaleIndex"
    #show.Magnitude "selectionindex? MagScaleIndex"
    if TimeScaleIndex = 0 OR MagScaleIndex = 0 then
        notice "select the proper time and magnitude scale"
        wait
    end if

    gosub [TimeScaleSelect]

    StartTime$ = StartJ2000$
    gosub [DrawTimeScale]

    gosub [FindMagRange]

    gosub [DrawMagScale]

    gosub [DrawTitles]

    gosub [DrawData]

    print #show.MoveTimeLeft, "!enable"
    print #show.MoveTimeLeftFast, "!enable"
    print #show.MoveTimeRight, "!enable"
    print #show.MoveTimeRightFast, "!enable"
    print #show.EnterDate, "!enable"

    print #show.MoveMagUp, "!enable"
    print #show.MoveMagDown, "!enable"
    print #show.EnterMag, "!enable"

    HmagFlag = 0

    if val(VarItem$(1,10)) <> 0 then
        print #show.PlotH, "!enable"
    end if

    CursorFlag = 1
return
'
[PlotColorIndex]
    print #show.graphicbox1, "delsegment ";DrawDataSegment
    print #show.graphicbox1, "delsegment ";DrawJSegment
    print #show.graphicbox1, "delsegment ";DrawHSegment

    if DataFile$ = "" then
        notice "open a variable star data file first"
        wait
    end if

    #show.TimeScale "selectionindex? TimeScaleIndex"
    #show.Magnitude "selectionindex? MagScaleIndex"
    if TimeScaleIndex = 0 OR MagScaleIndex = 0 then
        notice "select the proper time and magnitude scale"
        wait
    end if

    gosub [TimeScaleSelect]

    StartTime$ = StartJ2000$
    gosub [DrawTimeScale]

    gosub [FindMagRange]

    gosub [DrawMagScale]

    gosub [DrawTitles]

    gosub [DrawJHindex]

    print #show.MoveTimeLeft, "!enable"
    print #show.MoveTimeLeftFast, "!enable"
    print #show.MoveTimeRight, "!enable"
    print #show.MoveTimeRightFast, "!enable"
    print #show.EnterDate, "!enable"

    print #show.MoveMagUp, "!enable"
    print #show.MoveMagDown, "!enable"
    print #show.EnterMag, "!enable"

    print #show.PlotH, "!disable"

    JHindexFlag = 1

    if val(VarItem$(1,10)) <> 0 then
        print #show.PlotIndex, "!enable"
    end if

    CursorFlag = 1
return
'
[Find_File_Name]        'seperate out filename and extension from info() path/filename
    FileNameIndex = len(DataFile$)
    FileNameLength = len(DataFile$)
    while mid$(DataFile$, FileNameIndex,1)<>"\"          'look for the last backlash
        FileNameIndex = FileNameIndex - 1
    wend
    FileNamePath$ = left$(DataFile$, FileNameIndex)
    DataFileName$ = right$(DataFile$, FileNameLength-FileNameIndex)

    print #show.FileName, DataFileName$                 'display filename in "File" textbox
return
'
[TimeScaleSelect]       'establish the parameters for the time scale
    open "IREX.txt" for append as #IREX
        print #IREX, "Time Scale Array"
        print #IREX, " "

    Select Case
        Case TimeScale = 60
            TimeScaleArray(0) = 0
            For I = 0 to 9
                TimeScaleArray(I) =  10 * I
                TimeScaleArray$(I) = using("###", TimeScaleArray(I))
            Next
            TimeDiv$ = "1 div = 1 minute"
            TimeScaleJ2000 = 0.00069444
        Case TimeScale = 120
            TimeScaleArray(0) = 0
            For I = 0 to 9
                TimeScaleArray(I) =  20 * I
                TimeScaleArray$(I) = using("###", TimeScaleArray(I))
            Next
            TimeDiv$ = "1 div = 2 minute"
            TimeScaleJ2000 = 0.00138888
        Case TimeScale = 300
            TimeScaleArray(0) = 0
            For I = 0 to 9
                TimeScaleArray(I) = 50 * I
                TimeScaleArray$(I) = using("###", TimeScaleArray(I))
            Next
            TimeDiv$ = "1 div = 5 minute"
            TimeScaleJ2000 = 0.0034722
        Case TimeScale = 600
            TimeScaleArray(0) = 0
            For I = 0 to 9
                TimeScaleArray(I) =  100 * I
                TimeScaleArray$(I) = using("###", TimeScaleArray(I))
            Next
            TimeDiv$ = "1 div = 10 minutes"
            TimeScaleJ2000 = 0.0069444
        Case TimeScale = 3600
            TimeScaleArray(0) = 0
            For I = 0 to 9
                TimeScaleArray(I) =  10 * I
                TimeScaleArray$(I) = using("###", TimeScaleArray(I))
            Next
            TimeDiv$ = "1 div = 1 hour"
            TimeScaleJ2000 = 0.0416667
        Case TimeScale = 7200
            TimeScaleArray(0) = 0
            For I = 0 to 9
                TimeScaleArray(I) =  20 * I
                TimeScaleArray$(I) = using("###", TimeScaleArray(I))
            Next
            TimeDiv$ = "1 div = 2 hour"
            TimeScaleJ2000 = 0.0833334
        Case TimeScale = 18000
            TimeScaleArray(0) = 0
            For I = 0 to 9
                TimeScaleArray(I) =  50 * I
                TimeScaleArray$(I) = using("###", TimeScaleArray(I))
            Next
            TimeDiv$ = "1 div = 5 hours"
            TimeScaleJ2000 = 0.2083335
        Case TimeScale = 36000
            TimeScaleArray(0) = 0
            For I = 0 to 9
                TimeScaleArray(I) =  100 * I
                TimeScaleArray$(I) = using("###", TimeScaleArray(I))
            Next
            TimeDiv$ = "1 div = 10 hours"
            TimeScaleJ2000 = 0.416667
        Case TimeScale = 86400
            TimeScaleArray(0) = 0
            For I = 0 to 9
                TimeScaleArray(I) =  10 * I
                TimeScaleArray$(I) = using("###", TimeScaleArray(I))
            Next
            TimeDiv$ = "1 div = 1 day"
            TimeScaleJ2000 = 1.00000
        Case TimeScale = 172800
            TimeScaleArray(0) = 0
            For I = 0 to 9
                TimeScaleArray(I) =  20 * I
                TimeScaleArray$(I) = using("###", TimeScaleArray(I))
            Next
            TimeDiv$ = "1 div = 2 day"
            TimeScaleJ2000 = 2.00000
        Case TimeScale = 432000
            TimeScaleArray(0) = 0
            For I = 0 to 9
                TimeScaleArray(I) =  50 * I
                TimeScaleArray$(I) = using("###", TimeScaleArray(I))
            Next
            TimeDiv$ = "1 div = 5 days"
            TimeScaleJ2000 = 5.00000
        Case TimeScale = 864000
            TimeScaleArray(0) = 0
            For I = 0 to 9
                TimeScaleArray(I) =  100 * I
                TimeScaleArray$(I) = using("###", TimeScaleArray(I))
            Next
            TimeDiv$ = "1 div = 10 days"
            TimeScaleJ2000 = 10.00000
        Case TimeScale = 1728000
            TimeScaleArray(0) = 0
            For I = 0 to 9
                TimeScaleArray(I) =  200 * I
                TimeScaleArray$(I) = using("####", TimeScaleArray(I))
            Next
            TimeDiv$ = "1 div = 20 days"
            TimeScaleJ2000 = 20.00000
    End Select

    Print #IREX,TimeScaleArray$(0);" ";_
                TimeScaleArray$(1);" ";_
                TimeScaleArray$(2);" ";_
                TimeScaleArray$(3);" ";_
                TimeScaleArray$(4);" ";_
                TimeScaleArray$(5);" ";_
                TimeScaleArray$(6);" ";_
                TimeScaleArray$(7);" ";_
                TimeScaleArray$(8);" ";_
                TimeScaleArray$(9)

    print #IREX," "
    close #IREX
return
'
[FindMagRange]
    open "IREX.txt" for append as #IREX

    VarMagMax = val(VarItem$(1,8))              'find the brightest J magnitude
    For VarIndex = 2 to VarIndexMax
        if val(VarItem$(VarIndex,8)) < VarMagMax then
            VarMagMax = val(VarItem$(VarIndex,8))
            VarMagMax$ = VarItem$(VarIndex,8)
        end if
    Next

    print #IREX, "VarMagMax ";VarMagMax
    print #IREX," "
    close #IREX


    Select Case MagScale
        Case 0.01
            MagScaleArray(0) = val(using("###.##",VarMagMax)) - 0.01
            VarColorMax = 0.05
        Case 0.02
            MagScaleArray(0) = val(using("###.##",VarMagMax)) - 0.02
            VarColorMax = 0.10
        Case 0.05
            VarMagMax = val(using("###.##",VarMagMax)) - 0.05
            MagScaleArray(0) = VarMagMax
            VarColorMax = 0.25
        Case 0.10
            VarMagMax = val(using("###.#",VarMagMax)) - 0.1
            MagScaleArray(0) = VarMagMax
            VarColorMax = 0.50
        Case 0.20
            VarMagMax = val(using("###.#",VarMagMax)) - 0.2
            MagScaleArray(0) = VarMagMax
            VarColorMax = 1.00
        Case 0.50
            VarMagMax = val(using("###.#",VarMagMax)) - 0.5
            MagScaleArray(0) = VarMagMax
            VarColorMax = 2.50
        Case 1.00
            VarMagMax = val(using("###",VarMagMax)) - 1.0
            MagScaleArray(0) = VarMagMax
            VarColorMax = 5.00
    End Select

    gosub [MakeMagArray]
return
'
[MakeMagArray]
    Select Case PlotFlag
        Case 0
            MagScaleArray(0) = CountMax
            For I = 1 to 20
                MagScaleArray(I) = MagScaleArray(0) - (MagScale * I)
                if MagScaleArray(I) < 0 then
                    MagScaleArray(I) = 0
                end if
            Next
        Case 1
 '           MagScaleArray(0) = VarMagMax
            For I = 1 to 20
                MagScaleArray(I) = MagScaleArray(0) + (MagScale * I)
            Next
        Case 2
            MagScaleArray(0) = VarColorMax
            For I = 1 to 20
                MagScaleArray(I) = MagScaleArray(0) - (MagScale * I)
            Next
     end select

    open "IREX.txt" for append as #IREX
    print #IREX, "MakeMagArray"
    print #IREX,MagScaleArray(0);" ";_
                MagScaleArray(1);" ";_
                MagScaleArray(2);" ";_
                MagScaleArray(3);" ";_
                MagScaleArray(4);" ";_
                MagScaleArray(5);" ";_
                MagScaleArray(6);" ";_
                MagScaleArray(7);" ";_
                MagScaleArray(8);" ";_
                MagScaleArray(9);" ";_
                MagScaleArray(10);" ";_
                MagScaleArray(11);" ";_
                MagScaleArray(12);" ";_
                MagScaleArray(13);" ";_
                MagScaleArray(14);" ";_
                MagScaleArray(15);" ";_
                MagScaleArray(16);" ";_
                MagScaleArray(17);" ";_
                MagScaleArray(18);" ";_
                MagScaleArray(19)

    print #IREX," "
    close #IREX
return
'
[DrawHmags]
    if HmagFlag = 1 then
        gosub [DrawH]
    end if
return
'
[DrawColorIndicies]
    if JHindexFlag = 1 then
        gosub [DrawJHindex]
    end if
return
'
'=====subroutines for calculations
'
[FindJ2000]    'convert UT time and date to Julian, epcoh J2000

                                'A = int(UTyear/100)
        A = int (UTyear/100)
        B = 2 - A + int(A/4)
                                'C = int(365.25 * UTyear)
        C = int(365.25 * UTyear)
                                'D = int(30.6001 *(UTmonth + 1))
        D = int(30.6001 * UTmonth + 1)
                                'JD = B + C + D - 730550.5 + UTday + (UThours + UTmin/60 + UTsec/3600)/24
        JD = B + C + D - 730550.5 + UTday + (UThours + UTmin/60 + UTsec/3600)/24
                                'Julian century
        JT = JD/36525
return
'
[FindStartTime]     'convert StartTime in seconds to time in 12:12:12.123 format

    HR = int(StartTime / 3600)
    MIN = int((StartTime MOD 3600) / 60)
    SEC = StartTime - 3600 * HR - 60 * MIN

    if HR >= 24 then
        HR = HR - 24
    end if

    HR$ = using("##",HR)
    MIN$ = using("##",MIN)
    SEC$ = using("##.###",SEC)
    StartTime$ = HR$+":"+MIN$+":"+SEC$

    open "IREX.txt" for append as #IREX
    print #IREX, "StartTime = ";StartTime;"  HR = ";HR;"  MIN = ";MIN;"  SEC = ";SEC
    print #IREX, ""
    close #IREX
return
'
'=====graphics routines
'
[Draw_Graph_Outline]
    print #show.graphicbox1, "color black"
    print #show.graphicbox1, "backcolor white"

    print #show.graphicbox1, "line 40 20 40 520"
    print #show.graphicbox1, "line 990 20 990 520"
    print #show.graphicbox1, "line 40 520 990 520"
    print #show.graphicbox1, "line 40 20 990 20"

    for I = 1 to 20
        yD =  25 * I - 5
        print #show.graphicbox1, "line 40 ";yD;" 44 ";yD
        print #show.graphicbox1, "line 990 ";yD;" 986 ";yD
    next
    for I = 1 to 94
        xD = 40 + 10 * I
        print #show.graphicbox1, "line ";xD;" 520 ";xD;" 516"
        print #show.graphicbox1, "line ";xD;" 20 ";xD;" 24"
    next
    for I = 0 to 19
        xD = 40 + 100 * I
        print #show.graphicbox1, "line ";xD;" 520 ";xD;" 510"
        print #show.graphicbox1, "line ";xD;" 20 ";xD;" 30"
    next
    for I = 0 to 19
        xD = 90 + 100 * I
        print #show.graphicbox1, "line ";xD;" 520 ";xD;" 513"
        print #show.graphicbox1, "line ";xD;" 20 ";xD;" 27"
    next
    print #show.graphicbox1, "flush"
return
'
[DrawCursor]

    if (MouseX < 40) OR (MouseX > 990) OR (MouseY < 20) OR (MouseY > 520) then
        notice "pointer out of plot area"
        wait
    end if

    Select Case CursorID$
        Case "A"
            print #show.graphicbox1, "delsegment ";CursorSegmentA
            print #show.graphicbox1, "redraw"
            print #show.graphicbox1, "color green"
            print #show.graphicbox1, "line 40 ";MouseY;" 990 ";MouseY
            print #show.graphicbox1, "line ";MouseX;" 20 ";MouseX;" 520"
            print #show.graphicbox1, "Segment CursorSegmentA"
        Case "B"
            print #show.graphicbox1, "delsegment ";CursorSegmentB
            print #show.graphicbox1, "redraw"
            print #show.graphicbox1, "color cyan"
            print #show.graphicbox1, "line 40 ";MouseY;" 990 ";MouseY
            print #show.graphicbox1, "line ";MouseX;" 20 ";MouseX;" 520"
            print #show.graphicbox1, "Segment CursorSegmentB"
        Case "C"
            print #show.graphicbox1, "delsegment ";CursorSegmentC
            print #show.graphicbox1, "redraw"
            print #show.graphicbox1, "color black"
            print #show.graphicbox1, "line ";MouseX;" 20 ";MouseX;" 520"
            print #show.graphicbox1, "Segment CursorSegmentC"
    End Select

    print #show.graphicbox1, "flush"
    print #show.graphicbox1, "color black"
return
'
[DrawCursorValues.A]
    print #show.graphicbox1, "delsegment ";CursorValuesSegment.A
    print #show.graphicbox1, "redraw"
    print #show.graphicbox1, "color darkgreen"
    print #show.graphicbox1, "font arial 7 12"

    if PlotFlag = 1 OR PlotFlag = 2 then
        print #show.graphicbox1, "place 170 10"
        print #show.graphicbox1, "\ Cursor A:"

        if JDFlag = 1 then
            print #show.graphicbox1, "place 230 10"
            print #show.graphicbox1, "\ JD"+CursorTime.A$
        else
            print #show.graphicbox1, "place 230 10"
            print #show.graphicbox1, "\ HJD"+CursorTime.A$
        end if

        print #show.graphicbox1, "place 330 10"
        print #show.graphicbox1, "\ Mag. "+CursorMag.A$
    else
        print #show.graphicbox1, "place 200 10"
        print #show.graphicbox1, "\ Cursor A:"

        print #show.graphicbox1, "place 260 10"
        print #show.graphicbox1, "\ UT"+CursorTime.A$

        print #show.graphicbox1, "place 365 10"
        print #show.graphicbox1, "\ Count "+CursorMag.A$
    end if

    print #show.graphicbox1, "Segment CursorValuesSegment.A"
    print #show.graphicbox1, "flush"
return
'
[DrawCursorValues.B]
    print #show.graphicbox1, "delsegment ";CursorValuesSegment.B
    print #show.graphicbox1, "redraw"
    print #show.graphicbox1, "color darkcyan"
    print #show.graphicbox1, "font arial 7 12"

    if PlotFlag = 1 OR PlotFlag = 2 then
        print #show.graphicbox1, "place 440 10"
        print #show.graphicbox1, "\ Cursor B:"

        if JDFlag = 1 then
            print #show.graphicbox1, "place 500 10"
            print #show.graphicbox1, "\ JD"+CursorTime.B$
        else
            print #show.graphicbox1, "place 500 10"
            print #show.graphicbox1, "\ HJD"+CursorTime.B$ 
        end if

        print #show.graphicbox1, "place 600 10"
        print #show.graphicbox1, "\ Mag. "+CursorMag.B$
    else
        print #show.graphicbox1, "place 455 10"
        print #show.graphicbox1, "\ Cursor B:"

        print #show.graphicbox1, "place 515 10"
        print #show.graphicbox1, "\ UT"+CursorTime.B$

        print #show.graphicbox1, "place 620 10"
        print #show.graphicbox1, "\ Count "+CursorMag.B$
    end if

    print #show.graphicbox1, "Segment CursorValuesSegment.B"
    print #show.graphicbox1, "flush"
return
'
[DrawCursorValues.C]
    print #show.graphicbox1, "delsegment ";CursorValuesSegment.C
    print #show.graphicbox1, "color black"
    print #show.graphicbox1, "backcolor white"
    print #show.graphicbox1, "redraw"
    print #show.graphicbox1, "font arial 8 12"

    print #show.graphicbox1, "place ";MouseX-40;" ";297
    print #show.graphicbox1, "\ "+CursorTime$

    print #show.graphicbox1, "Segment CursorValuesSegment.C"
    print #show.graphicbox1, "flush"
    print #show.graphicbox1, "color black"
    print #show.graphicbox1, "backcolor white"
return
'
[DrawCursorDiffValues]
    print #show.graphicbox1, "delsegment ";CursorDiffSegment
    print #show.graphicbox1, "redraw"
    print #show.graphicbox1, "color black"
    print #show.graphicbox1, "font arial 7 12"


    print #show.graphicbox1, "place 710 10"
    print #show.graphicbox1, "\ B-A: "

    if PlotFlag = 1 OR PlotFlag = 2 then
        print #show.graphicbox1, "place 740 10"
        if Sign = 1 then
            print #show.graphicbox1, "\ Days -"+CursorTimeDiff$
        else
            print #show.graphicbox1, "\ Days "+CursorTimeDiff$ 
        end if

        print #show.graphicbox1, "place 840 10"
        print #show.graphicbox1, "\ Mag. Diff. "+CursorMagDiff$ 
    else
        print #show.graphicbox1, "place 745 10"
        if Sign = 1 then
            print #show.graphicbox1, "\ Time -"+CursorTimeDiff$
        else
            print #show.graphicbox1, "\ Time "+CursorTimeDiff$
        end if

        print #show.graphicbox1, "place 850 10"
        print #show.graphicbox1, "\ Count Diff. "+CursorMagDiff$ 
    end if

    print #show.graphicbox1, "Segment CursorDiffSegment"
    print #show.graphicbox1, "flush"
return
'
[DrawTimeScale]
    print #show.graphicbox1, "color black"
    print #show.graphicbox1, "backcolor white"

    print #show.graphicbox1, "delsegment ";TimeScaleSegment
    print #show.graphicbox1, "redraw"
    print #show.graphicbox1, "font arial 6 12"

    For I = 0 to 9
        xD = 30 + 100 * I
        print #show.graphicbox1, "place ";xD;" ";535
        print #show.graphicbox1, "\ "+TimeScaleArray$(I)
    next

    if PlotFlag = 1 OR PlotFlag = 2 then
        print #show.graphicbox1, "place 10 545"
        if JDFlag = 1 then
            print #show.graphicbox1, "\ JD"+StartTime$
        else
            print #show.graphicbox1, "\ HJD"+StartTime$
        end if
        print #show.graphicbox1, "place 900 545"
        print #show.graphicbox1, "\ "+TimeDiv$
    else
        print #show.graphicbox1, "place 5 545"
        print #show.graphicbox1, "\ UT"+StartTime$
        print #show.graphicbox1, "place 900 545"
        print #show.graphicbox1, "\ "+TimeDiv$ 
    end if

    print #show.graphicbox1, "Segment TimeScaleSegment"
    print #show.graphicbox1, "flush"
return
'
[DrawMagScale]
    print #show.graphicbox1, "color black"
    print #show.graphicbox1, "backcolor white"
    print #show.graphicbox1, "delsegment ";MagScaleSegment
    print #show.graphicbox1, "redraw"
    print #show.graphicbox1, "font arial 6 12"

    for I = 0 to 20
        yD =  25 * I + 23

        if PlotFlag = 1 OR PlotFlag = 2 then
            print #show.graphicbox1, "place 5 ";yD
            MagScaleArray$(I) = using ("###.##", MagScaleArray(I))
        else
            print #show.graphicbox1, "place 7 ";yD
            MagScaleArray$(I) = using ("#####", MagScaleArray(I))
        end if
        print #show.graphicbox1, "\ "+MagScaleArray$(I)
    next

    Select Case PlotFlag
        case 0
            print #show.graphicbox1, "place 4 10"
            print #show.graphicbox1, "\ Counts"
        case 1
            print #show.graphicbox1, "place 2 10"
            print #show.graphicbox1, "\ Magnitude"
        case 2
            print #show.graphicbox1, "place 2 10"
            print #show.graphicbox1, "\ Color Index"
        end select

    print #show.graphicbox1, "Segment MagScaleSegment"
    print #show.graphicbox1, "flush"
return
'
[DrawTitles]
    print #show.graphicbox1, "color black"
    print #show.graphicbox1, "backcolor white"

    print #show.graphicbox1, "delsegment ";DrawTitle
    print #show.graphicbox1, "redraw"

    print #show.graphicbox1, "font arial 10 12"
    print #show.graphicbox1, "place 460 560"
    print #show.graphicbox1, "\ "+PlotFileName$

    print #show.graphicbox1, "font arial 6 12"
    print #show.graphicbox1, "place 5 560"
    print #show.graphicbox1, "\ File info: "+FileHeader$ 

    print #show.graphicbox1, "Segment DrawTitle"
    print #show.graphicbox1, "flush"
    print #show.graphicbox1, "color black"
    print #show.graphicbox1, "backcolor white"
return
'
[DrawLabel]
    print #show.graphicbox1, "color black"
    print #show.graphicbox1, "backcolor white"
    print #show.graphicbox1, "redraw"

    print #show.graphicbox1, "font arial 8 12"
    print #show.graphicbox1, "place ";MouseX;" ";MouseY
    print #show.graphicbox1, "\ "+Label$

    DrawLabel = DrawLabel(LabelCounter)
    print #show.graphicbox1, "Segment DrawLabel"
    LabelCounter = LabelCounter + 1
    DrawLabel(LabelCounter) = DrawLabel

    print #show.graphicbox1, "flush"
    print #show.graphicbox1, "color black"
    print #show.graphicbox1, "backcolor white"
return
'
[DrawData]
    open "IREX.txt" for append as #IREX
    print #IREX, "output from [DrawData]"
    print #IREX, " "

    print #show.graphicbox1, "delsegment ";DrawDataSegment
    print #show.graphicbox1, "redraw"

    if PlotFlag = 1 then                'draw plot for variable star data
        print #show.graphicbox1, "color red"
        print #show.graphicbox1, "backcolor red"

        PixelTime = 10/TimeScaleJ2000
        PixelMag = 25/MagScale

        print #IREX,"PixelTime ";PixelTime
        print #IREX,"PixelMag  ";PixelMag
        print #IREX," "
        print #IREX, "VarIndex  ";"VarMagDiff  ";"   yD     ";"VarTimeDiff ";"   xD   ";"    VarError"

        for VarIndex = 1 to VarIndexMax

            If (ConnectDotFlag = 1) AND (VarIndex > 1) then
                xDlast = xD
                yDlast = yD
            end if

            VarMagDiff = val(VarItem$(VarIndex,8)) - MagScaleArray(0)
            yD = VarMagDiff * PixelMag + 20

            VarError = val(VarItem$(VarIndex,9))

            VarTimeDiff = val(VarItem$(VarIndex,7)) - val(StartJ2000$)
            xD = VarTimeDiff * PixelTime + 40

            If (ConnectDotFlag = 1) AND (VarIndex > 1) AND (xDlast >= 40) AND (xD >= 40) then
                print #show.graphicbox1,"line ";xDlast;" ";yDlast;" ";xD;" ";yD
            end if

            print #IREX, using("###",VarIndex)+"     "+_
                         using("#####.##",VarMagDiff)+"      "+_
                         using("####.##",yD)+"  "+_
                         using("#####.####",VarTimeDiff)+"  "+_
                         using("######.###",xD)+"  "+_
                         using("####.###",VarError)

            if xD >= 40  then
                print #show.graphicbox1, "place ";xD;" ";yD
                print #show.graphicbox1, "circlefilled 2"
            end if

            if PlotErrorFlag = 1 then
                VarErrorReduced = VarError * PixelMag * 0.5
                yEup = yD + VarErrorReduced
                yEdown = yD - VarErrorReduced
                print #show.graphicbox1, "line ";xD;" ";yEdown;" ";xD;" ";yEup
            end if
        next
    end if

    print #show.graphicbox1, "Segment DrawDataSegment"
    print #show.graphicbox1, "flush"
    print #show.graphicbox1, "color black"
    print #show.graphicbox1, "backcolor white"

    print #show.graphicbox1, "delsegment ";DrawHSegment
    print #show.graphicbox1, "redraw"

    print #IREX, " "
    close #IREX
return
'
[DrawJHindex]
    open "IREX.txt" for append as #IREX
    print #IREX, "output from [DrawJHindex]"
    print #IREX, " "

    print #show.graphicbox1, "delsegment ";DrawJHSegment
    print #show.graphicbox1, "redraw"

    print #show.graphicbox1, "font arial 6 12"
    print #show.graphicbox1, "color black"
    print #show.graphicbox1, "backcolor black"

    print #IREX, "VarIndex  ";"VarMagDiff  ";"   yD     ";"VarTimeDiff ";"   xD   ";"    VarError"

    PixelTime = 10/TimeScaleJ2000
    PixelMag = 25/MagScale

    for VarIndex = 1 to VarIndexMax

        If (ConnectDotFlag = 1) AND (VarIndex > 1) then
            xDlast = xD
            yDlast = yD
        end if

        VarMagDiff = VarColorMax - val(VarItem$(VarIndex,10))
        yD = VarMagDiff * PixelMag + 20

        VarTimeDiff = val(VarItem$(VarIndex,7)) - val(StartJ2000$)
        xD = VarTimeDiff * PixelTime + 40

        VarError = val(VarItem$(VarIndex,11))

        print #IREX, using("###",VarIndex)+"     "+_
                     using("#####.##",VarMagDiff)+"      "+_
                     using("####.##",yD)+"  "+_
                     using("#####.####",VarTimeDiff)+"  "+_
                     using("######.###",xD)+"  "+_
                     using("####.###",VarError)

        If (ConnectDotFlag = 1) AND (VarIndex > 1) AND (xDlast >= 40) AND (xD >= 40) then
            print #show.graphicbox1,"line ";xDlast;" ";yDlast;" ";xD;" ";yD
        end if

        print #show.graphicbox1, "place ";xD;" ";yD
        print #show.graphicbox1, "circlefilled 2"

        if PlotErrorFlag = 1 then
            VarErrorReduced = VarError * PixelMag * 0.5
            yEup = yD + VarErrorReduced
            yEdown = yD - VarErrorReduced
            print #show.graphicbox1, "line ";xD;" ";yEdown;" ";xD;" ";yEup
        end if

    next

    print #show.graphicbox1, "Segment DrawJHSegment"
    print #show.graphicbox1, "flush"
    print #show.graphicbox1, "color black"
    print #show.graphicbox1, "backcolor white"

    JHindexFlag = 1

    print #IREX, " "
    close #IREX
return
'
[DrawH]
    open "IREX.txt" for append as #IREX
    print #IREX, "output from [DrawH]"
    print #IREX, " "

    print #show.graphicbox1, "delsegment ";DrawHSegment
    print #show.graphicbox1, "redraw"

    print #show.graphicbox1, "font arial 6 12"
    print #show.graphicbox1, "color darkred"
    print #show.graphicbox1, "backcolor darkred"

    print #IREX, "VarIndex  ";"VarMagDiff  ";"   yD     ";"VarTimeDiff ";"   xD   "

    for VarIndex = 1 to VarIndexMax

        If (ConnectDotFlag = 1) AND (VarIndex > 1) then
            xDlast = xD
            yDlast = yD
        end if
                                            'H = J - (J-H)
        VarMagDiff = val(VarItem$(VarIndex,8)) - val(VarItem$(VarIndex,10)) - MagScaleArray(0)

        yD = VarMagDiff * PixelMag + 20

        VarTimeDiff = val(VarItem$(VarIndex,7)) - val(StartJ2000$)
        xD = VarTimeDiff * PixelTime + 40


        print #IREX, using("###",VarIndex)+"     "+_
                     using("#####.##",VarMagDiff)+"      "+_
                     using("####.##",yD)+"  "+_
                     using("#####.####",VarTimeDiff)+"  "+_
                     using("######.###",xD)

        If (ConnectDotFlag = 1) AND (VarIndex > 1) AND (xDlast >= 40) AND (xD >= 40) then
            print #show.graphicbox1,"line ";xDlast;" ";yDlast;" ";xD;" ";yD
        end if

        print #show.graphicbox1, "place ";xD;" ";yD
        print #show.graphicbox1, "circlefilled 2"
    next

    print #show.graphicbox1, "Segment DrawHSegment"
    print #show.graphicbox1, "flush"
    print #show.graphicbox1, "color black"
    print #show.graphicbox1, "backcolor white"

    HmagFlag = 1

    print #IREX, " "
    close #IREX
return
'
'=====sub routines
'
[SAVE_PPparms]
    open "PPparms_JH.txt" for output as #PPparms
        print #PPparms, Location$                       'latitude and longitude in degrees
        print #PPparms, using("#.###",KJ)               'first order extinction for J
        print #PPparms, using("#.###",KH)               'first order extinction for H
        print #PPparms, using("#.###",KKjh)             'second order extinction for j-h, default = 0
        print #PPparms, using("#.###",Rho)              'transformation coeff. for J using J-H
        print #PPparms, using("#.###",Lambda)           'transformation coeff. J-H
        print #PPparms, JDFlag                          '1 if using JD and 0 if using HJD
        print #PPparms, OBSCODE$                        'AAVSO observatory code
        print #PPparms, Spare2                          '0 not used at this time
        print #PPparms, using("##.###",ZPj)             'zero-point constant for j
        print #PPparms, using("##.###",ZPjh)            'zero-point constant for j-h
        print #PPparms, using("##.###",Ej)              'standard error for j
        print #PPparms, using("##.###",Ejh)             'standard error for j-h
    close #PPparms
return
'
SUB Pause mil
    t = time$("milliseconds")
    endtime = t + mil
    if endtime > 86400000 then
        endtime = endtime - 86400000
        while time$("milliseconds") > t
            calldll #kernel32, "Sleep",1 as ulong,r as void
        wend
        while time$("milliseconds") < endtime
            calldll #kernel32, "Sleep",1 as ulong,r as void
        wend
    else
        while time$("milliseconds") < endtime and time$("milliseconds") >= t
            calldll #kernel32, "Sleep",1 as ulong,r as void
        wend
    end if
END SUB
'
'================Make AAVSI data file
'
[AAVSO]
    NOMAINWIN
    WindowWidth = 350 : WindowHeight = 270
    UpperLeftX = INT((DisplayWidth-WindowWidth)/2)
    UpperLeftY = INT((DisplayHeight-WindowHeight)/2)
    Menu        #export, "File", "Quit", [QuitExport]

        statictext #export.varName, "variable name", 140,22,130,20
        textbox    #export.var, 10,15,120,22
        statictext #export.compName, "comparison name", 140,47,150,20
        textbox    #export.comp, 10,45,120,22
        statictext #export.obscodeName, "AAVSO observatory code", 140,77,180,20
        textbox    #export.obscode, 10,75,120,22
        statictext #export.chartName, "chart name", 140,107,130,20
        textbox    #export.chart, 10,105,120,22

        checkbox   #export.JD, "JD", [setJD], [resetJD], 10,135,35,25
        checkbox   #export.HJD, "HJD", [setHJD], [resetHJD], 80,135,45,25
        statictext #export.JDName, "Julian date type", 140,137,130,20

        button #export.save, "Make/Save AAVSO Data", [MakeAAVSOdata], UL, 85,170,180,30

    Open "Export data in AAVSO extended format" for Window as #export
    #export "trapclose [QuitExport]"
    #export "font ariel 10"

        if PlotFileName$ = ""  then
            notice "open a var data file "
        end if

        if VarStar$ = "" then
            print #export.var, PlotFileName$        'print default filename in text box
        else
            print #export.var, VarStar$             'print star names from file header
            print #export.comp, CompStar$
        end if

        print #export.chart, "na"                   'print default "na" in text box

        AAVSOFlag = 1

        if JDFlag = 1 then                          'check PPparms value of JDFlag
            print #export.JD, "set"
            print #export.HJD, "reset"
        else
            print #export.HJD, "set"
            print #export.JD, "reset"
            JDFlag = 0
        end if

        if JDFlag$ = "1" then                       'check filed header value for JDFlag
            print #export.JD, "set"
            print #export.HJD, "reset"
            JDFlag = 1
        end if
        if JDFlag$ = "0" then
            print #export.HJD, "set"
            print #export.JD, "reset"
            JDFlag = 0
        end if

        print #export.obscode, OBSCODE$

    wait

    [setJD]
        JDFlag = 1
        print #export.HJD, "reset"
    wait
    [setHJD]
        JDFlag = 0
        print #export.JD, "reset"
    wait
    [resetJD]
        JDFlag = 0
        print #export.HJD, "set"
    wait
    [resetHJD]
        JDFlag = 1
        print #export.JD, "set"
    wait

    [MakeAAVSOdata]

        print #export.var, "!contents? VAR$";
            VAR$ = upper$(VAR$)
        print #export.comp, "!contents? COMP$";
            COMP$ = upper$(COMP$)

        print #export.obscode, "!contents? OBSCODEnew$";
        OBSCODEnew$ = upper$(OBSCODEnew$)
        if OBSCODEnew$ <> OBSCODE$ then
            OBSCODE$ = OBSCODEnew$
            gosub [SAVE_PPparms]
        end if


        print #export.chart, "!contents? CHART$";

        AAVSO$(1) = "#TYPE=EXTENDED"
        AAVSO$(2) = "#OBSCODE="+OBSCODE$
        AAVSO$(3) = "#SOFTWARE=OPTEC SSP4DATAQ2"
        AAVSO$(4) = "#DELIM=,"
        if JDFlag = 1 then
            AAVSO$(5) = "#DATE=JD"
        else
            AAVSO$(5) = "#DATE=HJD"
        end if
        AAVSO$(6) = "#OBSTYPE=PEP"

        Group = 1
        AAVSOIndex = 1
        for VarIndex = 1 to VarIndexMax
            Jflag = 0 : Hflag = 0
                                            'find how many colors for each data line used
            Colors = 0
            if val(VarItem$(VarIndex,8)) <> 0  then
                Colors  = Colors + 1
                Jflag = 1
            end if
            if val(VarItem$(VarIndex,10)) <> 0  then
                Colors  = Colors + 1
                Hflag = 1
            end if

            for J = 1 to Colors
                AAVSOitem$(AAVSOIndex, 1) = VAR$
                                            'calculate JDN from J2000 date
                AAVSOitem$(AAVSOIndex, 2) = using("#######.####", (val(VarItem$(VarIndex, 7)) + 2451545))
                                            'separate out colors from color index and find JH
                if Jflag = 1 then
                    AAVSOitem$(AAVSOIndex, 3) = using("##.###",(val(VarItem$(VarIndex,8))))
                    if val(VarItem$(VarIndex,9)) = 0 then
                        AAVSOitem$(AAVSOIndex, 4) = "na"
                    else
                        AAVSOitem$(AAVSOIndex, 4) = VarItem$(VarIndex,9)
                    end if
                    AAVSOitem$(AAVSOIndex, 5) = "J"
                    Jflag = 0
                    goto [EndColors]
                end if

                if Hflag = 1 then
                    AAVSOitem$(AAVSOIndex, 3) = using("##.###",(val(VarItem$(VarIndex,8)) - val(VarItem$(VarIndex,10))))
                    if val(VarItem$(VarIndex,11)) = 0 then
                        AAVSOitem$(AAVSOIndex, 4) = "na"
                    else
                        AAVSOitem$(AAVSOIndex, 4) = VarItem$(VarIndex,11)
                    end if
                    AAVSOitem$(AAVSOIndex, 5) = "H"
                    Hflag = 0
                    goto [EndColors]
                end if

                [EndColors]

                AAVSOitem$(AAVSOIndex, 6) = "YES"
                AAVSOitem$(AAVSOIndex, 7) = "STD"
                AAVSOitem$(AAVSOIndex, 8) = COMP$
                AAVSOitem$(AAVSOIndex, 9) = "na"
                AAVSOitem$(AAVSOIndex,10) = "na"
                AAVSOitem$(AAVSOIndex,11) = "na"
                AAVSOitem$(AAVSOIndex,12) = "na"
                AAVSOitem$(AAVSOIndex,13) = str$(Group)
                AAVSOitem$(AAVSOIndex,14) = CHART$
                AAVSOitem$(AAVSOIndex,15) = "na"

                AAVSOIndex = AAVSOIndex + 1
            next
            Group = Group + 1
        next
        AAVSOIndexMax = AAVSOIndex - 1

        gosub [SaveExportFile]

    wait

    [SaveExportFile]
        filedialog "Save AAVSO data", PathAAVSO$, AAVSOFile$

        for I = len(AAVSOFile$) to 1 step -1
            if mid$(AAVSOFile$,I,1) = "\" then
                ShortAAVSO$ = mid$(AAVSOFile$,I+1)
                PathAAVSO$ = left$(AAVSOFile$,I)+"*.txt"
                exit for
            end if
        next

        if AAVSOFile$ = "" then [QuitExport]

        files "c:\", AAVSOFile$, info$()
        if val(info$(0,0)) <> 0 then
           confirm "Write over existing file?"; Answer$
               if Answer$ = "no" then [SaveExportFile]
        end if

        if (right$(AAVSOFile$,4) = ".txt") OR (right$(AAVSOFile$,4) = ".TXT") then
            open AAVSOFile$ for Output as #AAVSOFile
        else
            AAVSOFile$ = AAVSOFile$+".txt"
            open AAVSOFile$ for Output as #AAVSOFile
        end if

        print #AAVSOFile, AAVSO$(1)     '"#TYPE=EXTENDED"
        print #AAVSOFile, AAVSO$(2)     '"#OBSCODE="+OBSCODE$
        print #AAVSOFile, AAVSO$(3)     '"#SOFTWARE=OPTEC SSPDATAQ2 V2.21"
        print #AAVSOFile, AAVSO$(4)     '"#DELIM=,"
        print #AAVSOFile, AAVSO$(5)     '"#DATE=JD"
        print #AAVSOFile, AAVSO$(6)     '"#OBSTYPE=PEP"

        for AAVSOIndex = 1 to AAVSOIndexMax

            print #AAVSOFile, AAVSOitem$(AAVSOIndex,1);",";_    'VAR$, variable star name
                              AAVSOitem$(AAVSOIndex,2);",";_    'Julian Day
                              AAVSOitem$(AAVSOIndex,3);",";_    'magnitude
                              AAVSOitem$(AAVSOIndex,4);",";_    'magnitude standard error
                              AAVSOitem$(AAVSOIndex,5);",";_    'filter: JH
                              AAVSOitem$(AAVSOIndex,6);",";_    '"YES", use transformation coefficients
                              AAVSOitem$(AAVSOIndex,7);",";_    '"STD", magnitude type is standard
                              AAVSOitem$(AAVSOIndex,8);",";_    'COMP$, comparison star name
                              AAVSOitem$(AAVSOIndex,9);",";_    '"na", instrument magnitude of comp
                              AAVSOitem$(AAVSOIndex,10);",";_   '"na", name of check star
                              AAVSOitem$(AAVSOIndex,11);",";_   '"na", instrument magnitude of check star
                              AAVSOitem$(AAVSOIndex,12);",";_   '"na", airmass
                              AAVSOitem$(AAVSOIndex,13);",";_   '"Group$, grouping identifier
                              AAVSOitem$(AAVSOIndex,14);",";_   'CHART$, chart name
                              AAVSOitem$(AAVSOIndex,15)         '"na", notes to data line

        next
        close #AAVSOFile
    wait

    [QuitExport]
    AAVSOFlag = 0               'AAVSO export window is closed
    close #export
wait
'
'============== Eclipse Binary Analysis
'
[FindMin]

    NOMAINWIN
    WindowWidth = 328 : WindowHeight = 610
    UpperLeftX = INT((DisplayWidth-WindowWidth)/2)
    UpperLeftY = INT((DisplayHeight-WindowHeight)/2)
    Menu        #FindMinimum, " File " ,  "Exit", [Quit_FindMinimum]
    Menu        #FindMinimum, " Config ", "JD2000 Epoch", [Set_JD2000],_
                                        "JD4713BC Epoch", [Set_JD4713BC]

    groupbox   #FindMinimum.Minimum, "Find Time of Minimum  HJD2000", 9, 9, 300, 310
    statictext #FindMinimum.text1, "start time (cursor A)", 120,35,180,20
    textbox    #FindMinimum.start, 20, 30, 90, 25

    statictext #FindMinimum.text2, "end time (cursor B)", 120,65,180,20
    textbox    #FindMinimum.end, 20, 60, 90, 25

    statictext #FindMinimum.text3, "trial min. (cursor C)", 120,95,180,20
    textbox    #FindMinimum.trial, 20, 90, 90, 25

    statictext #FindMinimum.text4, "intervals", 120,125,180,20
    textbox    #FindMinimum.intervals, 80, 120, 30,25

    statictext #FindMinimum.text5, chr$(68)+"T", 120,155,100,20
    textbox    #FindMinimum.deltaT, 20, 150, 90, 25

    statictext #FindMinimum.text6, "T", 120,185,10,20
    statictext #FindMinimum.text9, "min", 133,190,30,20
    textbox    #FindMinimum.minimum, 20, 180, 90, 25

    statictext #FindMinimum.text25, "Yo =", 120,215,40,20
    statictext #FindMinimum.text7, "S", 160, 214,10,20
    statictext #FindMinimum.text8, "|Diff|"+chr$(178), 173,215,80,20
    textbox    #FindMinimum.diff, 20, 210, 90, 25

    statictext #FindMinimum.text24, "Probable Error", 120,244,200,20
    textbox    #FindMinimum.error, 20,240,90,25

    button     #FindMinimum.update, "Update", [UpdateFindMinimum], UL, 120, 280, 75, 30

    groupbox   #FindMinimum.OC, "Find O-C  HJD4713BC", 9, 330, 300, 215

    statictext #FindMinimum.text10, "24 ", 20,354,20,20
    textbox    #FindMinimum.epoch, 42, 350, 100, 25
    statictext #FindMinimum.text11, "Enter Epoch", 152, 355, 120, 20

    textbox    #FindMinimum.period, 42, 380, 100, 25
    statictext #FindMinimum.text13, "Enter Period", 152, 385, 130, 20

    statictext #FindMinimum.text14, "24 ", 20,424,20,20
    textbox    #FindMinimum.Tcalc,  42, 420, 100, 25
    statictext #FindMinimum.text15, "T", 152, 425, 10, 20
    statictext #FindMinimum.text16, "min", 165, 430, 25, 20
    statictext #FindMinimum.text17, "calculated", 190, 425, 100, 20

    statictext #FindMinimum.text18, "24 ", 20,454,20,20
    textbox    #FindMinimum.Tobs,   42, 450, 100, 25
    statictext #FindMinimum.text19, "T", 152, 455, 10, 20
    statictext #FindMinimum.text20, "min", 165, 460, 25, 20
    statictext #FindMinimum.text21, "observed", 190, 455, 100, 20

    textbox    #FindMinimum.cycles, 42, 480, 100, 25
    statictext #FindMinimum.text23, "Cycles", 152, 485, 100, 20

    textbox    #FindMinimum.OminusC, 42, 510, 100,25
    statictext #FindMinimum.text22, "O-C", 152,515,40,20

    Open "Eclipsing Binary Analysis" for Window as #FindMinimum
    #FindMinimum "trapclose [Quit_FindMinimum]"
    #FindMinimum "font courier_new 8 16"

    print #FindMinimum.text5,  "!font symbol 10 14"
    print #FindMinimum.text6,  "!font symbol 10 14"
    print #FindMinimum.text7,  "!font symbol 13 17"
    print #FindMinimum.text8,  "!font courier 8 16"
    print #FindMinimum.text9,  "!font courier_new 8"
    print #FindMinimum.text15, "!font symbol 10 14"
    print #FindMinimum.text25, "!font courier 8 16"
    print #FindMinimum.text16, "!font courier_new 8"
    print #FindMinimum.text19, "!font symbol 10 14"
    print #FindMinimum.text20, "!font courier_new 8"

    FindMinimumFlag = 1

    DateEpoch$ = "JD4713BC"
    if CursorTime.A$ = "" or CursorTime.B$ = "" or CursorTime.C$ = "" then
        notice "set cursors"
        wait
    end if
    print #FindMinimum.start, CursorTime.A$ 
    print #FindMinimum.end, CursorTime.B$ 
    print #FindMinimum.trial, CursorTime.C$

    gosub [FindIntervals]
    print #FindMinimum.intervals, Intervals$ 

    wait

    [Set_JD2000]
        DateEpoch$ = "JD2000"
        print #FindMinimum.text10, "!hide"
        print #FindMinimum.text14, "!hide"
        print #FindMinimum.text18, "!hide"
    wait
    [Set_JD4713BC]
        DateEpoch$ = "JD4713BC"
        print #FindMinimum.text10, "!show"
        print #FindMinimum.text14, "!show"
        print #FindMinimum.text18, "!show"
    wait

    [UpdateFindMinimum]
        if CursorTime.A$ = "" or CursorTime.B$ = "" or CursorTime.C$ = "" then
            notice "set cursors"
            wait
        end if
        print #FindMinimum.start, CursorTime.A$
        print #FindMinimum.end, CursorTime.B$
        print #FindMinimum.trial, CursorTime.C$

        print #FindMinimum.intervals, "!contents? Temporary$";
        if int(val(Temporary$)) <> Intervals then
            Intervals = int(val(Temporary$))
            Intervals$ = using("##", Intervals)
        end if
        print #FindMinimum.intervals, Intervals$

        gosub [FindDeltaT]
        print #FindMinimum.deltaT, DeltaT$

        open "IREX.txt" for append as #IREX
        print #IREX, "output from [FindMin]"
        print #IREX, " "
        print #IREX, "CursorTime.A$  CursorTime.B$  CursorTime.C$  Intervals   DeltaT"
        print #IREX, CursorTime.A$+"      "+CursorTime.B$+"      "+CursorTime.C$+"         "+_
                 Intervals$+"      "+DeltaT$ 
                                                    'find Yzero
        TimeZero = CursorTime.C
        gosub [FindMinimum]

        print #IREX, " "
        print #IREX, "I   DateDecending  DateAscending  MagDecending  MagAscending  MagDiff"
        for I = Intervals to 2 step -1
            print #IREX, using("##",I)+"    "+_
                         using("####.####",DateDecending(I))+"      "+_
                         using("####.####",DateAscending(I))+"     "+_
                         using("###.###",MagDecending(I))+"       "+_
                         using("###.###",MagAscending(I))+"       "+_
                         using("##.###",MagDiff(I))
        next
        Yzero = MagDiff
        print #IREX, "Yzero = "+using("##.######", Yzero)
        print #IREX, " "
                                                    'find Yminus
        TimeZero = CursorTime.C - DeltaT
        gosub [FindMinimum]

        print #IREX, " "
        print #IREX, "I   DateDecending  DateAscending  MagDecending  MagAscending  MagDiff"
        for I = Intervals to 2 step -1
            print #IREX, using("##",I)+"    "+_
                         using("####.####",DateDecending(I))+"      "+_
                         using("####.####",DateAscending(I))+"     "+_
                         using("###.###",MagDecending(I))+"       "+_
                         using("###.###",MagAscending(I))+"       "+_
                         using("##.###",MagDiff(I))
        next
        Yminus = MagDiff
        print #IREX, "Yminus = "+using("##.######", Yminus)
        print #IREX, " "
                                                    'find Yplus
        TimeZero = CursorTime.C + DeltaT
        gosub [FindMinimum]

        print #IREX, " "
        print #IREX, "I   DateDecending  DateAscending  MagDecending  MagAscending  MagDiff"
        for I = Intervals to 2 step -1
            print #IREX, using("##",I)+"    "+_
                         using("####.####",DateDecending(I))+"      "+_
                         using("####.####",DateAscending(I))+"     "+_
                         using("###.###",MagDecending(I))+"       "+_
                         using("###.###",MagAscending(I))+"       "+_
                         using("##.###",MagDiff(I))
        next
        Yplus = MagDiff
        print #IREX, "Yplus = "+using("##.######", Yplus)
        print #IREX, " "

                                                    'find Tminimum
        ReducedMinimum = CursorTime.C + 0.5*((Yminus - Yplus)/(Yminus - 2*Yzero + Yplus))*DeltaT
        ReducedMinimum$ = using("####.####", ReducedMinimum)
        Yzero$ = using("##.######",Yzero)
        print #FindMinimum.minimum, ReducedMinimum$
        print #FindMinimum.diff, Yzero$
                                                    'find probable error
        TimeZero = ReducedMinimum
        gosub [FindMinimum]
        print #IREX, " "
        print #IREX, "I   DateDecending  DateAscending  MagDecending  MagAscending  MagDiff"
        for I = Intervals to 2 step -1
            print #IREX, using("##",I)+"    "+_
                         using("####.####",DateDecending(I))+"      "+_
                         using("####.####",DateAscending(I))+"     "+_
                         using("###.###",MagDecending(I))+"       "+_
                         using("###.###",MagAscending(I))+"       "+_
                         using("##.###",MagDiff(I))
        next
        Yminimum = MagDiff
        print #IREX, "Yminimum = "+using("##.######", Yminimum)
        print #IREX, " "
        close #IREX

        ProbableError = SQR(Yminimum/(Intervals - 1)) * 1.11 * 0.68
        ProbableError$ = using("##.####", ProbableError)
        print #FindMinimum.error, ProbableError$

                                                    'draw Tminimum on graph
        Xmin = (ReducedMinimum - val(StartJ2000$))* PixelTime + 40
        print #show.graphicbox1, "delsegment ";CursorSegmentD
        print #show.graphicbox1, "redraw"
        print #show.graphicbox1, "color lightgray"
        print #show.graphicbox1, "line ";Xmin;" 20 ";Xmin;" 270"
        print #show.graphicbox1, "Segment CursorSegmentD"
        print #show.graphicbox1, "flush"
        print #show.graphicbox1, "color black"

        print #FindMinimum.epoch,  "!contents? epoch$";
        print #FindMinimum.period, "!contents? period$";
        epoch = val(epoch$)
        period = val(period$)
        if DateEpoch$ = "JD4713BC" then
            Tobserved = ReducedMinimum + 51545
            Tobserved$ = using("#####.####",Tobserved)
            print #FindMinimum.Tobs, Tobserved$
        else
            Tobserved  = ReducedMinimum
            Tobserved$ = ReducedMinimum$ 
            print #FindMinimum.Tobs, Tobserved$ 
        end if
        if epoch <> 0 and period <> 0 then
            gosub [FindOC]
        end if
    wait

    [Quit_FindMinimum]
        close #FindMinimum
        FindMinimumFlag = 0
wait
'
'============== Eclipse Binary Analysis subroutines
'
[FindIntervals]
    N = 0
    for I = 1 to VarIndexMax
        if val(VarItem$(I,7)) >= CursorTime.A AND val(VarItem$(I,7)) <= CursorTime.B then
            N = N + 1
        end if
    next
    Intervals = int(N/2)
    Intervals$ = str$(Intervals)
return
'
[FindDeltaT]
    DeltaT = (CursorTime.B - CursorTime.A)/(Intervals * 2)
    DeltaT$ = using("##.#####",DeltaT)
return
'
[FindMinimum]
    For I =  Intervals to 1 step -1
        DeltaTsum = I * DeltaT
        DateDecending(I) = TimeZero - DeltaTsum
        DateAscending(I) = TimeZero + DeltaTsum
    Next

    For I = Intervals to 2 step -1
        X = DateDecending(I)
        For VarIndex = 1 to VarIndexMax
            if  val(VarItem$(VarIndex,7)) <= X AND val(VarItem$(VarIndex+1,7)) > X then
                X1 = val(VarItem$(VarIndex,7))
                X2 = val(VarItem$(VarIndex + 1,7))
                Y1 = val(VarItem$(VarIndex,8))
                Y2 = val(VarItem$(VarIndex + 1,8))
                Y  = Y1 + ((Y2 - Y1)/(X2 - X1))*(X - X1)
                MagDecending(I) = Y
                exit for
            end if
        next
    next

    For I = Intervals to 2 step -1
        X = DateAscending(I)
        For VarIndex = 1 to VarIndexMax
            if  val(VarItem$(VarIndex,7)) <= X AND val(VarItem$(VarIndex+1,7)) > X then
                X1 = val(VarItem$(VarIndex,7))
                X2 = val(VarItem$(VarIndex + 1,7))
                Y1 = val(VarItem$(VarIndex,8))
                Y2 = val(VarItem$(VarIndex + 1,8))
                Y  = Y1 + ((Y2 - Y1)/(X2 - X1))*(X - X1)
                MagAscending(I) = Y
                exit for
            end if
        next
    next

    MagDiff = 0
    For I = Intervals to 2 step -1
        MagDiff(I) = MagAscending(I) - MagDecending(I)
        MagDiff = MagDiff + (abs(MagDiff(I)))^2
    next
return
'
[FindOC]
    cycles = (Tobserved - epoch)/period
    cyclesRemainder = cycles MOD 1
    cyclesInterger = int(cycles + 0.5)
    cyclesInterger$ = using("#######",cyclesInterger)
    print #FindMinimum.cycles, cyclesInterger$

    Tcalculated = cyclesInterger * period + epoch
    Tcalculated$ = using("#####.####", Tcalculated)
    print #FindMinimum.Tcalc, Tcalculated$

    OC = Tobserved - Tcalculated
    OC$ = using("###.#####", OC)
    print #FindMinimum.OminusC, OC$
return
'
'
'====================Phase Plot Analysis
'
[FindPeriod]

    NOMAINWIN
    WindowWidth = 320 : WindowHeight = 220
    UpperLeftX = INT((DisplayWidth-WindowWidth)/2)
    UpperLeftY = INT((DisplayHeight-WindowHeight)/2)
    Menu        #FindPeriod, " File " ,  "Exit", [Quit_FindPeriod]

    statictext  #FindPeriod.text1, "start date", 120,25,110,20
    textbox     #FindPeriod.start, 20, 20, 90, 25

    groupbox    #FindPeriod.enter, "enter period", 10,55,292,55
    radiobutton #FindPeriod.day, "day", [setDay], [resetDay], 130,80,40,20
    radiobutton #FindPeriod.hour, "hour", [setHour], [resetHour], 190,80,50,20
    radiobutton #FindPeriod.min,   "min", [setMin], [resetMin], 250,80,50,20
    textbox     #FindPeriod.period, 20, 75, 90, 25

    button      #FindPeriod, "Update", [UpdateFindPeriod], UL 110, 125,80,30

    Open "Phase Plot Analysis" for Window as #FindPeriod
        #FindPeriod "trapclose [Quit_FindPeriod]"
        #FindPeriod "font courier_new 8 16"
        print #FindPeriod.day, "set"
        DayHourMin = 1
        FindPeriodFlag = 1

        print #FindPeriod.start, StartJ2000$ 
    wait
                                                'set units for period, all calculations done in days
    [setDay]
        DayHourMin = 1
    wait

    [setHour]
        DayHourMin = 24
    wait

    [setMin]
        DayHourMin = 1440
    wait

    [resetDay]
    [resetHour]
    [resetMin]
    wait

    [UpdateFindPeriod]
        print #FindPeriod.start, StartJ2000$ 
        print #FindPeriod.start, "!contents? StartTime$"
        print #FindPeriod.period, "!contents? EnterPeriod$"

        StartTime = val(StartTime$)
        EnterPeriod = val(EnterPeriod$)

        if EnterPeriod = 0 then
            notice "enter valid period"
            wait
        end if

        EnterPeriod = EnterPeriod/DayHourMin

        gosub [DrawDataPeriod]
    wait

    [Quit_FindPeriod]
        close #FindPeriod
        FindPeriodFlag = 0
wait
'
'====================Phase Plot Analysis subroutines
'
    [DrawDataPeriod]
        open "IREX.txt" for append as #IREX
        print #IREX, "output from [DrawDataPeriod]"
        print #IREX, " "

        print #show.graphicbox1, "delsegment ";DrawDataSegment
        print #show.graphicbox1, "redraw"
        print #show.graphicbox1, "color darkgreen"
        print #show.graphicbox1, "backcolor darkgreen"

        if PlotFlag = 1 then                'draw plot for variable star data

            PixelTime = 10/TimeScaleJ2000
            PixelMag = 25/MagScale

            print #IREX,"PixelTime ";PixelTime
            print #IREX,"PixelMag  ";PixelMag
            print #IREX," "
            print #IREX, "VarIndex  ";"VarMagDiff  ";"   yD     ";"VarTimeDiff ";"   xD   ";"    VarError"

            for VarIndex = 1 to VarIndexMax

                VarMagDiff = val(VarItem$(VarIndex,8)) - VarMagMax
                yD = VarMagDiff * PixelMag + 20

                VarError = val(VarItem$(VarIndex,9))

                VarTimeDiff = val(VarItem$(VarIndex,7)) - StartTime

                VarTimePhase = (VarTimeDiff/EnterPeriod) MOD 1

                xD = VarTimePhase * EnterPeriod * PixelTime + 40

                print #IREX, using("###",VarIndex)+"     "+_
                             using("#####.##",VarMagDiff)+"      "+_
                             using("####.##",yD)+"  "+_
                             using("#####.####",VarTimeDiff)+"  "+_
                             using("######.###",xD)+"  "+_
                             using("####.###",VarError)

                if xD >= 40  then
                    print #show.graphicbox1, "place ";xD;" ";yD
                    print #show.graphicbox1, "circlefilled 2"
                end if

                if PlotErrorFlag = 1 then
                    VarErrorReduced = VarError * PixelMag * 0.5
                    yEup = yD + VarErrorReduced
                    yEdown = yD - VarErrorReduced
                    print #show.graphicbox1, "line ";xD;" ";yEdown;" ";xD;" ";yEup
                end if
            next
                                                'print vertical line showing end of period
            Xmin = EnterPeriod * PixelTime + 40
            print #show.graphicbox1, "line ";Xmin;" 20 ";Xmin;" 270"

            print #show.graphicbox1, "line 40 240 ";Xmin;" 240"

            for I = 1 to 10
                X = int(EnterPeriod * PixelTime/10) * I + 40
                print #show.graphicbox1, "line ";X;" 240 ";X;" 230"
            next
            X = int(EnterPeriod * PixelTime/10) * 5 + 40
            print #show.graphicbox1, "line ";X;" 240 ";X;" 220"

            print #show.graphicbox1, "color black"
            print #show.graphicbox1, "backcolor white"
            print #show.graphicbox1, "font arial 8 12"

            X = int(EnterPeriod * PixelTime/10) * 5 + 25
            print #show.graphicbox1, "place ";X;" 255"
            shit$ = "0.5"
            print #show.graphicbox1, "\ "+shit$

            X = int(EnterPeriod * PixelTime/10) * 10 + 20
            print #show.graphicbox1, "place ";X;" 255"
            print #show.graphicbox1, "\1.0"

            print #show.graphicbox1, "Segment DrawDataSegment"
            print #show.graphicbox1, "flush"
            print #show.graphicbox1, "redraw"

            print #IREX, " "
            close #IREX
        else
            notice "only slow data can be plotted"
        end if
    return
'
'
'=================Fourier Analysis with PERIOD04
'
[FindFourier]
    NOMAINWIN
    WindowWidth = 270 : WindowHeight = 220
    UpperLeftX = INT((DisplayWidth-WindowWidth)/2)
    UpperLeftY = INT((DisplayHeight-WindowHeight)/2)
    Menu        #FindFourier, " File " ,  "Exit", [Quit_FindFourier]

    groupbox    #FindFourier.file, "create time-string file", 10,15,242,100
    radiobutton #FindFourier.J, "J", [setJ], [resetJ], 30,40,30,20
    radiobutton #FindFourier.H, "H", [setH], [resetH], 85,40,30,20

    button      #FindFourier.file, "Make File", [MakePeriod04File], UL 55,75,160,30

    button      #FindFourier.run, "Run Period04", [RunPeriod04], UL 55, 125,160,30

    Open "Fourier Analysis" for Window as #FindFourier
        #FindFourier "trapclose [Quit_FindFourier]"
        #FindFourier "font courier_new 8 16"
        print #FindFourier.J, "set"
        FourierColor$ = "J"

        FindFourierFlag = 1
    wait

    [setJ]
        FourierColor$ = "J"
    wait

    [setH]
        FourierColor$ = "H"
    wait

    [resetJ]
    [resetH]
    wait

    [RunPeriod04]
        run "period04.exe"
    wait

    [MakePeriod04File]
                                            'find which colors are available
        Jflag = 0 : Hflag = 0
        if val(VarItem$(1,8)) <> 0  then
            Jflag = 1
        end if
        if val(VarItem$(1,10)) <> 0  and Jflag = 1 then
            Hflag = 1
        end if

                                            'select UBVRI color for making datafile
        select case
            case (Jflag = 1) and (FourierColor$ = "J")
                FourierColorIndex = 8
            case (Hflag = 1) and (FourierColor$ = "H")
                FourierColorIndex = 10
            case else
                notice "error - no magnitudes"
                wait
        end select
                                            'make time string array for writing to data file
        for TimeIndex = 1 to VarIndexMax
            TimeString$(TimeIndex,1) = VarItem$(TimeIndex,7)
            select case FourierColorIndex
                case 8
                    TimeString$(TimeIndex,2) = VarItem$(TimeIndex,8)
                case 10
                    TimeString$(TimeIndex,2) = str$(val(VarItem$(TimeIndex,8)) - val(VarItem$(TimeIndex,10)))
            end select
        next

        [TimeString]                        'make data file with .txt type as default
        filedialog "Save Time String data", PathTimeString$, TimeStringFile$

         for I = len(TimeStringFile$) to 1 step -1
             if mid$(TimeStringFile$,I,1) = "\" then
                 ShortTimeString$ = mid$(TimeStringFile$,I+1)
                 PathTimeString$ = left$(TimeStringFile$,I)+"*.txt"
                 exit for
             end if
         next

         if TimeStringFile$ = "" then [QuitTimeString]

         files "c:\", TimeStringFile$, info$()
         if val(info$(0,0)) <> 0 then
            confirm "Write over existing file?"; Answer$
               if Answer$ = "no" then [TimeString]
         end if

         if (right$(TimeStringFile$,4) = ".txt") OR (right$(TimeStringFile$,4) = ".TXT") then
             open TimeStringFile$ for Output as #TimeStringFile
         else
             TimeStringFile$ = TimeStringFile$+".txt"
             open TimeStringFile$ for Output as #TimeStringFile
         end if

         for TimeIndex = 1 to VarIndexMax
            print #TimeStringFile, TimeString$(TimeIndex,1);"  ";TimeString$(TimeIndex,2)
         next
         close  #TimeStringFile

        [QuitTimeString]
    wait

    [Quit_FindFourier]
        close #FindFourier
        FindFourierFlag = 0
wait
'
end


























