'                           Reduction Module - J H  Photometry
'                                              Optec, Inc.
'
'======version history
'V1.20, December 2015
'  added AllSky
'  PPparms2 configuration file
'
'V1.10 October 2015
'  compiled with LB4.50
'  increased dim
'
'V1.01
'  added JDFlag, COMP, and VAR info to file header
'
'V1.00
'
'
'=====dimension statements and initial conditions
'
    DIM info$(10,10)
    files "c:\", info$()

    DIM ReducData$(2000)                 'reduction data lines for list box table, index = ReducIndex
        ReducData$(1) = "open raw data file first and select comp and variable star"

    DIM ReducStar$(2000)                 'comp or variable star name
    DIM ReducType$(2000)                 'C or V
    DIM ReducJD(2000)                    'Julean date of observation, epoch 2000
    DIM ReducCount(2000)                 'net count, average of (star - sky)

    DIM ReducDelta.j(2000)               'differential j instrument magnitude
    DIM ReducDelta.h(2000)               'differential h instrument magnitude

    DIM ReducDelta.jo(2000)              'differential jo instrument magnitude
    DIM ReducDelta.ho(2000)              'differential ho instrument magnitude

    DIM ReducDelta.J(2000)               'differential J standard magnitude, equation 2.48
    DIM ReducDelta.JH(2000)              'differential J - H standard magnitude, equation 2.49


    DIM ReducJ(2000)                     'standard J magnitude, equation 2.51
    DIM ReducJH(2000)                    'standard J-H index, equation 2.52

    DIM ReducX(2000)                     'air mass for comp or variable
    DIM ReducFilter$(2000)               'filter, J H

    DIM CompStar$(200)                   'list of comparison C stars, index = CompIndex
        CompStar$(1) = "   "
    DIM VarStar$(200)                    'list of variable V stars, index - VarIndex
        VarStar$(1) = "   "

    DIM StarData$(200)                   'Comp/Variable/Check star data
        StarData$(1) = "open RAW data file first"

'======arrays used in Star Data file reduction

    DIM SDitem$(2000,10)                'individual data items from Star Data file
                                        'SDitem$(DataIndex,1)  = comp or variable star name
                                        'SDitem$(DataIndex,2)  = type, either C or V
                                        'SDitem$(DataIndex,3)  = RA hour
                                        'SDitem$(DataIndex,4)  = RA minute
                                        'SDitem$(DataIndex,5)  = RA second
                                        'SDitem$(DataIndex,6)  = DEC degree
                                        'SDitem$(DataIndex,7)  = DEC minute
                                        'SDitem$(DataIndex,8)  = DEC second
                                        'SDitem$(DataIndex,9)  = J magnitude
                                        'SDitem$(DataIndex,10) = J-H color index

'=======arrays used in raw file reduction

    DIM RawData$(2000)                  'data from raw file of observations

    DIM RDitem$(2000,15)                'individual data items from raw file of observations
                                        'RDitem$(RawIndex,1)  = UT month
                                        'RDitem$(RawIndex,2)  = UT day
                                        'RDitem$(RawIndex,3)  = UT year
                                        'RDitem$(RawIndex,4)  = UT hour
                                        'RDitem$(RawIndex,5)  = UT minute
                                        'RDitem$(RawIndex,6)  = UT second
                                        'RDitem$(RawIndex,7)  = catalog type, C, V, SKY, SKYLAST or SKYNEXT
                                        'RDitem$(RawIndex,8)  = star name, 8 bytes
                                        'RDitem$(RawIndex,9)  = filter, J H
                                        'RDitem$(RawIndex,10) = count 1
                                        'RDitem$(RawIndex,11) = count 2
                                        'RDitem$(RawIndex,12) = count 3
                                        'RDitem$(RawIndex,13) = count 4
                                        'RDitem$(RawIndex,14) = integration time, 1 or 10 seconds
                                        'RDitem$(RawIndex,15) = scale factor, 1, 10 or 100

    DIM CountFinal(2000)                'average count including integration and scale, index var = RawIndex
    DIM JD(2000)                        'Julean date from 2000, index var = RawIndex
    DIM JT(2000)                        'Julean century from 2000, index var = RawIndex
    DIM X(2000)                         'air mass, index var = RawIndex
'
'=====initialize and start up values
'
VersionNumber$ = "1.20"
PathDataFile$ = "*.raw"                             'default path for data files
PathSaveData$ = "*.var"                             'default path for var saved data file
ChoiceFlag = 0                                      'default value to indicate that choice window is closed
SaveDataFlag = 0                                    'default value to indicate that SaveData window is closed
'
'=====open and read in PPparms data for configuration
'
open "PPparms2_JH.txt" for input as #PPparms
    input #PPparms, Location$                       'latitude and longitude in degrees
    input #PPparms, KJ                              'first order extinction for J
    input #PPparms, KH                              'first order extinction for H
    input #PPparms, KKjh                            'second order extinction for j-h, default = 0
    input #PPparms, Rho                             'transformation coeff. for J using J-H
    input #PPparms, Lambda                          'transformation coeff. J-H
    input #PPparms, JDFlag                          '1 if using JD and 0 if using HJD
    input #PPparms, OBSCODE$                        'AAVSO observatory code
    input #PPparms, Spare2                          '0 not used at this time
    input #PPparms, ZPj                             'zero-point constant for j
    input #PPparms, ZPjh                            'zero-point constant for j-h
    input #PPparms, Ej                              'standard error for j
    input #PPparms, Ejh                             'standard error for j-h
close #PPparms

gosub [Find_Lat_Long]

            open "IREX.txt" for output as #IREX
                print #IREX, "Reduction JH module, Version:"+VersionNumber$
                print #IREX, " "
                print #IREX, "PPparms2 JH.txt"
                print #IREX, "  Location$     ";Location$
                print #IREX, "  KJ            ";KJ
                print #IREX, "  KH            ";KH
                print #IREX, "  KKjh          ";KKjh
                print #IREX, "  Rho           ";Rho
                print #IREX, "  Lambda        ";Lambda
                print #IREX, "  JDFlag        "; JDFlag
                print #IREX, "  OBSCODE       "; OBSCODE$
                print #IREX, "  Spare2        "; Spare2
                print #IREX, "  ZPj           "; ZPj
                print #IREX, "  ZPjh          "; ZPjh
                print #IREX, "  Ej            "; Ej
                print #IREX, "  Ejh           "; Ejh
                print #IREX, " "
            close #IREX
'
'======open and read in Star Data
'
open "Star Database JH.txt" for input as #StarData
    DataIndex = 0
    while eof(#StarData) = 0
        DataIndex = DataIndex + 1
        input #StarData,SDitem$(DataIndex,1),_      'comp or var star name
                        SDitem$(DataIndex,2),_      'type, either C or V
                        SDitem$(DataIndex,3),_      'RA hour
                        SDitem$(DataIndex,4),_      'RA minute
                        SDitem$(DataIndex,5),_      'RA second
                        SDitem$(DataIndex,6),_      'DEC degree
                        SDitem$(DataIndex,7),_      'DEC minute
                        SDitem$(DataIndex,8),_      'DEC second
                        SDitem$(DataIndex,9),_      'J magnitude, ##.##
                        SDitem$(DataIndex,10)       'J-H index, ##.##
    wend
    DataIndexMax = DataIndex
close #StarData
'
'=====set up main GUI control window
'
[WindowSetup]
    NOMAINWIN
    WindowWidth = 900 : WindowHeight = 465
    UpperLeftX = INT((DisplayWidth-WindowWidth)/2)
    UpperLeftY = INT((DisplayHeight-WindowHeight)/2)

[ControlSetup]
    Menu        #reduc, "File", "Open File", [Open_File],_
                                "Save Data to Variable Star File", [Save_Data],_
                                "Quit", [Quit_Reduc]
    Menu        #reduc, "Information", "View Extinction and Transformation Coefficients", [View_Coefficients],_
                                       "Comp/Variable/Check Star Database", [View_StarDatabase]
    Menu        #reduc, "Help", "About", [About], "Help", [Help]

    groupbox    #reduc.groupbox1, "Raw Data File", 11, 310, 250, 90
    groupbox    #reduc.groupbox2, "Comparison  -  Variable/Check", 265, 310, 290, 90
    groupbox    #reduc.groupbox4, "Process", 560, 310, 75, 90
    groupbox    #reduc.groupbox3, "Mean Standard Magnitude", 640, 310, 240, 90

    statictext  #reduc.statictext1,  "star", 30, 20, 40, 14
    statictext  #reduc.statictext2, "type", 135, 20, 40, 14
    statictext  #reduc.statictext3, "UT", 225, 7, 30, 14
    statictext  #reduc.statictext4,  "J2000.0", 211, 20, 170, 14
    statictext  #reduc.statictext5,  "net", 325, 7, 30, 14
    statictext  #reduc.statictext6, "count", 315, 20, 60, 14
    statictext  #reduc.statictext7, "F", 405, 20, 10, 14
    statictext  #reduc.statictext8,  chr$(68), 475, 20, 10, 14
    statictext  #reduc.statictext9,  "j", 485, 20, 10, 14
    statictext  #reduc.statictext10,  chr$(68), 565, 20, 10, 14
    statictext  #reduc.statictext11,  "h", 575, 20, 10, 14

    statictext  #reduc.statictext18, "J", 670, 20, 70, 14
    statictext  #reduc.statictext19, "J-H", 750, 20, 70, 14

    statictext  #reduc.statictext30, "J", 713, 342, 10, 14
    statictext  #reduc.statictext31, "J-H", 697, 357, 30, 14
    statictext  #reduc.statictext32, "UT J2000", 650, 372, 80, 14

    button      #reduc.Start, "Start", [Start_Reduction],UL,573,350

    textbox     #reduc.Analysis, 730, 338, 135, 50

    textbox     #reduc.FileName, 15, 350, 240, 25

    listbox     #reduc.Table, ReducData$(),[Reduction_Table.click], 10, 40, 870, 265

    combobox    #reduc.Comp,CompStar$(),[Select_Comp.click], 270, 350, 135, 300
    combobox    #reduc.Variable,VarStar$(),[Select_Var.click],410, 350, 140, 300

    Open "Data Reduction - J H Photometry" for Window as #reduc
    #reduc "trapclose [Quit_Reduc]"
    #reduc.Table "selectindex 1"
    #reduc.Comp "selectindex 1"
    #reduc "font courier_new 10 14"

    print #reduc.statictext8,  "!font symbol 10 14"
    print #reduc.statictext10, "!font symbol 10 14"

    print #reduc.FileName, "open raw data file"

    StartVar.Flag = 0               'turn off START button until data and stars are loaded
    StartComp.Flag = 0
    print #reduc.Start, "!Disable"

Wait                                'finished setting up, wait here for new command
'
'======menu controls
'
[Open_File]
    filedialog "Open Data File", PathDataFile$, DataFile$

    for I = len(DataFile$) to 1 step -1             'remember path for opened folder and file
        if mid$(DataFile$,I,1) = "\" then
            ShortDataFile$ = mid$(DataFile$,I+1)
            PathDataFile$ = left$(DataFile$,I)+"*raw"
            exit for
        end if
    next I

    if DataFile$ = "" then
        print #reduc.FileName, "open raw data file"
    else
        files "c:\", DataFile$, info$()
        if val(info$(0, 0)) = 0 then
            notice "cannot create new file"
        else
                                                    'clear the contents of the comparison & variable comboboxes
            redim CompStar$(200)                    'list of comparison C stars, index = CompIndex
                  CompStar$(1) = "   "
            redim VarStar$(200)                     'list of variable V stars, index - VarIndex
                  VarStar$(1) = "   "
            redim ReducJ(2000)                      'standard J magnitude
            redim ReducJH(2000)                     'standard J-H index
            ReducJ.average = 0
            ReducJH.average = 0
            ReducJ.std.deviation = 0
            ReducJH.std.deviation = 0
                                                    'clear reduction table
            redim ReducData$(2000)
            ReducData$(1) = "select comp and variable star"
            print #reduc.Table, "reload"
                                                    'clear the Comp & Var input boxes
            print #reduc.Comp, "reload"
            print #reduc.Comp, "selectindex 1"
            print #reduc.Variable, "reload"
            print #reduc.Variable, "selectindex 1"
                                                    'clear analysis results box
            print #reduc.Analysis, ""

            StartVar.Flag = 0                       'turn off START button until data and stars are loaded
            StartComp.Flag = 0
            CompFlag = 0
            print #reduc.Start, "!Disable"
                                                    'input the contents of the file
            open DataFile$ for input as #RawFile
            for RawIndex = 1 to 4                   'read first 4 descriptive lines with line input to capture commas
                line input #RawFile, RawData$(RawIndex)
            next RawIndex
            RawIndex = 4
            while eof(#RawFile)=0                   'read rest of the data to end of file
                RawIndex = RawIndex + 1
                input #RawFile, RawData$(RawIndex)
            wend
            RawIndexMax = RawIndex
            close #RawFile

            open "IREX.txt" for append as #IREX
                print #IREX, "output from [Find_File_Name]"
                gosub [Find_File_Name]
                print #IREX, " "
            close #IREX

            open "IREX.txt" for append as #IREX
                print #IREX, "output from [Convert_RawFile]"
                print #IREX, "M - D - Year  H - M - S   T  Object        F"+_
                             "    C1     C2     C3     C4    S   I"
                gosub [Convert_RawFile]
                print #IREX, " "
            close #IREX

            gosub [Total_Count_RawFile]
            gosub [Julian_Day_RawFile]

            CompIndexMax = 1                     'reset array index to 1 for Comp Star list
            VarIndexMax = 1                      'reset array index to 1 for Variable Star list

            open "IREX.txt" for append as #IREX
                print #IREX, "output from [IREX_RawFile]"
                print #IREX, "star         Julean date   net count  filter SkyPast   Sky Future,"

                Filter$ = "J"
                gosub [IREX_RawFile]
                Filter$ = "H"
                gosub [IREX_RawFile]

                print #IREX, " "
            close #IREX
            print #reduc.Variable, "reload"
            print #reduc.Comp, "reload"

            gosub [Make_CompVarCheck_List]

        end if
    end if
Wait

[Save_Data]
'--------------open file dialog for saving *.var reduced data file
    if (ReducJ.average = 0) AND (ReducJH.average = 0) then
        notice "nothing to save"
        wait
    end if
    filedialog "Open Variable Data File", PathSaveData$, SaveData$

    for I = len(SaveData$) to 1 step -1             'remember path for opened folder and file
        if mid$(SaveData$,I,1) = "\" then
            ShortSaveData$ = mid$(SaveData$,I+1)
            PathSaveData$ = left$(SaveData$,I)+"*.var"
            exit for
        end if
    next I

    files "c:\", SaveData$, info$()
    if SaveData$ <> "" then
        if val(info$(0,0)) = 0  then
            confirm "Create new data file?"; Answer$
            if Answer$ = "yes" then
                if (right$(SaveData$,4) = ".var") OR (right$(SaveData$,4) = ".VAR") then
                    open SaveData$ for output as #SaveData
                else
                    SaveData$ = SaveData$+".var"
                    open SaveData$ for output as #SaveData
                end if
                                                    'print header to var file
                                                    'FileHeaderFlag = "#"
                                                    'JDFlag         0 = HJD,   1 = JD
                                                    'Comp star name
                                                    'variable star name
                print #SaveData$, "#,"+str$(JDFlag)+","+trim$(selectedComp$)+","+trim$(selectedVar$)
            else
                Wait
            end if
        else
            confirm "Append data to "+ShortSaveData$+"?"; Answer$
            if Answer$ = "yes" then
                open SaveData$ for append as #SaveData
            else
                Wait
            end if
        end if

        SaveDataFlag = 1                            'save date file window is opened

'-------------open window to select saving mean or itermediate results
        NOMAINWIN
        WindowWidth = 220 : WindowHeight = 150
        UpperLeftX = INT((DisplayWidth-WindowWidth)/2)
        UpperLeftY = INT((DisplayHeight-WindowHeight)/2)

        radiobutton #choice.mean, "save mean magnitude values", [setMean], [resetMean], 10,10,170,20
        radiobutton #choice.intermediate, "save intermediate values", [setInt], [resetIng], 10,35,170,20
        button #choice.continue, "continue", [ContinueSave], UL, 80,85,55,25
        button #choice.cancel, "cancel", [QuitChoice], UL, 150,85,55,25

        open "Save Options" for window as #choice
        #choice "trapclose [QuitChoice]"
        ChoiceFlag = 1

        print #choice.mean, "set"

        [setMean]
            Choice$ = "mean"
        wait

        [setInt]
            Choice$ = "intermediate"
        wait

        [resetMean]
        [resetInt]
        wait
'--------------save reduced data to *.var file
        [ContinueSave]
        Select Case Choice$
        Case "mean"              'save mean magnitude index and J2000 date
            print #SaveData,JDtoCAL$(J2000.Mean)+" "+_
                            using("####.####",J2000.Mean)+" "+_
                            using("###.###",ReducJ.average)+" "+using("#.###",ReducJ.std.deviation)+" "+_
                            using("###.###",ReducJH.average)+" "+using("#.###",ReducJH.std.deviation)

        Case "intermediate"      'save individual J and index values with dates
            PrintFlag = 0
            for ReducIndex = 1 to ReducIndexMax
                if ReducType$(ReducIndex) = "V" then
                    PrintFlag = 1
                    Select Case ReducFilter$(ReducIndex)
                        Case "J"
                            SaveV = ReducJ(ReducIndex)
                            SaveDateCAL$ = JDtoCAL$(ReducJD(ReducIndex))
                            SaveDateJD = ReducJD(ReducIndex)
                        Case "H"
                            SaveJH = ReducJH(ReducIndex)
                                                     'check to see of value is zero and change to +/-0.001 if it is
                            TestValue = SaveJH
                            gosub [TestZero]
                            SaveJH = TestValue

                    End Select
                else
                    if PrintFlag = 1 then
                                                    'convert Juliean Date to HJD if JDFlag = 0
                        if JDFlag = 0 then
                            JDtemporary = SaveDateJD
                            gosub [JDtoHJD]
                            SaveDateJD = JDtemporary
                        end if

                        print #SaveData,SaveDateCAL$+" "+_
                            using("####.####",SaveDateJD)+" "+_
                            using("###.###",SaveV)+" "+using("#.###",0)+" "+_
                            using("###.###",SaveJH)+" "+using("#.###",0)
                        PrintFlag = 0
                    end if
                end if
            next
        End Select
    end if

    [QuitChoice]
    if ChoiceFlag = 1 then
        close #choice
        ChoiceFlag = 0
    end if
    if SaveDataFlag = 1 then
        close #SaveData
        SaveDataFlag = 0
    end if
Wait
'
'=====PPparms coefficient editor beginning
'
[View_Coefficients]

    NOMAINWIN
    WindowWidth = 410 : WindowHeight = 280

    UpperLeftX = INT((DisplayWidth-WindowWidth)/2)
    UpperLeftY = INT((DisplayHeight-WindowHeight)/2)

    Menu        #coeff, "File", "Use but do not save coefficients", [Use_Coefficients],_
                        "Save and use coefficients", [Save_Coefficients],_
                        "Quit", [Quit_Coefficients]

    statictext  #coeff.static1,  "K'",   30,  32, 15, 20
    statictext  #coeff.static2,  "j",    45,  37, 10, 20
    statictext  #coeff.static3,  "K'",   30,  67, 15, 20
    statictext  #coeff.static4,  "h",    45,  72, 10, 20
    statictext  #coeff.static5,  "K''",  30, 102, 15, 20
    statictext  #coeff.static6,  "jh",   45, 107, 10, 20
    statictext  #coeff.static7,  "r",    30, 135, 15, 20
    statictext  #coeff.static8,  "l",    30, 170, 10, 20

    textbox     #coeff.KJ,        70, 30,  65, 25
    textbox     #coeff.KH,        70, 65,  65, 25
    textbox     #coeff.KKjh,      70, 100, 65, 25
    textbox     #coeff.Rho,       70, 135, 65, 25
    textbox     #coeff.Lambda,    70, 170, 65, 25

    groupbox    #coeff.Date, "Select date protocol",160,20,225,80
    checkbox    #coeff.JD, "JD, Julian Date", [setJD], [resetJD],         190, 40, 150, 25
    checkbox    #coeff.HJD, "HJD, Heliocentric JD", [setHJD], [resetHJD], 190, 65, 150, 25

    groupbox    #coeff.AllSky, "Zero point constants (All Sky)", 160,110,225,95
    statictext  #coeff.static9, "Z",   170, 139, 10, 20
    statictext  #coeff.static10, "j",  180, 144, 10, 20
    textbox     #coeff.Zj,             200, 135, 65, 25
    statictext  #coeff.static11, "Z",  170, 174, 10, 20
    statictext  #coeff.static12, "jh", 180, 179, 20, 20
    textbox     #coeff.Zjh,            200, 170, 65, 25
    statictext  #coeff.static13, "s",  282, 137, 10, 20
    statictext  #coeff.static14, "j",  295, 146, 10, 20
    textbox     #coeff.Ej,             310, 135, 65, 25
    statictext  #coeff.static15, "s",  282, 172, 10, 20
    statictext  #coeff.static16, "jh", 295, 181, 15, 20
    textbox     #coeff.Ejh,            310, 170, 65, 25

    Open "PPparms JH coefficients" for Window as #coeff
        #coeff "trapclose [Quit_Coefficients]"
        #coeff "font ms_sans_serif 10"
        print #coeff.static8,  "!font symbol 14"             'font for printing epsilon
        print #coeff.static7,  "!font symbol 14"             'font for printing epsilon
        print #coeff.static13, "!font symbol 12"
        print #coeff.static15, "!font symbol 12"

                    'print PPparms coefficients into textboxes
        print #coeff.KJ, using("###.###",KJ)                'first order extinction for u: K'j
        print #coeff.KH, using("###.###",KH)                'first order extinction for b: K'h
        print #coeff.KKjh, using("###.###",KKjh)            'second order extinction for j-h
        print #coeff.Rho, using("###.###",Rho)              'transformation coefficient rho
        print #coeff.Lambda, using("###.###",Lambda)        'transformation coefficient lambda
        print #coeff.Zj, using("###.###", ZPj)              'zero point constant for j
        print #coeff.Zjh, using("###.###", ZPjh)            'zero point constant for j-h
        print #coeff.Ej, using("###.###", Ej)               'error for zero point constant j
        print #coeff.Ejh, using("###.###", Ejh)             'error for zero point constant j-h

        CoeffFlag = 1

        if JDFlag = 1 then
            print #coeff.JD, "set"
        else
            print #coeff.HJD, "set"
        end if
    wait

    [setJD]
        JDFlag = 1
        print #coeff.HJD, "reset"
    wait

    [setHJD]
        JDFlag = 0
        print #coeff.JD, "reset"
    wait

    [resetJD]
        JDFlag = 0
        print #coeff.HJD, "set"
    wait

    [resetHJD]
        JDFlag = 1
        print #coeff.JD, "set"
    wait

    [Use_Coefficients]
        temporary$ = ""
        print #coeff.KJ, "!contents? temporary$";
        KJ = val(temporary$)
        print #coeff.KH, "!contents? temporary$";
        KH = val(temporary$)
        print #coeff.KKjh, "!contents? temporary$";
        KKjh = val(temporary$)
        print #coeff.Rho, "!contents? temporary$";
        Rho = val(temporary$)
        print #coeff.Lambda, "!contents? temporary$";
        Lambda = val(temporary$)
        print #coeff.Zj, "!contents? temporary$";
        ZPj = val(temporary$)
        print #coeff.Zjh, "!contents? temporary$";
        ZPjh = val(temporary$)
        print #coeff.Ej, "!contents? temporary$";
        Ej = val(temporary$)
        print #coeff.Ejh, "!contents? temporary$";
        Ejh = val(temporary$)
    wait

    [Save_Coefficients]
        temporary$ = ""
        print #coeff.KJ, "!contents? temporary$";
        KJ = val(temporary$)
        print #coeff.KH, "!contents? temporary$";
        KH = val(temporary$)
        print #coeff.KKjh, "!contents? temporary$";
        KKjh = val(temporary$)
        print #coeff.Rho, "!contents? temporary$";
        Rho = val(temporary$)
        print #coeff.Lambda, "!contents? temporary$";
        Lambda = val(temporary$)
        print #coeff.Zj, "!contents? temporary$";
        ZPj = val(temporary$)
        print #coeff.Zjh, "!contents? temporary$";
        ZPjh = val(temporary$)
        print #coeff.Ej, "!contents? temporary$";
        Ej = val(temporary$)
        print #coeff.Ejh, "!contents? temporary$";
        Ejh = val(temporary$)

        open "PPparms2_JH.txt" for output as #PPparms
            print #PPparms, Location$                   'latitude and longitude
            print #PPparms, using("##.###",KJ)          'first order extinction for J
            print #PPparms, using("##.###",KH)          'first order extinction for H
            print #PPparms, using("##.###",KKjh)        'second order extinction for j-h, default = 0
            print #PPparms, using("##.###",Rho)         'transformation coeff. J
            print #PPparms, using("##.###",Lambda)      'transformation coeff. J-H
            print #PPparms, JDFlag                      '1 if using JD and 0 if using HJD
            print #PPparms, OBSCODE$                    'AAVSO observatory code
            print #PPparms, Spare2                      '0 not used at this time
            print #PPparms, using("##.###",ZPj)         'zero-point constant for j
            print #PPparms, using("##.###",ZPjh)        'zero-point constant for j-h
            print #PPparms, using("##.###",Ej)          'standard error for j
            print #PPparms, using("##.###",Ejh)         'standard error for j-h
        close #PPparms
    wait

    [Quit_Coefficients]
        CoeffFlag = 0
        close #coeff
wait
'
'=====View Star Database
'
[View_StarDatabase]
        if ViewStarDataFlag = 1 then
            notice "Comp/Variable/Check Star Database already opened"
            wait
        end if

        NOMAINWIN
        WindowWidth = 510 : WindowHeight = 290
        UpperLeftX = INT((DisplayWidth-WindowWidth)/2)
        UpperLeftY = INT((DisplayHeight-WindowHeight)/2)

        Menu        #ViewStarData, "File", "Quit", [QuitViewStarData]

        listbox #ViewStarData.Table, StarData$(),[ViewStarData.click], 10, 40, 480, 180

        statictext  #ViewStarData.RA,  "---RA---",  175,  5, 80, 15
        statictext  #ViewStarData.DEC,  "---DEC---",  265,  5, 90, 15
        statictext  #ViewStarData.name,  "Star Name",  20,  20, 90, 15
        statictext  #ViewStarData.type,  "Type",  130,  20, 40, 15
        statictext  #ViewStarData.RAhour,  "H",  180,  20, 10, 15
        statictext  #ViewStarData.RAmin,  "M",  210,  20, 10, 15
        statictext  #ViewStarData.RAsec,  "S",  235,  20, 10, 15
        statictext  #ViewStarData.DECdeg,  "D",  270,  20, 10, 15
        statictext  #ViewStarData.DECmin,  "M",  300,  20, 10, 15
        statictext  #ViewStarData.DECsec,  "S",  325,  20, 10, 15
        statictext  #ViewStarData.V,       "J",  370,  20, 10, 15
        statictext  #ViewStarData.BV,     "J-H",  415,  20, 30, 15

        open "Comp/Variable/Check Star Database" for window as #ViewStarData
        #ViewStarData "trapclose [QuitViewStarData]"

        ViewStarDataFlag = 1
        #ViewStarData "font courier_new 10 14"
        print #ViewStarData.Table, "reload"

    [ViewStarData.click]

Wait
    [QuitViewStarData]
        ViewStarDataFlag = 0
        close #ViewStarData
Wait
'
[About]
    notice "Data Reduction   JH Photometry"+chr$(13)+_
           "Version "+VersionNumber$+chr$(13)+_
           "copyright 2013-2015, Gerald Persha"+chr$(13)+_
           "www.SSPDataq.com"
Wait

[Help]
    run "hh photometry jh.chm"
Wait

[Quit_Reduc]                                'exit program
    confirm "do you wish to exit program?"; Answer$

    if Answer$ = "yes" then
        if CoeffFlag = 1 then
            close #coeff
            CoeffFlag = 0
        end if
        if ChoiceFlag = 1 then
            close #choice
            ChoiceFlag = 0
        end if
        if SaveDataFlag = 1 then
            close #SaveData
            SaveDataFlag = 0
        end if
        close #reduc
        if ViewStarDataFlag = 1 then
            close #ViewStarData
            ViewStarDataFlag = 0
        end if
    else
        wait
    end if
END
'
'=====control buttons and control boxes
'
[Reduction_Table.click]
    #reduc.Table "selection? selected$"
wait

[Select_Comp.click]
    print #reduc.Comp, "selection? selectedComp$"

    RAComp = 0
    DECComp = 0
                                            'get RA and DEC info of comp star from Star Data file
    for DataIndex = 1 to DataIndexMax
        if selectedComp$ = left$(SDitem$(DataIndex,1)+"           ",12) then
                                            'convert RA and DEC to decimal
            RAComp =  val(SDitem$(DataIndex,3)) + val(SDitem$(DataIndex,4))/60 + val(SDitem$(DataIndex,5))/3600
            RAComp = (RAComp/24) * 360      'convert RA to degrees
            DECComp = abs(val(right$(SDitem$(DataIndex,6),2))) +_
                      val(SDitem$(DataIndex,7))/60 + val(SDitem$(DataIndex,8))/3600
                                            'see if there are any minus signs in DECd and make DECComp negative if so
            if left$(SDitem$(DataIndex,6),1) = "-" then
                DECComp = DECComp * -1
            end if
                                            'get standard V mag, B-V,U-B,V-R & V-I index for selected comparison star
            CompJ  = val(SDitem$(DataIndex,9))
            CompJH = val(SDitem$(DataIndex,10))

            exit for
        end if
    next
    if (RAComp = 0) AND (DECComp = 0) then
        notice "could not find selected Comp Star in Star Data file"
        wait
    end if
    StartComp.Flag = 1
    if StartComp.Flag = 1 and StartVar.Flag = 1 then
        print #reduc.Start, "!Enable"
    end if

    open "IREX.txt" for append as #IREX
    print #IREX, "RAComp     DECComp    CompJ     CompJH"
    print #IREX, using("###.####", RAComp);"  ";_
                 using("###.####", DECComp);"  ";_
                 using("###.###", CompJ);"   ";_
                 using("###.###", CompJH)

    print #IREX, " "
    close #IREX

wait

[Select_Var.click]
    print #reduc.Variable, "selection? selectedVar$"

    RAVar = 0
    DECVar = 0
                                           'get RA and DEC info of variable star from Star Data file
    for DataIndex = 1 to DataIndexMax
        if selectedVar$ = left$(SDitem$(DataIndex,1)+"           ",12) then
                                           'convert RA and DEC to decimal
            RAVar =  val(SDitem$(DataIndex,3)) + val(SDitem$(DataIndex,4))/60 + val(SDitem$(DataIndex,5))/3600
            RAVar = (RAVar/24) * 360       'convert RA to degrees
            DECVar = abs(val(SDitem$(DataIndex,6)) + val(SDitem$(DataIndex,7))/60 + val(SDitem$(DataIndex,8))/3600)

                                            'see if there are any minus signs in DECd and make DECVar negative if so
            if mid$(SDitem$(DataIndex,6),1,1) = "-" OR_
               mid$(SDitem$(DataIndex,6),2,1) = "-" OR_
               mid$(SDitem$(DataIndex,6),3,1) = "-" then
                DECVar = DECVar * -1
            end if
            exit for
        end if
    next
    if (RAVar = 0) AND (DECVar = 0) then
        notice "could not find selected Variable Star in Star Data file"
        wait
    end if
    StartVar.Flag = 1
    if StartComp.Flag = 1 and StartVar.Flag = 1 then
        print #reduc.Start, "!Enable"
    end if
    if CompFlag = 0 and StartVar.Flag = 1 then
        print #reduc.Start, "!Enable"
    end if
wait

[Start_Reduction]
    open "IREX.txt" for append as #IREX
        print #IREX, "output from [Create_Reduction_Table]"
        print #IREX, "ReducIndex  star          type    J2000      net count     X      filter"
        print #reduc.Start, "wait"
        gosub [Create_Reduction_Table]
        print #reduc.Start, "Start"
        print #IREX, " "
    close #IREX
wait
'
'=====subroutines for extracting data from opened raw file
'
[Convert_RawFile]       'extract individual data items from each indexed data line
    For RawIndex = 5 to RawIndexMax                                'skip header and get all raw data
        if len(RawData$(RawIndex)) < 70 then                       'delete any extra junk at end of file
            RawIndexMax = RawIndex-1
            exit for
        end if
        RDitem$(RawIndex,1)  = mid$(RawData$(RawIndex),1,2)        'UT month
        RDitem$(RawIndex,2)  = mid$(RawData$(RawIndex),4,2)        'UT day
        RDitem$(RawIndex,3)  = mid$(RawData$(RawIndex),7,4)        'UT year
        RDitem$(RawIndex,4)  = mid$(RawData$(RawIndex),12,2)       'UT hour
        RDitem$(RawIndex,5)  = mid$(RawData$(RawIndex),15,2)       'UT minute
        RDitem$(RawIndex,6)  = mid$(RawData$(RawIndex),18,2)       'UT second
        RDitem$(RawIndex,7)  = mid$(RawData$(RawIndex),21,1)       'catalog: C, V, Q, SKY, SKYNEXT or SKYLAST
        RDitem$(RawIndex,8)  = mid$(RawData$(RawIndex),26,12)      'star name
        RDitem$(RawIndex,9)  = mid$(RawData$(RawIndex),41,1)       'filter: J H
        RDitem$(RawIndex,10) = mid$(RawData$(RawIndex),44,5)       'Count 1
        RDitem$(RawIndex,11) = mid$(RawData$(RawIndex),51,5)       'Count 2
        RDitem$(RawIndex,12) = mid$(RawData$(RawIndex),58,5)       'Count 3
        RDitem$(RawIndex,13) = mid$(RawData$(RawIndex),65,5)       'Count 4
        RDitem$(RawIndex,14) = mid$(RawData$(RawIndex),72,2)       'integration time in seconds: 1 or 10
        RDitem$(RawIndex,15) = mid$(RawData$(RawIndex),75,3)       'scale: 1, 10 or 100

        print #IREX, RDitem$(RawIndex,1);"  ";_
            RDitem$(RawIndex,2);"  ";_
            RDitem$(RawIndex,3);"  ";_
            RDitem$(RawIndex,4);"  ";_
            RDitem$(RawIndex,5);"  ";_
            RDitem$(RawIndex,6);"  ";_
            RDitem$(RawIndex,7);"  ";_
            RDitem$(RawIndex,8);"  ";_
            RDitem$(RawIndex,9);"  ";_
            RDitem$(RawIndex,10);"  ";_
            RDitem$(RawIndex,11);"  ";_
            RDitem$(RawIndex,12);"  ";_
            RDitem$(RawIndex,13);"  ";_
            RDitem$(RawIndex,14);"  ";_
            RDitem$(RawIndex,15)
    next
return
'
[Total_Count_RawFile]   'compute the total averaged count and scale for gain and integration settings
    For RawIndex = 5 to RawIndexMax
        CountSum =  val(RDitem$(RawIndex,10)) +_                    'Count 1
                    val(RDitem$(RawIndex,11)) +_                    'Count 2
                    val(RDitem$(RawIndex,12)) +_                    'Count 3
                    val(RDitem$(RawIndex,13))                       'Count 4
        Divider = 4                                                 'find divider to get average of Count
        if val(RDitem$(RawIndex,13)) = 0 then
            Divider = Divider - 1
        end if
        if val(RDitem$(RawIndex,12)) = 0 then
            Divider = Divider - 1
        end if
        if val(RDitem$(RawIndex,11)) = 0 then
            Divider = Divider - 1
        end if
        if val(RDitem$(RawIndex,10)) = 0 then
            ' do something to prevent division by 0
        end if
        Integration = val(RDitem$(RawIndex,14))                     'find integration time :1 or 10 seconds
        Scale = val(RDitem$(RawIndex,15))                           'find scale factor: 1, 10 or 100
        CountFinal(RawIndex) = int((CountSum/Divider) * (1000/(Integration * Scale)))
    next
return
'
[Julian_Day_RawFile]    'convert UT time and date to Julian, epcoh J2000
    For RawIndex = 5 to RawIndexMax

        UTyear =  val(RDitem$(RawIndex,3))
        UTmonth = val(RDitem$(RawIndex,1))

        If (UTmonth = 1) or (UTmonth = 2) then
            UTmonth = UTmonth + 12
            UTyear = UTyear - 1
        end if

                                'A = int(UTyear/100)
        A = int(UTyear/100)
        B = 2 - A + int(A/4)
                                'C = int(365.25 * UTyear)
        C = int(365.25 * UTyear)
                                'D = int(30.6001 *(UTmonth + 1))
        D = int(30.6001 * (UTmonth + 1))
                                'JD = B + C + D - 730550.5 + UTday + (UThours + UTmin/60 + UTsec/3600)/24
        JD(RawIndex) = B + C + D - 730550.5 + val(RDitem$(RawIndex,2)) +_
                       (val(RDitem$(RawIndex,4)) + val(RDitem$(RawIndex,5))/60 + val(RDitem$(RawIndex,6))/3600)/24

                                'Julian century
        JT(RawIndex) = JD(RawIndex)/36525
    next
return
'
[IREX_RawFile]      'subtract skies from comp star data
    for RawIndex = 5 to RawIndexMax
                    'go through raw file and pick out only the COMP stars with the selected filter B or V
        if ((RDitem$(RawIndex,7) = "C") OR (RDitem$(RawIndex,7) = "V")) OR (RDitem$(RawIndex,7) = "Q")_
            AND (RDitem$(RawIndex,9) = Filter$) then

            if (RDitem$(RawIndex,7) = "C") then
                    'make list of  comparison stars for COMP combobox
                CompFlag = 0
                for CompIndex = 1 to CompIndexMax          'see if the Comp Star is new to the list
                    if CompStar$(CompIndex) = RDitem$(RawIndex,8) then
                        CompFlag = 1
                    end if
                next
                if CompFlag = 0 then                      'add the new Comp Star to the list
                    CompIndexMax = CompIndexMax + 1
                    CompStar$(CompIndex) = RDitem$(RawIndex,8)
                end if
            else
                    'make list of variable stars for Variable combobox
                VarFlag = 0
                for VarIndex = 1 to VarIndexMax
                    if VarStar$(VarIndex) = RDitem$(RawIndex,8) then
                        VarFlag = 1
                    end if
                next
                if VarFlag = 0 then
                    VarIndexMax = VarIndexMax + 1
                    VarStar$(VarIndex) = RDitem$(RawIndex,8)
                end if
            end if

                    'find the past sky count to apply
            SkyPastCount = 0
            PastTime = 0
            for I = RawIndex to 5 step -1
                if (RDitem$(I,8) = "SKY         ") AND (RDitem$(I,9) = Filter$) OR_
                   (RDitem$(I,8) = "SKYNEXT     ") AND (RDitem$(I,9) = Filter$) then
                    SkyPastCount = CountFinal(I)
                    PastTime = JD(I)
                    exit for
                end if
            next
                    'find the future sky count to apply
            SkyFutureCount = 0
            FutureTime = 0
            for I = RawIndex to RawIndexMax step 1
                if (RDitem$(I,8) = "SKY         ") AND (RDitem$(I,9) = Filter$) OR_
                   (RDitem$(I,8) = "SKYLAST     ") AND (RDitem$(I,9) = Filter$) then
                    SkyFutureCount = CountFinal(I)
                    FutureTime = JD(I)
                    exit for
                end if
            next
                    'subtract sky from star count depending on SKY, SKYNEXT or SKYLAST protocol and change
                    'CountFinal value accordingly
            select case
                case (SkyPastCount = 0) AND (SkyFutureCount = 0)
                    notice "no SKY counts for  star at line# "; RawIndex
                    wait
                case (SkyPastCount > 0) AND (SkyFutureCount = 0)
                    CountFinal(RawIndex) = CountFinal(RawIndex) - SkyPastCount
                case (SkyPastCount = 0) AND (SkyFutureCount > 0)
                    CountFinal(RawIndex) = CountFinal(RawIndex) - SkyFutureCount
                case else
                        '                           y2 - y1
                        'interpolation:   y = y1 + --------- * (x - x1)
                        '                           x2 - x1
                    SkyCurrentCount = SkyPastCount + ((SkyFutureCount - SkyPastCount)/(FutureTime - PastTime))*_
                                      (JD(RawIndex) - PastTime)
                    CountFinal(RawIndex) = CountFinal(RawIndex) - SkyCurrentCount
            end select

            print #IREX, RDitem$(RawIndex,8);" ";_
                         using("#####.#####",JD(RawIndex));"  ";_
                         using("#######", CountFinal(RawIndex));"       ";_
                         Filter$;"   ";_
                         using("#######", SkyPastCount);"   ";_
                         using("#######",SkyFutureCount)
        end if
    next
return
'
[Make_CompVarCheck_List]
    StarDataIndex = 0
    For CompIndex = 1 to CompIndexMax
        For DataIndex = 1 to DataIndexMax
            if CompStar$(CompIndex) = left$(SDitem$(DataIndex,1)+"           ",12) then
                StarDataIndex = 1 + StarDataIndex
                StarData$(StarDataIndex) = left$(SDitem$(DataIndex,1)+"           ",12)+"  "+_
                                           SDitem$(DataIndex,2)+"   "+_
                                           using("##",val(SDitem$(DataIndex,3)))+" "+_
                                           using("##",val(SDitem$(DataIndex,4)))+" "+_
                                           using("##",val(SDitem$(DataIndex,5)))+" "+_
                                           using("###",val(SDitem$(DataIndex,6)))+" "+_
                                           using("##",val(SDitem$(DataIndex,7)))+" "+_
                                           using("##",val(SDitem$(DataIndex,8)))+" "+_
                                           using("##.##",val(SDitem$(DataIndex,9)))+" "+_
                                           using("##.##",val(SDitem$(DataIndex,10)))

            end if
        next
    next
    For VarIndex = 1 to VarIndexMax
        For DataIndex = 1 to DataIndexMax
            if VarStar$(VarIndex) = left$(SDitem$(DataIndex,1)+"           ",12) then
                StarDataIndex = 1 + StarDataIndex
                StarData$(StarDataIndex) = left$(SDitem$(DataIndex,1)+"           ",12)+"  "+_
                                           SDitem$(DataIndex,2)+"   "+_
                                           using("##",val(SDitem$(DataIndex,3)))+" "+_
                                           using("##",val(SDitem$(DataIndex,4)))+" "+_
                                           using("##",val(SDitem$(DataIndex,5)))+" "+_
                                           using("###",val(SDitem$(DataIndex,6)))+" "+_
                                           using("##",val(SDitem$(DataIndex,7)))+" "+_
                                           using("##",val(SDitem$(DataIndex,8)))+" "+_
                                           using("##.##",val(SDitem$(DataIndex,9)))+" "+_
                                           using("##.##",val(SDitem$(DataIndex,10)))

            end if
        next
    next
    StarDataIndexMax = StarDataIndex
return
'
'=====subroutines for creating estinction table
'
                                          'create ReducData$() for display in listbox
[Create_Reduction_Table]
    redim ReducData$(2000)                 'string data lines for reduction table in list box

    redim ReducStar$(2000)                 'comp or variable star name
    redim ReducType$(2000)                 'C or V
    redim ReducJD(2000)                    'Julean date of observation, epoch 2000
    redim ReducCount(2000)                 'net count, average of (star - sky)

    redim ReducDelta.j(2000)               'differential j instrument magnitude
    redim ReducDelta.h(2000)               'differential h instrument magnitude


    redim ReducDelta.jo(2000)              'differential jo instrument magnitude
    redim ReducDelta.ho(2000)              'differential ho instrument magnitude

    redim ReducDelta.J(2000)               'differential J standard magnitude, equation 2.48
    redim ReducDelta.JH(2000)              'differential J-H standard magnitude, equation 2.49

    redim ReducJ(2000)                     'standard J magnitude, equation 2.51
    redim ReducJH(2000)                    'standard J-H index, equation 2.52

    redim ReducX(2000)                     'air mass for comp or variable
    redim ReducFilter$(2000)               'filter, J H

    FilterFlagJ = 0                        'reset all filter flags to 0 or false
    FilterFlagH = 0

    gosub [Reduction_Table_Step1]                   'get star names, net count, date, RA, DEC and airmass

    if FilterFlagJ >= 3 AND CompFlag = 1 then
        print #reduc.statictext10, chr$(68)
        print #reduc.statictext8, chr$(68)
        gosub [Reduction_Table_Step2J]
    end if
    if FilterFlagJ >= 3 AND CompFlag = 0 then
        print #reduc.statictext10, ""
        print #reduc.statictext8, ""
        gosub [Reduction_Table_AllSky_Step2J]
    end if
    if FilterFlagH >= 3 AND CompFlag = 1 then
        gosub [Reduction_Table_Step2H]
    end if
    if FilterFlagH >= 3 AND CompFlag = 0 then
        gosub [Reduction_Table_AllSky_Step2H]
    end if

    if FilterFlagJ >= 3 AND FilterFlagH >= 3 AND CompFlag = 1 then
        gosub [Reduction_Table_Step3JH]             'associate the jo and ho mags with each other and calculate results
    end if
    if FilterFlagJ >= 3 AND FilterFlagH >= 3 AND CompFlag = 0 then
        gosub [Reduction_Table_AllSky_Step3JH]      'associate the jo and ho mags with each other and calculate results
    end if

    gosub [Report_Results]                          'send average magnitudes and error to Avg. Standard Magnitude textbox

    for ReducIndex = 1 to ReducIndexMax             'write data items to the data string array and display
        ReducData$(ReducIndex) = ReducStar$(ReducIndex)+"  "+_
                                 ReducType$(ReducIndex)+"      "+_
                                 using("####.####",ReducJD(ReducIndex))+"   "+_
                                 using("######",ReducCount(ReducIndex))+"    "+_
                                 ReducFilter$(ReducIndex)+"    "

                                 if ReducDelta.j(ReducIndex) = 0 then
                                    Temp$ = ". . . . . "
                                 else
                                    Temp$ = using("###.##",ReducDelta.j(ReducIndex))+" "
                                 end if
                                 ReducData$(ReducIndex) = ReducData$(ReducIndex)+Temp$

                                 if ReducDelta.h(ReducIndex) = 0 then
                                    Temp$ = ". . . . . "
                                 else
                                    Temp$ = using("###.##",ReducDelta.h(ReducIndex))+" "
                                 end if
                                 ReducData$(ReducIndex) = ReducData$(ReducIndex)+Temp$ 


                                 if ReducJ(ReducIndex) = 0 then
                                    Temp$ = ". . . . . "
                                 else
                                    Temp$ = ". . "+using("###.##",ReducJ(ReducIndex))+" "
                                 end if
                                 ReducData$(ReducIndex) = ReducData$(ReducIndex)+Temp$

                                 if ReducJH(ReducIndex) = 0 then
                                    Temp$ = ". . . . . "
                                 else
                                    Temp$ = ". . "+using("###.##",ReducJH(ReducIndex))
                                 end if
                                 ReducData$(ReducIndex) = ReducData$(ReducIndex)+Temp$

    next
    print #reduc.Table, "reload"
return
'
[Reduction_Table_Step1]             'get stars, type, date and counts - compute airmass
    ReducIndex = 0
    for RawIndex = 5 to RawIndexMax
        if (selectedComp$ = RDitem$(RawIndex,8)) OR (selectedVar$ = RDitem$(RawIndex,8)) then
            ReducIndex = ReducIndex + 1
            ReducStar$(ReducIndex) = RDitem$(RawIndex,8)        'star name
            ReducType$(ReducIndex) = RDitem$(RawIndex,7)        'type, C or V
            ReducJD(ReducIndex)    = JD(RawIndex)               'Julean date for observation
            ReducCount(ReducIndex) = CountFinal(RawIndex)       'net count for observation

            if ReducType$(ReducIndex) = "C" then                'get the RA and DEC for comp or var
                RA = RAComp
                DEC = DECComp
            else
                RA = RAVar
                DEC = DECVar
            end if
            gosub [Siderial_Time]
            gosub [Find_Air_Mass]                               'compute airmass X
            ReducX(ReducIndex) = AirMass                        'airmass for comp or var

            ReducFilter$(ReducIndex) = RDitem$(RawIndex,9)      'filter, J H

            if RDitem$(RawIndex,9) = "J" then                   'see what filters are used in data set and
                FilterFlagJ = 1 + FilterFlagJ                   'set flag to true.
            end if                                              'would need 3 readings minimum for data to be good
            if RDitem$(RawIndex,9) = "H" then
                FilterFlagH = 1 + FilterFlagH
            end if

            print #IREX, using("####",ReducIndex);"        ";_  'send intermediate results to IREX file for diagnostic
                         ReducStar$(ReducIndex);"   ";_
                         ReducType$(ReducIndex);"    ";_
                         using("####.####",ReducJD(ReducIndex));"      ";_
                         using("######",ReducCount(ReducIndex));"   ";_
                         using("##.###",ReducX(ReducIndex));"     ";_
                         ReducFilter$(ReducIndex)
        end if
    next
    ReducIndexMax = ReducIndex
    print #IREX, " "
return
'
'
[Reduction_Table_Step2J]                        'calculate j and jo

    print #IREX, "Start of step 2J"
    print #IREX, "ReducIndex  AirMassBack   AirMassForward   ReducDelta.j   ReducDelta.jo    CompCountAvg"

    for ReducIndex = 1 to ReducIndexMax
                                                'find the variable star with filter = J
        if (ReducType$(ReducIndex)) = "V" OR (ReducType$(ReducIndex)) = "Q"_
            AND (ReducFilter$(ReducIndex)) = "J" then
                                                'get the back and forward comp star counts and air masses
            for I = ReducIndex to 1 step -1
                if (ReducType$(I) = "C") AND (ReducFilter$(I) = "J") then
                    CompCountBack = ReducCount(I)
                    AirMassBack = ReducX(I)
                    exit for
                end if
            next
            for I = ReducIndex to ReducIndexMax step 1
                if (ReducType$(I) = "C") AND (ReducFilter$(I) = "J") then
                    CompCountForward = ReducCount(I)
                    AirMassForward = ReducX(I)
                    exit for
                end if
            next
            CompCountAvg = (CompCountBack + CompCountForward)/2
            AirMassAvg = (AirMassBack + AirMassForward)/2
                                                'calculate delta j using equation 2.35
            ReducDelta.j(ReducIndex) = -1.0857 * log(ReducCount(ReducIndex)/CompCountAvg)
                                                'calculate delta jo using equation 2.43
            ReducDelta.jo(ReducIndex) = ReducDelta.j(ReducIndex) - KJ * (ReducX(ReducIndex) - AirMassAvg)

            print #IREX, using("####",ReducIndex);"        ";_
                         using("###.###",AirMassBack);"         ";_
                         using("###.###",AirMassForward);"       ";_
                         using("####.###",ReducDelta.j(ReducIndex));"       ";_
                         using("####.###",ReducDelta.jo(ReducIndex));"          ";_
                         using("########",CompCountAvg)
        end if
    next
        print #IREX, " "
return
'
[Reduction_Table_AllSky_Step2J]
    J2000.Total = 0
    J2000.Divider = 0
    print #IREX, "Start of AllSky step 2J"
    print #IREX, "ReducIndex  ReducX      Reduc.j     Reduc.jo "
    for ReducIndex = 1 to ReducIndexMax
                                                'find the variable star with filter = J
        if (ReducType$(ReducIndex)) = "V" OR (ReducType$(ReducIndex)) = "Q" AND (ReducFilter$(ReducIndex)) = "J" then
                                                'sum the J2000 dates for j magnitude time to calculate mean J2000 date
            J2000.Total = J2000.Total + ReducJD(ReducIndex)
            J2000.Divider = J2000.Divider + 1

                                                'calculate delta v using equation 2.35
            ReducDelta.j(ReducIndex) = -1.0857 * log(ReducCount(ReducIndex))
                                                'calculate delta vo using equation 2.43

            ReducDelta.jo(ReducIndex) = ReducDelta.j(ReducIndex) - KJ * (ReducX(ReducIndex))


            print #IREX, using("####",ReducIndex);"      ";_
                         using("###.###",ReducX(ReducIndex));"      ";_
                         using("####.###",ReducDelta.j(ReducIndex));"     ";_
                         using("####.###",ReducDelta.jo(ReducIndex))
        end if
    next
    J2000.Mean = J2000.Total/J2000.Divider      'mean J2000 date for v magnitude measurements
    print #IREX, " "
return
'
[Reduction_Table_Step2H]                        'calculate mean julian date, j and  jo
    J2000.Total = 0
    J2000.Divider = 0
    print #IREX, "Start of step 2H"
    print #IREX, "ReducIndex  AirMassBack   AirMassForward   ReducDelta.h   ReducDelta.ho    CompCountAvg"
    for ReducIndex = 1 to ReducIndexMax
                                                'find the variable star with filter = H
        if (ReducType$(ReducIndex)) = "V" OR (ReducType$(ReducIndex)) = "Q"_
            AND (ReducFilter$(ReducIndex)) = "H" then
                                                'sum the J2000 dates for v magnitude time to calculate mean J2000 date
            J2000.Total = J2000.Total + ReducJD(ReducIndex)
            J2000.Divider = J2000.Divider + 1
                                                'get the back and forward comp star counts and air masses
            for I = ReducIndex to 1 step -1
                if (ReducType$(I) = "C") AND (ReducFilter$(I) = "H") then
                    CompCountBack = ReducCount(I)
                    AirMassBack = ReducX(I)
                    exit for
                end if
            next
            for I = ReducIndex to ReducIndexMax step 1
                if (ReducType$(I) = "C") AND (ReducFilter$(I) = "H") then
                    CompCountForward = ReducCount(I)
                    AirMassForward = ReducX(I)
                    exit for
                end if
            next
            CompCountAvg = (CompCountBack + CompCountForward)/2
            AirMassAvg = (AirMassBack + AirMassForward)/2
                                                'calculate delta v using equation 2.35
            ReducDelta.h(ReducIndex) = -1.0857 * log(ReducCount(ReducIndex)/CompCountAvg)
                                                'calculate delta vo using equation 2.43
            ReducDelta.ho(ReducIndex) = ReducDelta.h(ReducIndex) - KH * (ReducX(ReducIndex) - AirMassAvg)

            print #IREX, using("####",ReducIndex);"        ";_
                         using("###.###",AirMassBack);"         ";_
                         using("###.###",AirMassForward);"       ";_
                         using("####.###",ReducDelta.h(ReducIndex));"       ";_
                         using("####.###",ReducDelta.ho(ReducIndex));"          ";_
                         using("########",CompCountAvg)
        end if
    next
    J2000.Mean = J2000.Total/J2000.Divider      'mean J2000 date for v magnitude measurements

    print #IREX, " "
return
'
[Reduction_Table_AllSky_Step2H]
    print #IREX, "Start of AllSky step2H"
    print #IREX, "ReducIndex  ReducX      Reduc.h      Reduc.ho"

    for ReducIndex = 1 to ReducIndexMax
        if (ReducType$(ReducIndex)) = "V" OR (ReducType$(ReducIndex)) = "Q" AND (ReducFilter$(ReducIndex)) = "H" then
            ReducDelta.h(ReducIndex) = -1.0857 * log(ReducCount(ReducIndex))

            ReducDelta.ho(ReducIndex) = ReducDelta.h(ReducIndex) - KH * ReducX(ReducIndex)

            print #IREX, using("####",ReducIndex);"      ";_
                         using("###.###",ReducX(ReducIndex));"      ";_
                         using("####.###",ReducDelta.h(ReducIndex));"     ";_
                         using("####.###",ReducDelta.ho(ReducIndex))
        end if
    next
    print #IREX, " "
return
'
[Reduction_Table_Step3JH]                       'reduction for j and j-h

    print #IREX, "Start of step 3JH"
    print #IREX, "ReducIndex  AirMassAverage.Comp   delta.jh   delta.joho  ReducDelta.JH  ReducDelta.J"

    direction = 1
    for ReducIndex = 1 to ReducIndexMax
                                                'associate the delta j and h to each other
        if (ReducDelta.j(ReducIndex) <> 0) AND (direction = 1) then
            select case
                case ReducDelta.h(ReducIndex + 1) <> 0
                    direction = 1
                    delta.jh = ReducDelta.j(ReducIndex) - ReducDelta.h(ReducIndex + 1)
                    delta.jo = ReducDelta.jo(ReducIndex)
                    delta.jo.index = ReducIndex
                case ReducDelta.h(ReducIndex - 1) <> 0
                    direction = 0
                case else
                    notice "cannot associate delta j to a delta r"
                    wait
            end select
        end if
        if (ReducDelta.j(ReducIndex) <> 0) AND (direction = 0) then
            select case
                case ReducDelta.h(ReducIndex - 1) <> 0
                    direction = 0
                    delta.jh = ReducDelta.j(ReducIndex) - ReducDelta.h(ReducIndex - 1)
                    delta.jo = ReducDelta.jo(ReducIndex)
                    delta.jo.index = ReducIndex
                case ReducDelta.h(ReducIndex + 1) <> 0
                    direction = 1
                case else
                    notice "cannot associate delta j to a delta r"
                    wait
            end select
        end if
                                                    'get the back and forward comp air masses
        for I = ReducIndex to 1 step -1
            if (ReducType$(I) = "C") AND (ReducFilter$(I) = "H") then
                AirMassBack.H = ReducX(I)
                exit for
            end if
        next
        for I = ReducIndex to ReducIndexMax step 1
            if (ReducType$(I) = "C") AND (ReducFilter$(I) = "H") then
                AirMassForward.H = ReducX(I)
                exit for
            end if
        next
        for I = ReducIndex to 1 step -1
            if (ReducType$(I) = "C") AND (ReducFilter$(I) = "J") then
                AirMassBack.J = ReducX(I)
                exit for
            end if
        next
        for I = ReducIndex to ReducIndexMax step 1
            if (ReducType$(I) = "C") AND (ReducFilter$(I) = "J") then
                AirMassForward.J = ReducX(I)
                exit for
            end if
        next

        if (ReducDelta.h(ReducIndex) <> 0) then
            AirMassAverage.Comp = (AirMassBack.H + AirMassForward.H + AirMassBack.J + AirMassForward.J)/4
                                                'instrument delta(j-h)o using equation 2.46
            delta.joho = delta.jh - (KJ - KH) * (ReducX(ReducIndex) - AirMassAverage.Comp) -_
                         KKjh * delta.jh * ((ReducX(ReducIndex) + AirMassAverage.Comp)/2)
                                                'standard delta(J-H) using equation 2.49
            ReducDelta.JH(ReducIndex) = Lambda * delta.joho

                                                'caculate standard J if using epsilon from J-H
                                                'standard delta J using equation 2.48
            ReducDelta.J(delta.jo.index) = delta.jo + Rho * ReducDelta.JH(ReducIndex)
                                                'standard J magnitude using equation 2.51
            ReducJ(delta.jo.index) = CompJ + ReducDelta.J(delta.jo.index)

                                                'standard J-H index using equation 2.52
            ReducJH(ReducIndex) = CompJH + ReducDelta.JH(ReducIndex)

            print #IREX, using("####",ReducIndex);"         ";_
                         using("###.###",AirMassAverage.Comp);"             ";_
                         using("###.###",delta.jh);"     ";_
                         using("####.###",delta.joho);"     ";_
                         using("####.###",ReducDelta.JH(ReducIndex));"       ";_
                         using("####.###",ReducDelta.J(delta.jo.index))
        end if

    next
    print #IREX, " "
return
'
[Reduction_Table_AllSky_Step3JH]

    print #IREX, "Start of AllSky step 3JH"
    print #IREX, "ReducIndex  ReducX   delta.jh   delta.joho   Reduc.JH    Reduc.J"

    direction = 1
    for ReducIndex = 1 to ReducIndexMax
                                                'associate the delta b and v to each other
        if (ReducDelta.h(ReducIndex) <> 0) AND (direction = 1) then
            select case
                case ReducDelta.j(ReducIndex + 1) <> 0
                    direction = 1
                    delta.jh = ReducDelta.j(ReducIndex + 1) - ReducDelta.h(ReducIndex)
                    delta.jo = ReducDelta.jo(ReducIndex + 1)
                    delta.jo.index = ReducIndex + 1
                case ReducDelta.j(ReducIndex - 1) <> 0
                    direction = 0
                case else
                    notice "cannot associate delta j to a delta h"
                    wait
            end select
        end if
        if (ReducDelta.h(ReducIndex) <> 0) AND (direction = 0) then
            select case
                case ReducDelta.j(ReducIndex - 1) <> 0
                    direction = 0
                    delta.jh = ReducDelta.j(ReducIndex - 1) - ReducDelta.h(ReducIndex)
                    delta.jo = ReducDelta.jo(ReducIndex - 1)
                    delta.jo.index = ReducIndex - 1
                case ReducDelta.j(ReducIndex + 1) <> 0
                    direction = 1
                case else
                    notice "cannot associate delta j to a delta h"
                    wait
            end select
        end if

        if (ReducDelta.h(ReducIndex) <> 0) then

                                                    'calculate J-H using equation 2.10
            delta.joho = delta.jh - (KJ - KH) * (ReducX(ReducIndex)) -_
                         KKjh * delta.jh * (ReducX(ReducIndex))

            ReducJH(ReducIndex) = Lambda * delta.joho + ZPjh

            ReducJ(delta.jo.index) = delta.jo + Rho * ReducJH(ReducIndex) + ZPj

                    print #IREX, using("####",ReducIndex);"      ";_
                         using("###.###",ReducX(ReducIndex));"    ";_
                         using("###.###",delta.jh);"    ";_
                         using("####.###",delta.joho);"   ";_
                         using("####.###",ReducJH(ReducIndex));"    ";_
                         using("####.###",ReducJ(delta.jo.index))
        end if
    next
    print #IREX, "Rho: ";using("###.###",Rho);"   ";"Lambda: ";using("###.###",Lambda)
    print #IREX, " "
return
'
[Report_Results]
    print #IREX, "Start of Report_Results"
                                            'compute mean J magnitude with standard deviation
    I = 0
    ReducJ.total = 0
    for ReducIndex = 1 to ReducIndexMax
        if ReducJ(ReducIndex) <> 0 then
            I = I + 1
            ReducJ.total = ReducJ.total + ReducJ(ReducIndex)
        end if
    next
    ReducJ.average = ReducJ.total/I

    N = 0
    ReducJ.deviation = 0

    print #IREX, "ReducIndex   ReducJ     ReducJ.deviation"
    for ReducIndex = 1 to ReducIndexMax
        if ReducJ(ReducIndex) <> 0 then
            N = N + 1
            ReducJ.deviation = (ReducJ(ReducIndex) - ReducJ.average)^2 + ReducJ.deviation

            print #IREX, using("####",ReducIndex)+"       "+using("###.####",ReducJ(ReducIndex))+_
                         "   "+using("###.######",ReducJ.deviation)
         end if
    next
    if N > 2 then
                                            'standard deviation of all J magnitudes using equation 3.8
        ReducJ.std.deviation = sqr(ReducJ.deviation/((N-1)))

        if CompFlag = 0 then                'add in standard error from zero points
            ReducJ.std.deviation = sqr(ReducJ.std.deviation^2 + Ej^2)
        end if
                                            'check to see of value is zero and change to +/-0.001 if it is
        TestValue = ReducJ.std.deviation
        gosub [TestZero]
        ReducJ.std.deviation = TestValue
    else
        ReducJ.std.deviation = 0
    end if

    print #IREX, " "
    print #IREX, "ReducJ.average = "+using("##.####", ReducJ.average)
    print #IREX, "ReducJ.std.deviation  "; using("###.####",ReducJ.std.deviation)

                                            'compute standard J-H if there is filter data
    if FilterFlagH >= 3 then
        I = 0
        ReducJH.total = 0
        for ReducIndex = 1 to ReducIndexMax
            if ReducJH(ReducIndex) <> 0 then
                I = I + 1
                ReducJH.total = ReducJH.total + ReducJH(ReducIndex)
            end if
        next
        ReducJH.average = ReducJH.total/I
                                            'check to see of value is zero and change to +/-0.001 if it is
        TestValue = ReducJH.average
        gosub [TestZero]
        ReducJH.average = TestValue

        N = 0
        ReducJH.deviation = 0
        for ReducIndex = 1 to ReducIndexMax
            if ReducJH(ReducIndex) <> 0 then
                N = N + 1
                ReducJH.deviation = (ReducJH(ReducIndex) - ReducJH.average)^2 + ReducJH.deviation
            end if
        next
        if N > 2 then
                                            'standard deviation of all J-H indexes using equation 3.6
            ReducJH.std.deviation = sqr(ReducJH.deviation/((N-1)))

            if CompFlag = 0 then                'add in standard error from zero points
                ReducJH.std.deviation = sqr(ReducJH.std.deviation^2 + Ejh^2)
            end if
                                            'check to see of value is zero and change to +/-0.001 if it is
            TestValue = ReducJH.std.deviation
            gosub [TestZero]
            ReducJH.std.deviation = TestValue

        else
            ReducJH.std.deviation = 0
        end if
    end if

    print #IREX, " "
    print #IREX, "ReducJH.average       "; using("###.####",ReducJH.average)
    print #IREX, "ReducJH.std.deviation "; using("###.####",ReducJH.std.deviation)

                                                'print results in Analysis textbox with errors
    Temp$ = using("####.###",ReducJ.average)+chr$(177)+_
            using("#.###",ReducJ.std.deviation)+chr$(13)+chr$(10)

    if FilterFlagH >= 3 then
        Temp$ = Temp$ + using("####.###",ReducJH.average)+chr$(177)+_
                                using("#.###",ReducJH.std.deviation)+chr$(13)+chr$(10)
    else
        Temp$ = Temp$ + " - - - - - - - "+chr$(13)+chr$(10)
    end if

    JDlabel$ = "  JD "
    if JDFlag = 0 then
        JDtemporary = J2000.Mean
        gosub [JDtoHJD]
        J2000.Mean = JDtemporary
        JDlabel$ = " HJD "
    end if

    Temp$ = Temp$ + JDlabel$ + using("####.####",J2000.Mean)

    print #reduc.Analysis, Temp$

return
'
'
'====subroutines for file operations
'
'
[Find_File_Name]        'seperate out filename and extension from info() path/filename
    FileNameIndex = len(DataFile$)
    FileNameLength = len(DataFile$)
    while mid$(DataFile$, FileNameIndex,1)<>"\"               'look for the last backlash
        FileNameIndex = FileNameIndex - 1
    wend
    FileNamePath$ = left$(DataFile$, FileNameIndex)
    DataFileName$ = right$(DataFile$, FileNameLength-FileNameIndex)

    print #reduc.FileName, DataFileName$                      'display filename in "File" textbox

    print #IREX, DataFileName$

return
'
'=====subroutines for calculations
'
[TestZero]              'check to see if value is zero and change to +/-0.001 if necessary
    if abs(TestValue) < 0.0006 then
        if TestValue >= 0 then
            TestValue = 0.001
        else
            TestValue = -0.001
        end if
    end if
return
'
[Find_Lat_Long]         'compute observers latitude and longitude in degrees
    LAT = val(mid$(Location$,2,4))
    if upper$(left$(Location$,1)) = "S" then
        LAT = LAT * -1
    end if
    LONGITUDE = val(mid$(Location$,8,5))
    if upper$(mid$(Location$,7,1)) = "W" then
        LONGITUDE = LONGITUDE * -1
    end if
return
'
[Siderial_Time]         'compute local siderial time
    MST = 280.46061837 + 360.98564736629 * JD(RawIndex) + 0.000387933 * JT(RawIndex) *_
          JT(RawIndex) - (JT(RawIndex) * JT(RawIndex) * JT(RawIndex))/38710000
    LMST = MST + LONGITUDE
    if LMST > 0 then
        while (LMST > 360)
            LMST = LMST - 360
        wend
    else
        while (LMST < 0)
            LMST = LMST + 360
        wend
    end if
return
'
[Find_Air_Mass]                       'compute air mass
    HA = LMST - RA                    'find hour angle in degrees
    if HA < 0 then
        HA = HA + 360
    end if

    HAradians = HA * 3.1416/180       'convert to radians
    DECradians = DEC * 3.1416/180
    LATradians = LAT * 3.1416/180

                    'compute secant of zenith angle - distance from zenith in radians
    secZ = 1/(sin(LATradians) * sin(DECradians) + cos(LATradians) * cos(DECradians) * cos(HAradians))
                    'compute air mass using Hardie equation
                    'X = sec Z - 0.0018167(sec Z - 1) - 0.002875(sec Z - 1)^2 - 0.0008083(sec Z - 1)^3
    AirMass = secZ - 0.0018167 * (secZ - 1) - 0.002875 * (secZ - 1)^2 - 0.0008083 * (secZ - 1)^3
return
'
function JDtoCAL$(JD)                   'J2000 day to calendar day and time
                                        'from web page at: www.tamuk.edu/math/scott/stars/jcal.htm
    JD = JD + 2451545.0                 'add J2000 days from start at 4713 BC
    z = int(JD + 0.5)
    f = JD + 0.5 - int(JD + 0.5)
    aa = int((z - 1867216.25) / 36524.25)
    a = z + 1 + aa - int(aa / 4)
    b = a + 1524
    c = int((b - 122.1) / 365.25)
    dd = int(365.25 * c)
    e = int((b - dd) / 30.6001)
    if e < 13.5 then
        mm = e - 1
    else
        mm = e - 13
    end if
    if mm > 2.5 then
        yy = c - 4716
    else
        yy = c - 4715
    end if
    dd = b - dd - int(30.6001 * e) + f
    hr = dd - int(dd)
    dd = dd - hr
    hr = hr * 24
    mn = hr - int(hr)
    hr = hr - mn
    mn = mn * 60
    ss = mn - int(mn)
    mn = mn - ss
    ss = ss * 60
    JDtoCAL$ = using("##", dd)+"/"+using("##", mm)+"/"+using("####", yy)+"  "+_
               using("##", hr)+":"+using("##", mn)+":"+using("##", ss)
end function
'
[JDtoHJD]
                                        'from section 5.3e of Astronomical Photometry, pages 113 - 116
    T = (JDtemporary - 36525)/36525

    SolLongitude = 279.696678 + 36000.76892 * T + 0.000303 * T^2_
                     - (1.39604 + 0.000308 * (T + 0.5)) * (T - 0.499998)

    SolAnomaly = 358.475833 + 35999.04975 * T - 0.00015 * T^2

    shit = cos(SolLongitude/57.2958)

    Xjd = 0.99986 * cos(SolLongitude/57.2958) - 0.025127 * cos((SolAnomaly - SolLongitude)/57.2958)_
          + 0.008374 * cos((SolAnomaly + SolLongitude)/57.2958)_
          + 0.000105 * cos((2*SolAnomaly + SolLongitude)/57.2958)_
          + 0.000063 * T * cos((SolAnomaly - SolLongitude)/57.2958)_
          + 0.000035 * cos((2 * SolAnomaly - SolLongitude)/57.2958)

    Yjd = 0.917308 * sin(SolLongitude/57.2958) + 0.023053 * sin((SolAnomaly - SolLongitude)/57.2958)_
          + 0.007683 * sin((SolAnomaly + SolLongitude)/57.2958)_
          + 0.000097 * sin((2 * SolAnomaly + SolLongitude)/57.2958)_
          - 0.000057 * T * sin((SolAnomaly - SolLongitude)/57.2958)_
          - 0.000032 * sin((2 * SolAnomaly - SolLongitude)/57.2958)

    DeltaT = -0.0057755 * ((cos(DECVar/57.2958) * cos(RAVar/57.2958)) * Xjd _
             + (tan(0.40928) * sin(DECVar/57.2958) + cos(DECVar/57.2958) * sin(RAVar/57.2958)) * Yjd)

   JDtemporary = JDtemporary + DeltaT
return
'
end


