'                        Transformation Module - Differential Photometry JH
'                                      Optec, Inc.
'
'======version history
'
'V1.20 December 2015
'  modified PPparms2_JH for AllSky
'
'V1.10 October 2015
' compiled with LB4.50
' increased dim
' added bmp save file
'
'V1.00
'
'
'=====dimension statements and initial conditions
'
    DIM info$(10,10)
    files "c:\", info$()

' display listbox arrays
    DIM TransData$(100)                 'data lines for listbox table, #trans.Table, index var = TransIndex
        TransData$(1) = "   "           'cannot use listbox unless something is put in before creation

    DIM TransStar$(100)                 'transformation star names in raw file, index var = TransIndex

    DIM Trans.J(100)                    'J, index var = TransIndex
    DIM Trans.Jj(100)                   'J-jo, index var = TransIndex

    DIM TransStdColor(100)              'J - H standard color index, index var = TransIndex
    DIM TransColor(100)                 '(j - h)o instrument colorindex corrected for K', index var = TransIndex
    DIM TransColor2(100)                '(J - H) - (j - h)o, index var = TransIndex

    DIM RA(100)                         'RA in decimal degrees for each trans star, index var = TransIndex
    DIM DEC(100)                        'DEC in decimal derees for each trans star, index var = TransIndex

' working arrays for computing values in TransData$ listbox table for each filter
' array(TransIndex,Filter)              TransIndex: 1 to 50 for each transformation star
'                                       Filter: 1 for J, 2 for H

    DIM X(100,5)                        'air mass for each star reading and for each filter
    DIM mo(100,5)                       'instrument magnitude for each filter reduced for K', jo, ho
    DIM m(100,5)                        'instrument magnitude for each filter, j h
    DIM FilterFlag(100,5)               'flag to determine if column entry should be 0

' transformation data file array
    DIM TDitem$(2000,13)                'data items from Transformation Data file, index var = DataIndex
                                        'TDitem$(x,1)  = star name, 8 bytes long
                                        'TDitem$(x,2)  = spectral type, not really needed
                                        'TDitem$(x,3)  = RA hour
                                        'TDitem$(x,4)  = RA minute
                                        'TDitem$(x,5)  = RA second
                                        'TDitem$(x,6)  = DEC degree
                                        'TDitem$(x,7)  = DEC minute
                                        'TDitem$(x,8)  = DEC second
                                        'TDitem$(x,9)  = J magnitude
                                        'TDitem$(x,10) = J-H magnitude


' raw data file arrays
    DIM RawData$(1000)                  'data lines from raw file of observations, index var = RawIndex
    DIM RDitem$(1000,15)                'data items from raw file of observations, index var = RawIndex
                                        'RDitem$(x,1)  = UT month
                                        'RDitem$(x,2)  = UT day
                                        'RDitem$(x,3)  = UT year
                                        'RDitem$(x,4)  = UT hour
                                        'RDitem$(x,5)  = UT minute
                                        'RDitem$(x,6)  = UT second
                                        'RDitem$(x,7)  = catalog, "T" for transformation star
                                        'RDitem$(x,8)  = object, star name, SKY, SKYNEXT or SKYLAST
                                        'RDitem$(x,9)  = filter, J or H
                                        'RDitem$(x,10) = count 1
                                        'RDitem$(x,11) = count 2
                                        'RDitem$(x,12) = count 3
                                        'RDitem$(x,13) = count 4
                                        'RDitem$(x,14) = integration time, 1 or 10 seconds
                                        'RDitem$(x,15) = scale, 1 10 or 100

    DIM CountFinal(1000)                 'average count including integration and scale, index var = RawIndex
    DIM JD(1000)                         'Julean date from 2000 for each RawIndex, index var = RawIndex
    DIM JT(1000)                         'Julean century fromm 2000 for each RawIndex, index var = RawIndex

' other arrays
    DIM Xaxis(100)                       'Xaxis() transformation star standard color index J-H
    DIM Yaxis(100)                       'Yaxis() J-jo to solve for rho, or (J-H)-(j-h)o to solve for lambda
'
'=====initialize and start up values
'
VersionNumber$ = "1.20"
PathDataFile$ = "*.raw"                 'default path for data files
PathPlotFile$ = "* .bmp"                'default path for plot files
'
'=====open and read in PPparms data for configuration
'
open "PPparms2_JH.txt" for input as #PPparms
    input #PPparms, Location$                       'latitude and longitude in degrees
    input #PPparms, KJ                              'first order extinction for J
    input #PPparms, KH                              'first order extinction for H
    input #PPparms, KKjh                            'second order extinction for j-h, default = 0
    input #PPparms, Rho                             'transformation coeff. for J using J-H
    input #PPparms, Lambda                          'transformation coeff. J-H
    input #PPparms, JDFlag                          '1 if using JD and 0 if using HJD
    input #PPparms, OBSCODE$                        'AAVSO observatory code
    input #PPparms, Spare2                          '0 not used at this time
    input #PPparms, ZPj                             'zero-point constant for j
    input #PPparms, ZPjh                            'zero-point constant for j-h
    input #PPparms, Ej                              'standard error for j
    input #PPparms, Ejh                             'standard error for j-h
close #PPparms
gosub [Find_Lat_Long]

            open "IREX.txt" for output as #IREX
                print #IREX, "Transformation JH module"
                print #IREX, " "
                print #IREX, "PPparms2 JH.txt"
                print #IREX, "  Location$     ";Location$
                print #IREX, "  KJ            ";KJ
                print #IREX, "  KH            ";KH
                print #IREX, "  KKjh          ";KKjh
                print #IREX, "  Rho           ";Rho
                print #IREX, "  Lambda        ";Lambda
                print #IREX, "  JDFlag        "; JDFlag
                print #IREX, "  OBSCODE       "; OBSCODE$
                print #IREX, "  Spare2        "; Spare2
                print #IREX, "  ZPj           "; ZPj
                print #IREX, "  ZPjh          "; ZPjh
                print #IREX, "  Ej            "; Ej
                print #IREX, "  Ejh           "; Ejh
                print #IREX, " "
            close #IREX
'
'======open and read in Star Data
'
open "Transformation Database JH.txt" for input as #TransData
    DataIndex = 0
    while eof(#TransData) = 0
        DataIndex = DataIndex + 1
        input #TransData,TDitem$(DataIndex,1),_      'star name
                         TDitem$(DataIndex,2),_      'spectral type
                         TDitem$(DataIndex,3),_      'RA hour
                         TDitem$(DataIndex,4),_      'RA minute
                         TDitem$(DataIndex,5),_      'RA second
                         TDitem$(DataIndex,6),_      'DEC degree
                         TDitem$(DataIndex,7),_      'DEC minute
                         TDitem$(DataIndex,8),_      'DEC second
                         TDitem$(DataIndex,9),_      'V magnitude, ##.##
                         TDitem$(DataIndex,10)       'J-H index, ##.##
    wend
    DataIndexMax = DataIndex
close #TransData
'
'=====set up main GUI control window
'
[WindowSetup]
    NOMAINWIN
    WindowWidth = 900 : WindowHeight = 457
    UpperLeftX = INT((DisplayWidth-WindowWidth)/2)
    UpperLeftY = INT((DisplayHeight-WindowHeight)/2)

[ControlSetup]
    Menu        #trans, "File",_
                        "Open File", [Open_File],_
                        "Save Plot Graphic", [Data_Save],_
                         "Quit", [Quit_Extinction]
    Menu        #trans, "Coefficients",_
                        "Load Saved Transformation Coefficients", [Load_Previous_Coeff],_
                        "Transfer Computed Coefficients to Results", [Use_Current_Coefficients],_
                        "Save Coefficients in Results to PPparms", [Save_Coefficients]
    Menu        #trans, "Help", "About", [About], "Help", [Help]

    graphicbox  #trans.graphicbox1,   400, 40, 480, 265

    groupbox    #trans.groupbox1, "Transformation Stars File", 11, 311, 300, 90
    groupbox    #trans.groupbox2, "Results", 325, 310, 185, 90
    groupbox    #trans.groupbox3, "Least-Squares Analysis", 525, 310, 355, 90

    statictext  #trans.statictext1,  "Star",    30, 20, 40, 14
    statictext  #trans.statictext2,  "X",      140, 20, 10, 14

    statictext  #trans.statictext3,  "J-j",    250, 20, 29, 14
    statictext  #trans.statictext4,  "o",      274, 23, 10, 14

    statictext  #trans.statictext5,  "(J-H)-", 308,  5, 60, 14
    statictext  #trans.statictext6,  "(j-h)",  308, 20, 44, 14
    statictext  #trans.statictext7,  "o",      351, 23, 10, 14

    statictext  #trans.statictext8,  "J-H",    195,  20, 60, 14

    statictext  #trans.statictext30,"slope", 652, 337, 50, 14
    statictext  #trans.statictext31,"intercept", 617, 352, 90, 14
    statictext  #trans.statictext32,"standard error", 575, 367, 132, 14
                                                'greek letters for results labels
    statictext  #trans.statictext33,"r", 375, 330, 12, 16       'rho
    statictext  #trans.statictext34,"l", 447, 330, 12, 16       'lambda

                                                'buttons for calculating coefficients
    button      #trans.Showrho, "rho",[Show_rho.click],UL, 525, 10, 95, 25
    button      #trans.Showlambda, "lambda",[Show_lambda.click],UL, 627, 10, 95, 25

    button      #trans.Print, "print",[DataPrint.click], UL, 810, 360, 57, 25

    textbox     #trans.rho, 350, 355, 64, 25
    textbox     #trans.lambda, 424, 355, 64, 25

    textbox     #trans.Analysis, 709, 335, 88, 50

    textbox     #trans.FileName, 19, 345, 285, 25

    listbox     #trans.Table, TransData$(),[Transformation_Table.click], 10, 40, 375, 265

    Open "Transformation Coefficients   JH Photometry" for Window as #trans
    #trans "trapclose [Quit_Extinction]"
    #trans.graphicbox1 "down; fill White; flush"
    #trans.graphicbox1 "setfocus; when mouseMove [MouseChange1]"
    #trans.Table "selectindex 1"
    #trans "font courier_new 10 14"
                                                    'select greek letters for results box
    print #trans.statictext33, "!font symbol 15 17"
    print #trans.statictext34, "!font symbol 13 17"

    print #trans.FileName, "open raw data file"

    print #trans.Showrho, "!Disable"
    print #trans.Showlambda, "!Disable"

[loop]

Wait                                'finised setting up, wait here for new command
'
'======menu controls
'
[Open_File]
'
'
    filedialog "Open Data File", PathDataFile$, DataFile$



    for I = len(DataFile$) to 1 step -1             'remember path for opened folder and file
        if mid$(DataFile$,I,1) = "\" then
            ShortDataFile$ = mid$(DataFile$,I+1)
            PathDataFile$ = left$(DataFile$,I)+"*raw"
            exit for
        end if
    next I

    if DataFile$ = "" then
        print #trans.FileName, "open raw data file"
    else
        files "c:\", DataFile$, info$()
        if val(info$(0, 0)) = 0 then
            notice "cannot create new file"
        else
            open DataFile$ for input as #RawFile
            for RawIndex = 1 to 4                   'read first 4 lines with line input to capture commas
                line input #RawFile, RawData$(RawIndex)
            next RawIndex
            RawIndex = 4
                while eof(#RawFile)=0               'read rest of data to end of file
                    RawIndex = RawIndex + 1
                    input #RawFile, RawData$(RawIndex)
                wend
                RawIndexMax = RawIndex
            close #RawFile

            open "IREX.txt" for append as #IREX
                print #IREX, "output from [Find_File_Name]"
                print #IREX, "filename"
                gosub [Find_File_Name]
                print #IREX, " "
            close #IREX

            open "IREX.txt" for append as #IREX
                print #IREX, "output from [Convert_RawFile]"
                print #IREX, "raw file data"
                gosub [Convert_RawFile]
                print #IREX, " "
            close #IREX

            gosub [Total_Count_RawFile]
            gosub [Julian_Day_RawFile]

            open "IREX.txt" for append as #IREX
                print #IREX, "output from [IREX_RawFile]"
                print #IREX, "star           Julean date  final count   filter"
                Filter$ = "J"
                gosub [IREX_RawFile]
                Filter$ = "H"
                gosub [IREX_RawFile]

                print #IREX, " "
            close #IREX

            open "IREX.txt" for append as #IREX
                print #IREX, "output from [Find_Trans_Stars]"
                print #IREX, "star       TransIndex"
                gosub [Find_Trans_Stars]
                print #IREX, " "
            close #IREX

            open "IREX.txt" for append as #IREX
                print #IREX, "output from [Get_Trans_Star_Data]"
                print #IREX, "star              RA        DEC       J      J-H"
                gosub [Get_Trans_Star_Data]
                print #IREX, " "
            close #IREX

            open "IREX.txt" for append as #IREX
                print #IREX, "output from [Create_Transformation_Table]"
                print #IREX, "star          LMST       AirMass  CntFinal  InstMag  InstMago  Filter"
                gosub [Create_Transformation_Table]
                print #IREX, " "
            close #IREX

                                                    'turn on buttons if there is data for each filter
            for TransIndex = 1 to TransIndexMax
                If FilterFlag(TransIndex,1) = 1 then
                    print #trans.Showlambda, "!Enable"
                end if
                If FilterFlag(TransIndex,2) = 1 then
                    print #trans.Showrho, "!Enable"
                end if
            next
        end if
    end if
Wait
'
[Data_Save]                                      'save bitmap image to disk
    filedialog "Save As...", PathPlotFile$, PlotFile$
    for I = len(PlotFile$) to 1 step -1
        if mid$(PlotFile$,I,1) = "\" then
            ShortPlotFile$ = mid$(PlotFile$,I+1)
            PathPlotFile$ = left$(PlotFile$,I)+"*bmp"
            exit for
        end if
    next I

    if PlotFile$ <> "" then
        print #trans.graphicbox1, "getbmp plot -2 -2 482 267"
        if (right$(PlotFile$,4) = ".bmp") OR (right$(PlotFile$,4) = ".BMP") then
            bmpsave "plot", PlotFile$
        else
            PlotFile$ = PlotFile$+".bmp"
            bmpsave "plot", PlotFile$
        end if
    end if
wait
'
[Load_Previous_Coeff]
    print #trans.lambda, using("##.###",Lambda)
    print #trans.rho, using("##.###",Rho)
Wait

[Save_Coefficients]
    print #trans.lambda, "!contents? LambdaTemp"
    print #trans.rho, "!contents? RhoTemp"

    confirm "This will save all non-zero contents"+chr$(13)_
            +"Type a zero in box to keep previous saved coefficient"+chr$(13)+chr$(13)_
            +"Do you wish to save values?"; Answer$
    if Answer$ = "yes" then
        if LambdaTemp <> 0 then
            Lambda = LambdaTemp
        end if
        if RhoTemp <> 0 then
            Rho = RhoTemp
        end if
        gosub [Write_PPparms]
    end if
Wait

[Use_Current_Coefficients]
    print #trans.lambda, using("##.###",Lambdatemporary)
    print #trans.rho, using("##.###",Rhotemporary)
Wait

[About]
    notice "Transformation Coefficients  JH Photometry "+chr$(13)+_
           "Version " +VersionNumber$+chr$(13)+_
           "copyright 2014-2015, Gerald Persha."+chr$(13)+_
           "SSPDataq.com"
Wait

[Help]
    run "hh photometry jh.chm"
Wait

[MouseChange1]
    'MouseX and MouseY contain mouse coordinates
wait

[Quit_Extinction]                   'exit program
    confirm "do you wish to exit program?"; Answer$

    if Answer$ = "yes" then
        close #trans
        END
    else
        wait
    end if
'
'=====control buttons and control boxes
'
[Show_rho.click]
    print#trans.Print, "!Enable"
    Rhotemporary = 0
    Graph$ = "rho"
    print #trans.graphicbox1, "cls"
    print #trans.graphicbox1, "color black"
    print #trans.graphicbox1, "font arial 6 12"
    print #trans.graphicbox1, "place 10 125"
    print #trans.graphicbox1, "\J-j"
    print #trans.graphicbox1, "place 28 130"
    print #trans.graphicbox1, "\o"
    print #trans.graphicbox1, "font arial 8 16"
    print #trans.graphicbox1, "place 225 250"
    print #trans.graphicbox1, "\J - H"

    gosub [Draw_Graph_Outline]

    open "IREX.txt" for append as #IREX         'save to IREX file for diagnostics
        print #IREX, "output from [Create_Regression_Array]"
        print #IREX, "Coefficient = rho"
        print #IREX, "star             Xaxis      Yaxis"
        gosub [Create_Regression_Array]
        print #IREX, " "
    close #IREX

    gosub [Solve_Regression_Matrix]
                                                'output to Analysis textbox the results
    print #trans.Analysis, using("####.###", Slope) + chr$(13) + chr$(10) +_
                           using("####.###", Intercept) + chr$(13) + chr$(10) +_
                           using("####.###", std.error)
    PointColor$ = "blue"
    print #trans.graphicbox1, "color red"
    gosub [Draw_Best_Line]

    gosub [Draw_Description]
    print #trans.graphicbox1, "flush"

    Rhotemporary = Slope
wait
'
'
[Show_lambda.click]
    print#trans.Print, "!Enable"
    Lambdatemporary = 0
    Graph$ = "lambda"
    print #trans.graphicbox1, "cls"
    print #trans.graphicbox1, "color black"
    print #trans.graphicbox1, "font arial 6 12"
    print #trans.graphicbox1, "place 2 125"
    print #trans.graphicbox1, "\(J-H)-"
    print #trans.graphicbox1, "place 6 140"
    print #trans.graphicbox1, "\(j-h)"
    print #trans.graphicbox1, "place 29 145"
    print #trans.graphicbox1, "\o"
    print #trans.graphicbox1, "font arial 8 16"
    print #trans.graphicbox1, "place 225 250"
    print #trans.graphicbox1, "\J - H"
    gosub [Draw_Graph_Outline]

    open "IREX.txt" for append as #IREX         'save to IREX file for diagnostics
        print #IREX, "output from [Create_Regression_Array]"
        print #IREX, "Coefficient = lambda"
        print #IREX, "star             Xaxis      Yaxis"
        gosub [Create_Regression_Array]
        print #IREX, " "
    close #IREX

    gosub [Solve_Regression_Matrix]
                                                'output to Analysis textbox the results
    print #trans.Analysis, using("####.###", Slope) + chr$(13) + chr$(10) +_
                           using("####.###", Intercept) + chr$(13) + chr$(10) +_
                           using("####.###", std.error)

    print #trans.graphicbox1, "color darkred"
    gosub [Draw_Best_Line]

    gosub [Draw_Description]
    print #trans.graphicbox1, "flush"
'
'               Slope = 1 - 1/mu from page 332
'               mu = 1/(1 - Slope)
    Lambdatemporary = 1/(1 - Slope)
wait
'
'
[DataPrint.click]

    printerdialog
    print #trans.graphicbox1, "Print VGA"

wait
'
[Extinction_Table.click]
    #trans.Table "selection? selected$"
wait
'
'=====subroutines for extracting data from opened raw file
'
[Convert_RawFile]       'extract individual data items from each indexed data line
    For RawIndex = 5 to RawIndexMax                                'skip header and get all raw data
        if len(RawData$(RawIndex)) < 70 then                       'delete any extra junk at end of file
            RawIndexMax = RawIndex-1
            exit for
        end if
        RDitem$(RawIndex,1)  = mid$(RawData$(RawIndex),1,2)        'UT month
        RDitem$(RawIndex,2)  = mid$(RawData$(RawIndex),4,2)        'UT day
        RDitem$(RawIndex,3)  = mid$(RawData$(RawIndex),7,4)        'UT year
        RDitem$(RawIndex,4)  = mid$(RawData$(RawIndex),12,2)       'UT hour
        RDitem$(RawIndex,5)  = mid$(RawData$(RawIndex),15,2)       'UT minute
        RDitem$(RawIndex,6)  = mid$(RawData$(RawIndex),18,2)       'UT second
        RDitem$(RawIndex,7)  = mid$(RawData$(RawIndex),21,1)       'catalog: T
        RDitem$(RawIndex,8)  = mid$(RawData$(RawIndex),26,12)      'star name
        RDitem$(RawIndex,9)  = mid$(RawData$(RawIndex),41,1)       'filter: J H
        RDitem$(RawIndex,10) = mid$(RawData$(RawIndex),44,5)       'Count 1
        RDitem$(RawIndex,11) = mid$(RawData$(RawIndex),51,5)       'Count 2
        RDitem$(RawIndex,12) = mid$(RawData$(RawIndex),58,5)       'Count 3
        RDitem$(RawIndex,13) = mid$(RawData$(RawIndex),65,5)       'Count 4
        RDitem$(RawIndex,14) = mid$(RawData$(RawIndex),72,2)       'integration time in seconds: 1 or 10
        RDitem$(RawIndex,15) = mid$(RawData$(RawIndex),75,3)       'scale: 1, 10 or 10

        print #IREX, RDitem$(RawIndex,1);"  ";_
                     RDitem$(RawIndex,2);"  ";_
                     RDitem$(RawIndex,3);"  ";_
                     RDitem$(RawIndex,4);"  ";_
                     RDitem$(RawIndex,5);"  ";_
                     RDitem$(RawIndex,6);"  ";_
                     RDitem$(RawIndex,7);"  ";_
                     RDitem$(RawIndex,8);"  ";_
                     RDitem$(RawIndex,9);"  ";_
                     RDitem$(RawIndex,10);"  ";_
                     RDitem$(RawIndex,11);"  ";_
                     RDitem$(RawIndex,12);"  ";_
                     RDitem$(RawIndex,13);"  ";_
                     RDitem$(RawIndex,14);"  ";_
                     RDitem$(RawIndex,15)
    next
return
'
[Total_Count_RawFile]   'compute the total averaged count and scale for gain and integration settings
    For RawIndex = 5 to RawIndexMax
        CountSum =  val(RDitem$(RawIndex,10)) +_                    'Count 1
                    val(RDitem$(RawIndex,11)) +_                    'Count 2
                    val(RDitem$(RawIndex,12)) +_                    'Count 3
                    val(RDitem$(RawIndex,13))                       'Count 4
        Divider = 4                                                 'find divider to get average of Count
        if val(RDitem$(RawIndex,13)) = 0 then
            Divider = Divider - 1
        end if
        if val(RDitem$(RawIndex,12)) = 0 then
            Divider = Divider - 1
        end if
        if val(RDitem$(RawIndex,11)) = 0 then
            Divider = Divider - 1
        end if
        if val(RDitem$(RawIndex,10)) = 0 then
            notice "ERROR"+chr$(13)+_
                   "Only zero count values for star at line "+str$(RawIndex)+chr$(13)+_
                   "REDUCTION STOPPED"
            wait
        end if
        Integration = val(RDitem$(RawIndex,14))                     'find integration time :1 or 10 seconds
        Scale = val(RDitem$(RawIndex,15))                           'find scale factor: 1, 10 or 100
        CountFinal(RawIndex) = int((CountSum/Divider) * (1000/(Integration * Scale)))
    next
return
'
[Julian_Day_RawFile]    'convert UT time and date to Julian, epcoh J2000
    For RawIndex = 5 to RawIndexMax
                                'A = int(UTyear/100)
        A = int(val(RDitem$(RawIndex,3))/100)
        B = 2 - A + int(A/4)
                                'C = int(365.25 * UTyear)
        C = int(365.25 * val(RDitem$(RawIndex,3)))
                                'D = int(30.6001 *(UTmonth + 1))
        D = int(30.6001 * (val(RDitem$(RawIndex,1)) + 1))
                                'JD = B + C + D - 730550.5 + UTday + (UThours + UTmin/60 + UTsec/3600)/24
        JD(RawIndex) = B + C + D - 730550.5 + val(RDitem$(RawIndex,2)) +_
                       (val(RDitem$(RawIndex,4)) + val(RDitem$(RawIndex,5))/60 + val(RDitem$(RawIndex,6))/3600)/24
                                'Julian century
        JT(RawIndex) = JD(RawIndex)/36525
    next
return
'
[Find_Trans_Stars]
                                        'make list of unique transformation stars contained in the raw file
    TransIndexMax = 0                   'reset transformation array index to 0, this will result in 1 being
                                        'the first array item
    for RawIndex = 5 to RawIndexMax
                                        'go through raw file and pick out only the transformation stars with
                                        'the selected filter B or V
        if (RDitem$(RawIndex,7) = "T") then
            TransFlag = 0
                                        'see if the Trans Star is new to the list
            for TransIndex = 1 to TransIndexMax
                if TransStar$(TransIndex) = RDitem$(RawIndex,8) then
                    TransFlag = 1
                end if
            next
            if TransFlag = 0 then       'add the new Trans Star to the list
                TransIndexMax = TransIndexMax + 1
                TransStar$(TransIndexMax) = RDitem$(RawIndex,8)
            end if
        end if
    next
    for TransIndex = 1 to TransIndexMax
        print #IREX, TransStar$(TransIndex);"   ";_     'unique trans star name
                     TransIndex                         'index for star
    next
return
'
[IREX_RawFile]      'subtract skies from transformation star data
    for RawIndex = 5 to RawIndexMax
                    'go through raw file and pick out only the transformation stars with the selected filter
        if (RDitem$(RawIndex,7) = "T")  AND (RDitem$(RawIndex,9) = Filter$) then
                    'find the past sky count to apply
            SkyPastCount = 0
            PastTime = 0
            for I = RawIndex to 5 step -1
                if (RDitem$(I,8) = "SKY         ") AND (RDitem$(I,9) = Filter$) OR_
                   (RDitem$(I,8) = "SKYNEXT     ") AND (RDitem$(I,9) = Filter$) then
                    SkyPastCount = CountFinal(I)
                    PastTime = JD(I)
                    exit for
                end if
            next
                    'find the future sky count to apply
            SkyFutureCount = 0
            FutureTime = 0
            for I = RawIndex to RawIndexMax step 1
                if (RDitem$(I,8) = "SKY         ") AND (RDitem$(I,9) = Filter$) OR_
                   (RDitem$(I,8) = "SKYLAST     ") AND (RDitem$(I,9) = Filter$) then
                    SkyFutureCount = CountFinal(I)
                    FutureTime = JD(I)
                    exit for
                end if
            next
                    'subtract sky from Trans star count depending on SKY, SKYNEXT or SKYLAST protocol and change
                    'CountFinal value accordingly
            select case
                case (SkyPastCount = 0) AND (SkyFutureCount = 0)
                    notice "no SKY counts for COMP star at line# "; RawIndex
                    wait
                case (SkyPastCount > 0) AND (SkyFutureCount = 0)
                    CountFinal(RawIndex) = CountFinal(RawIndex) - SkyPastCount
                case (SkyPastCount = 0) AND (SkyFutureCount > 0)
                    CountFinal(RawIndex) = CountFinal(RawIndex) - SkyFutureCount
                case else
                        '                           y2 - y1
                        'interpolation:   y = y1 + --------- * (x - x1),  equation 3.26
                        '                           x2 - x1
                    SkyCurrentCount = SkyPastCount + ((SkyFutureCount - SkyPastCount)/(FutureTime - PastTime))*_
                                      (JD(RawIndex) - PastTime)
                    CountFinal(RawIndex) = CountFinal(RawIndex) - SkyCurrentCount
            end select

            print #IREX,    RDitem$(RawIndex,8);"  ";_                              'star name
                            using("#####.#####",JD(RawIndex));"   ";_               'Julean Date
                            using("######", CountFinal(RawIndex));"         ";_     'final average count
                            Filter$                                                 'filter

        end if
    next
return
'
'=====subroutines for creating estinction table
'
[Get_Trans_Star_Data]
    redim RA(50)                        'RA in decimal degrees for each trans star, index var = TransIndex
    redim DEC(50)                       'DEC in decimal derees for each trans star, index var = TransIndex
    redim Trans.J(50)                   'J standard magnitude, index var = TransIndex
    redim TransStdColor(50)             'J - H standard color index, index var = TransIndex

                                                    'get RA and DEC info of comp star from Star Data file
    for TransIndex = 1 to TransIndexMax             'go through trans stars contained in raw file
        for DataIndex = 1 to DataIndexMax           'then go through transformation data file

            if TransStar$(TransIndex) = left$(TDitem$(DataIndex,1)+"           ",12) then
                                                    'get RA and convert to decimal
                RA(TransIndex) =            val(TDitem$(DataIndex,3)) +_
                                            val(TDitem$(DataIndex,4))/60 +_
                                            val(TDitem$(DataIndex,5))/3600
                                                    'convert RA to degrees
                RA(TransIndex) = (RA(TransIndex)/24) * 360
                                                    'get and convert DEC to decimal
                DEC(TransIndex) =           abs(val(right$(TDitem$(DataIndex,6),2))) +_
                                            val(TDitem$(DataIndex,7))/60 +_
                                            val(TDitem$(DataIndex,8))/3600
                                                    'see if there is a minus sign in DECd then make DEC negative
                if left$(TDitem$(DataIndex,6),1) = "-"  then
                    DEC(TransIndex) = DEC(TransIndex) * -1
                end if
                                                    'get J magnitude
                Trans.J(TransIndex) =     val(TDitem$(DataIndex,9))
                                                    'get J-H color index
                TransStdColor(TransIndex) = val(TDitem$(DataIndex,10))
            end if
        next
        if (RA(TransIndex) = 0) AND (DEC(TransIndex) = 0) then
            notice "could not find selected Comp Star in Star Data file"
            wait
        end if
        print #IREX, TransStar$(TransIndex);"   ";_
                     using("###.####",RA(TransIndex));"  ";_
                     using("###.####",DEC(TransIndex));"  ";_
                     using("###.##",Trans.J(TransIndex));"  ";_
                     using("###.##",TransStdColor(TransIndex))
    next
return
'
                                'create ExtData() for display in listbox
[Create_Transformation_Table]
    redim TransData$(50)

    redim FilterFlag(50,5)
    redim X(50,5)                           'air mass for each star reading and for each filter
    redim mo(50,5)                          'instrument mag. for each filter reduced for K', jo, ho
    redim m(50,5)                           'instrument mag. for each filter, j, h

    for TransIndex = 1 to TransIndexMax
        for RawIndex = 5 to RawIndexMax
            if TransStar$(TransIndex) = RDitem$(RawIndex,8) then
                gosub [Siderial_Time]
                gosub [Find_Air_Mass]
                InstrumentMag = -1.0857*log(CountFinal(RawIndex))                   'ln(x) = 2.3026 * log(x)
                select case RDitem$(RawIndex,9)
                    case "J"
                        FilterFlag(TransIndex,1) = 1
                        m(TransIndex,1) = InstrumentMag                             'j
                        mo(TransIndex,1) = InstrumentMag - KJ * AirMass             'jo
                        X(TransIndex,1) = AirMass
                        InstrumentMago = mo(TransIndex,1)
                    case "H"
                        FilterFlag(TransIndex,2) = 1
                        m(TransIndex,2) = InstrumentMag                             'h
                        mo(TransIndex,2) = InstrumentMag - KH * AirMass             'ho
                        X(TransIndex,2) = AirMass
                        InstrumentMago = mo(TransIndex,2)
                end select

                print #IREX, TransStar$(TransIndex);"  ";_                          'star
                             using("###.####",LMST);"  ";_                          'local mean sideral time
                             using("##.####",AirMass);"   ";_                       'air mass, X
                             using("######",CountFinal(RawIndex));"    ";_          'reduced average count
                             using("###.##",InstrumentMag);"    ";_                 'instrument mag, j, h
                             using("###.##",InstrumentMago);"   ";_
                             RDitem$(RawIndex,9)                                    'filter, J, H
            end if
        next
    next

    print #IREX, " "
    print #IREX, "TransIndex  Trans.Jj    Trans.J    TransColor    TransColor2"

    for TransIndex = 1 to TransIndexMax

        Trans.Jj(TransIndex) = Trans.J(TransIndex) - mo(TransIndex,1)                       'J - jo

        If FilterFlag(TransIndex,1) = 1 then
            'compute (j - h)o = (j - h) * (1 - K"jh * X) - K'jh * X, equation 2.7
            AvgAirMass = (X(TransIndex,1) + X(TransIndex,2))/2                              'average X for j and h
            TransColor(TransIndex) = (m(TransIndex,1) - m(TransIndex,2)) *_                 'j - h
                                    (1 - KKjh * AvgAirMass) -_                              '1 - K"jh * X
                                    (KJ - KH) * AvgAirMass                                  'K'jh * X
            TransColor2(TransIndex) = TransStdColor(TransIndex) - TransColor(TransIndex)    '(J-H) - (j-h)o
        else
            TransColor2(TransIndex) = 0
        end if


        TransData$(TransIndex) = TransStar$(TransIndex)+" "+_                               'star name
                                 using("#.##",X(TransIndex,2))+"  "+_                       'air mass for j
                                 using("##.##",TransStdColor(TransIndex))+" "+_             'J-H
                                 using("###.##",Trans.Jj(TransIndex))+"  "+_                'J-jo
                                 using("##.##",TransColor2(TransIndex))                     '(J-H) - (j-h)o

    print #IREX, using("###",TransIndex);"         ";_
                 using("###.###",Trans.Jj(TransIndex));"    ";_
                 using("###.###",Trans.J(TransIndex));"    ";_
                 using("###.###",TransColor(TransIndex));"       ";_
                 using("###.###",TransColor2(TransIndex))

    next
    print #trans.Table, "reload"
return
'
[Create_Regression_Array]
    for TransIndex = 1 to TransIndexMax
        select case Graph$
            case "rho"
                Xaxis(TransIndex) = TransStdColor(TransIndex)
                Yaxis(TransIndex) = Trans.Jj(TransIndex)
            case "lambda"
                Xaxis(TransIndex) = TransStdColor(TransIndex)
                Yaxis(TransIndex) = TransColor2(TransIndex)
        end select
        print #IREX, TransStar$(TransIndex);"  ";_
                     using("####.###",Xaxis(TransIndex));"   ";_
                     using("####.###",Yaxis(TransIndex))
    next
return
'
'====subroutines for file operations
'
[Write_PPparms]
    open "PPparms2_JH.txt" for output as #PPparms
        print #PPparms, Location$                   'latitude and longitude
        print #PPparms, using("##.###",KJ)          'first order extinction for J
        print #PPparms, using("##.###",KH)          'first order extinction for H
        print #PPparms, using("##.###",KKjh)        'second order extinction for j-h, default = 0
        print #PPparms, using("##.###",Rho)         'transformation coeff. J
        print #PPparms, using("##.###",Lambda)      'transformation coeff. J-H
        print #PPparms, JDFlag                      '1 if using JD and 0 if using HJD
        print #PPparms, OBSCODE$                    'AAVSO observatory code
        print #PPparms, Spare2                      '0 not used at this time
        print #PPparms, using("##.###",ZPj)         'zero-point constant for j
        print #PPparms, using("##.###",ZPjh)        'zero-point constant for j-h
        print #PPparms, using("##.###",Ej)          'standard error for j
        print #PPparms, using("##.###",Ejh)         'standard error for j-h
    close #PPparms
return
'
[Find_File_Name]        'seperate out filename and extension from info() path/filename
    FileNameIndex = len(DataFile$)
    FileNameLength = len(DataFile$)
    while mid$(DataFile$, FileNameIndex,1)<>"\"          'look for the last backlash
        FileNameIndex = FileNameIndex - 1
    wend
    FileNamePath$ = left$(DataFile$, FileNameIndex)
    DataFileName$ = right$(DataFile$, FileNameLength-FileNameIndex)

    print #trans.FileName, DataFileName$                 'display filename in "File" textbox
    print #IREX, DataFile$
return
'
'=====subroutines for calculations
'
[Find_Lat_Long]         'compute observers latitude and longitude in degrees
    LAT = val(mid$(Location$,2,4))
    if upper$(left$(Location$,1)) = "S" then
        LAT = LAT * -1
    end if
    LONGITUDE = val(mid$(Location$,8,5))
    if upper$(mid$(Location$,7,1)) = "W" then
        LONGITUDE = LONGITUDE * -1
    end if
return
'
[Siderial_Time]         'compute local siderial time
                        'equation by Stephen R. Schmitt found on the web at
                        'http://home.att.net/~srschmitt/celestial2horizon.html
    MST = 280.46061837 + 360.98564736629 * JD(RawIndex) + 0.000387933 * JT(RawIndex) *_
          JT(RawIndex) - (JT(RawIndex) * JT(RawIndex) * JT(RawIndex))/38710000
    LMST = MST + LONGITUDE
    if LMST > 0 then
        while (LMST > 360)
            LMST = LMST - 360
        wend
    else
        while (LMST < 0)
            LMST = LMST + 360
        wend
    end if
return
'
[Find_Air_Mass]                              'compute air mass
    HA = LMST - RA(TransIndex)               'find hour angle in degrees
    if HA < 0 then
        HA = HA + 360
    end if

    HAradians = HA * 3.1416/180             'convert to radians
    DECradians = DEC(TransIndex) * 3.1416/180
    LATradians = LAT * 3.1416/180

                        'compute secant of zenith angle - distance from zenith in radians
    secZ = 1/(sin(LATradians) * sin(DECradians) + cos(LATradians) * cos(DECradians) * cos(HAradians))
                        'compute air mass using Hardie equation
                        'X = sec Z - 0.0018167(sec Z - 1) - 0.002875(sec Z - 1)^2 - 0.0008083(sec Z - 1)^3
    AirMass = secZ - 0.0018167 * (secZ - 1) - 0.002875 * (secZ - 1)^2 - 0.0008083 * (secZ - 1)^3
return
'
[Solve_Regression_Matrix]
            'linear least squares routine from Nielson
            'pulled from appendix I.4 of Astronomical Photometry, written by Hendon 1973
            'inputs     Xaxis() V-vo to solve for eps, or (B-V)-(b-v)o to solve for mu
            '           Yaxis() transformation star standard color index B-V
            'outputs   Slope and Intercept,   Y = aX + b
            'number of elements in array = TransIndexMax
    a2 = 0
    a3 = 0
    c1 = 0
    c2 = 0
    a1 = TransIndexMax
    for I = 1 to TransIndexMax
        a2 = a2 + Xaxis(I)
        a3 = a3 + Xaxis(I) * Xaxis(I)
        c1 = c1 + Yaxis(I)
        c2 = c2 + Yaxis(I) * Xaxis(I)
    next
    det = 1/(a1 * a3 - a2 * a2)
    Intercept = -1 * (a2 * c2 - c1 * a3) * det
    Slope = (a1 * c2 - c1 * a2) * det

            'compute standard error using eq. 3.21
    if TransIndexMax > 2 then
        y.deviation.squared.sum = 0
        for N = 1 to TransIndexMax
            y.fit = Slope * Xaxis(N) + Intercept
            y.deviation = Yaxis(N) - y.fit
            y.deviation.squared.sum =  y.deviation.squared.sum + y.deviation^2
        next
        std.error = sqr((1/(N-2)) * y.deviation.squared.sum)
    else
        std.error = 0
    end if
return
'
'=====graphics routines
'
[Draw_Graph_Outline]
    print #trans.graphicbox1, "font arial 8 16"
    print #trans.graphicbox1, "font arial 6 12"
    print #trans.graphicbox1, "place 60 235"
    print #trans.graphicbox1, "\ -1.0"
    print #trans.graphicbox1, "place 110 235"
    print #trans.graphicbox1, "\ -0.5"
    print #trans.graphicbox1, "place 160 235"
    print #trans.graphicbox1, "\ 0.0"
    print #trans.graphicbox1, "place 210 235"
    print #trans.graphicbox1, "\ 0.5"
    print #trans.graphicbox1, "place 260 235"
    print #trans.graphicbox1, "\ 1.0"
    print #trans.graphicbox1, "place 310 235"
    print #trans.graphicbox1, "\ 1.5"
    print #trans.graphicbox1, "place 360 235"
    print #trans.graphicbox1, "\ 2.0"
    print #trans.graphicbox1, "place 410 235"
    print #trans.graphicbox1, "\ 2.5"
    print #trans.graphicbox1, "place 460 235"
    print #trans.graphicbox1, "\ 3.0"

    print #trans.graphicbox1, "line 40 20 40 220"
    print #trans.graphicbox1, "line 40 220 480 220"
    print #trans.graphicbox1, "font arial 6 12"

    for I = 1 to 10
        xD =  20 * I
        print #trans.graphicbox1, "line 40 ";xD;" 45 ";xD
    next
    for I = 1 to 42
        yD = 40 + 10 * I
        print #trans.graphicbox1, "line ";yD;" 220 ";yD;" 215"
    next
    for I = 0 to 8
        yD = 70 + 50 * I
        print #trans.graphicbox1, "line ";yD;" 220 ";yD;" 210"
    next
return
'
[Draw_Best_Line]
    ScaleTop = (int(Intercept * 10) + 5)/10
    ScaleBot = ScaleTop - 1

    StartLine = -1.3 * Slope + Intercept                        'start of best fit line at index = -1.3
    EndLine   = 3.0 * Slope + Intercept                         'end of best fit line at index = 3.0
    YaxisTop = ScaleTop * 200                                   'value for top of Y axis
                                                                '
    StartLine = YaxisTop - StartLine * 200 + 20
    EndLine = YaxisTop - EndLine * 200 + 20
    print #trans.graphicbox1, "line 40 ";StartLine;" 470 ";EndLine

                                                                'plot data points
    for TransIndex = 1 to TransIndexMax
        select case Graph$
            case "rho"
                Ypoint = YaxisTop - int(Trans.Jj(TransIndex) * 200) + 20
                Xpoint = int(TransStdColor(TransIndex) * 100) + 170
            case "lambda"
                Ypoint = YaxisTop - int(TransColor2(TransIndex) * 200) + 20
                Xpoint = int(TransStdColor(TransIndex) * 100) + 170
        end select
        print #trans.graphicbox1, "place ";Xpoint;" ";Ypoint
        print #trans.graphicbox1, "circle 3"
    next
'
                                                    'draw y scale values
    print #trans.graphicbox1, "color black"
    print #trans.graphicbox1, "font arial 6 12"
    print #trans.graphicbox1, "place 10 25"
    ScaleTop$ = "\"+using("###.#", ScaleTop)
    print #trans.graphicbox1, ScaleTop$
    print #trans.graphicbox1, "place 10 220"
    ScaleBot$ = "\"+using("###.#", ScaleBot)
    print #trans.graphicbox1, ScaleBot$

return
'
[Draw_Description]
                                                    'print file name
    print #trans.graphicbox1, "place 60 300"
    print #trans.graphicbox1, "\FILE NAME :"
    print #trans.graphicbox1, "place 160 300"
    GraphicFileName$ = "\"+ShortDataFile$
    print #trans.graphicbox1, GraphicFileName$ 
                                                    'print start Julian Date
    print #trans.graphicbox1, "place 60 325"
    print #trans.graphicbox1, "\START J2000:"
    print #trans.graphicbox1, "place 160 325"
    GraphicStartJD$ = "\"+Using("####.####", JD(5))
    print #trans.graphicbox1, GraphicStartJD$ 
                                                    'print end Julian Date
    print #trans.graphicbox1, "place 60 350"
    print #trans.graphicbox1, "\END J2000:"
    print #trans.graphicbox1, "place 160 350"
    GraphicEndJD$ = "\"+Using("####.####", JD(RawIndexMax))
    print #trans.graphicbox1, GraphicEndJD$ 
                                                    'print slope
    print #trans.graphicbox1, "place 60 375"
    print #trans.graphicbox1, "\SLOPE:"
    print #trans.graphicbox1, "place 160 375"
    GraphicSlope$ = "\"+Using("####.###", Slope)
    print #trans.graphicbox1, GraphicSlope$ 
                                                    'printer intercept
    print #trans.graphicbox1, "place 60 400"
    print #trans.graphicbox1, "\INTERCEPT:"
    print #trans.graphicbox1, "place 160 400"
    GraphicIntercept$ = "\"+Using("####.###", Intercept)
    print #trans.graphicbox1, GraphicIntercept$ 
                                                    'print standard error
    print #trans.graphicbox1, "place 60 425"
    print #trans.graphicbox1, "\STD. ERROR:"
    print #trans.graphicbox1, "place 160 425"
    GraphicStd.error$ = "\"+Using("####.###", std.error)
    print #trans.graphicbox1, GraphicStd.error$ 
                                                    'print stars
    print #trans.graphicbox1, "place 60 450"
    print #trans.graphicbox1, "\STARS :"
    For TransIndex = 1 to TransIndexMax
        GraphicStar$ = "\"+TransStar$(TransIndex)
        place$ = "place 160 "+str$(435 + (15*TransIndex))
        print #trans.graphicbox1, place$
        print #trans.graphicbox1, GraphicStar$
    Next

return
'
end
